







/* This file was automatically generated with m4 from m4/Twalias.m4, do not edit! */

/*
 *  alias_m4.h  --  handy aliases for libTw function prototypes
 *
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */







 
     
    
#define Tw_DeleteRow			Tw_DeleteObj                
#define Tw_CirculateRows			Tw_CirculateChildrenRow        
#define Tw_RestackRows			Tw_RestackChildrenRow    
#define Tw_Row4Menu			Tw_Create4MenuRow 
    
#define Tw_DeleteMenuItem			Tw_DeleteObj    
#define Tw_LowerMenuItem			Tw_LowerRow    
#define Tw_RaiseMenuItem			Tw_RaiseRow    
#define Tw_CirculateChildrenMenuItem			Tw_CirculateChildrenRow    
#define Tw_CirculateMenuItems			Tw_CirculateChildrenMenuItem    
#define Tw_RestackChildrenMenuItem			Tw_RestackChildrenRow    
#define Tw_RestackMenuItems			Tw_RestackChildrenMenuItem    
#define Tw_MenuItem4Menu			Tw_Create4MenuMenuItem    
#define Tw_Item4Menu			Tw_Create4MenuMenuItem    
#define Tw_Item4MenuCommon			Tw_Create4MenuCommonMenuItem 
    
#define Tw_DeleteMenu			Tw_DeleteObj    
#define Tw_Info4Menu			Tw_SetInfoMenu 
    
#define Tw_DeleteMsgPort			Tw_DeleteObj 
    
#define Tw_DeleteWidget			Tw_DeleteObj        
#define Tw_CirculateWidgets			Tw_CirculateChildrenWidget                                                            
#define Tw_RestackWidgets			Tw_RestackChildrenWidget             
    
#define Tw_DeleteGadget			Tw_DeleteObj    
#define Tw_CirculateChildrenGadget			Tw_CirculateChildrenWidget    
#define Tw_CirculateGadgets			Tw_CirculateChildrenGadget    
#define Tw_DrawGadget			Tw_DrawWidget    
#define Tw_Draw2Gadget			Tw_Draw2Widget    
#define Tw_DrawTextGadget			Tw_DrawTextWidget    
#define Tw_DrawHWFontGadget			Tw_DrawHWFontWidget    
#define Tw_DrawHWAttrGadget			Tw_DrawHWAttrWidget    
#define Tw_OwnerGadget			Tw_OwnerWidget    
#define Tw_O_PrevGadget			Tw_O_PrevWidget    
#define Tw_O_NextGadget			Tw_O_NextWidget    
#define Tw_LowerGadget			Tw_LowerWidget    
#define Tw_RaiseGadget			Tw_RaiseWidget    
#define Tw_MapGadget			Tw_MapWidget    
#define Tw_RecursiveDeleteGadget			Tw_RecursiveDeleteWidget    
#define Tw_ResizeGadget			Tw_ResizeWidget    
#define Tw_RestackChildrenGadget			Tw_RestackChildrenWidget    
#define Tw_RestackGadgets			Tw_RestackChildrenGadget    
#define Tw_ScrollGadget			Tw_ScrollWidget    
#define Tw_SetXYGadget			Tw_SetXYWidget    
#define Tw_UnMapGadget			Tw_UnMapWidget             
    
#define Tw_DeleteWindow			Tw_DeleteObj    
#define Tw_CirculateChildrenWindow			Tw_CirculateChildrenWidget    
#define Tw_CirculateWindows			Tw_CirculateChildrenWindow    
#define Tw_DrawWindow			Tw_DrawWidget    
#define Tw_Draw2Window			Tw_Draw2Widget    
#define Tw_DrawTextWindow			Tw_DrawTextWidget    
#define Tw_DrawHWFontWindow			Tw_DrawHWFontWidget    
#define Tw_DrawHWAttrWindow			Tw_DrawHWAttrWidget    
#define Tw_OwnerWindow			Tw_OwnerWidget    
#define Tw_O_PrevWindow			Tw_O_PrevWidget    
#define Tw_O_NextWindow			Tw_O_NextWidget    
#define Tw_LowerWindow			Tw_LowerWidget    
#define Tw_RaiseWindow			Tw_RaiseWidget    
#define Tw_MapWindow			Tw_MapWidget    
#define Tw_RecursiveDeleteWindow			Tw_RecursiveDeleteWidget    
#define Tw_ResizeWindow			Tw_ResizeWidget    
#define Tw_RestackChildrenWindow			Tw_RestackChildrenWidget    
#define Tw_RestackWindows			Tw_RestackChildrenWindow    
#define Tw_ScrollWindow			Tw_ScrollWidget    
#define Tw_SetXYWindow			Tw_SetXYWidget    
#define Tw_UnMapWindow			Tw_UnMapWidget    
#define Tw_Win4Menu			Tw_Create4MenuWindow 
    
#define Tw_DeleteScreen			Tw_DeleteObj    
#define Tw_CirculateChildrenScreen			Tw_CirculateChildrenWidget    
#define Tw_CirculateScreens			Tw_CirculateChildrenScreen    
#define Tw_DrawScreen			Tw_DrawWidget    
#define Tw_Draw2Screen			Tw_Draw2Widget    
#define Tw_DrawTextScreen			Tw_DrawTextWidget    
#define Tw_DrawHWFontScreen			Tw_DrawHWFontWidget    
#define Tw_DrawHWAttrScreen			Tw_DrawHWAttrWidget    
#define Tw_OwnerScreen			Tw_OwnerWidget    
#define Tw_O_PrevScreen			Tw_O_PrevWidget    
#define Tw_O_NextScreen			Tw_O_NextWidget    
#define Tw_LowerScreen			Tw_LowerWidget    
#define Tw_RaiseScreen			Tw_RaiseWidget    
#define Tw_MapScreen			Tw_MapWidget    
#define Tw_RecursiveDeleteScreen			Tw_RecursiveDeleteWidget    
#define Tw_ResizeScreen			Tw_ResizeWidget    
#define Tw_RestackChildrenScreen			Tw_RestackChildrenWidget    
#define Tw_RestackScreens			Tw_RestackChildrenScreen    
#define Tw_ScrollScreen			Tw_ScrollWidget    
#define Tw_SetXYScreen			Tw_SetXYWidget    
#define Tw_UnMapScreen			Tw_UnMapWidget

