



/* This file was automatically generated from m4/socklist.m4, do not edit!
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */








#define OK_MAGIC	((uldat)0x3E4B4F3Cul)
#define FAIL_MAGIC	((uldat)0xFFFFFFFFul)

#define FIND_MAGIC		((uldat)0x646E6946ul) /* i.e. "Find" */
#define MSG_MAGIC		((uldat)0x2167734dul) /* i.e. "Msg!" */












EL(FindFunction)


EL(SyncSocket)


EL(ServerSizeof)


EL(CanCompress)

EL(DoCompress)


EL(NeedResizeDisplay)


EL(AttachHW)

EL(DetachHW)


EL(SetFontTranslation)

EL(SetHWFontTranslation)


EL(DeleteObj)

EL(ChangeFieldObj)


EL(CreateWidget)

EL(RecursiveDeleteWidget)
EL(MapWidget)
EL(UnMapWidget)
EL(SetXYWidget)
EL(ResizeWidget)
EL(ScrollWidget)
EL(DrawWidget)

EL(SetVisibleWidget)

EL(FocusSubWidget)
EL(FindWidgetAtWidget)

EL(RaiseWidget)
EL(LowerWidget)
EL(RestackChildrenWidget)
EL(CirculateChildrenWidget)


EL(CreateGadget)

EL(CreateButtonGadget)

EL(WriteTextsGadget)
EL(WriteHWFontsGadget)

EL(CreateWindow)
EL(Create4MenuWindow)

EL(WriteAsciiWindow)
EL(WriteStringWindow)
EL(WriteHWFontWindow)
EL(WriteHWAttrWindow)

EL(GotoXYWindow)
EL(SetTitleWindow)
EL(SetColTextWindow)
EL(SetColorsWindow)
EL(ConfigureWindow)
EL(FindRowByCodeWindow)

EL(CreateGroup)
EL(InsertGadgetGroup)
EL(RemoveGadgetGroup)

EL(GetSelectedGadgetGroup)
EL(SetSelectedGadgetGroup)

EL(RaiseRow)
EL(LowerRow)
EL(RestackChildrenRow) 
EL(CirculateChildrenRow)

EL(Create4MenuAny)

EL(Create4MenuCommonMenuItem)

EL(CreateMenu)
EL(SetInfoMenu)

EL(CreateMsgPort)
EL(FindMsgPort)

EL(BgImageScreen)

EL(PrevObj)
EL(NextObj)
EL(ParentObj)

EL(FirstScreen)
EL(FirstWidget)
EL(FirstMsgPort)
EL(FirstMenu)
EL(FirstW)
EL(FirstGroup)
EL(FirstMutex)
EL(FirstMenuItem)
EL(FirstGadget)

EL(GetDisplayWidth)
EL(GetDisplayHeight)
EL(GetAll)

EL(SendToMsgPort)
EL(BlindSendToMsgPort)

EL(GetOwnerSelection)
EL(SetOwnerSelection)
EL(RequestSelection)
EL(NotifySelection)

EL(SetServerUid)

EL(OpenExtension)
EL(CallBExtension)
EL(CloseExtension)



