/*
 *  Twutf_defs.h  --  include for ASCII/VGA/UTF/UNICODE characters database
 *
 *  Copyright (C) 2001 by Massimiliano Ghilardi
 *
 *  This library is free software you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation either
 *  version 2 of the License, or (at your option) any later version.
 *
 */

#ifndef _TUTF_DEFS_H
#define _TUTF_DEFS_H

#include <Tutf/utf_16.h>
#include <Tutf/ascii.h>

#include <Tutf/iso_8859_1.h>
#include <Tutf/iso_8859_2.h>
#include <Tutf/iso_8859_3.h>
#include <Tutf/iso_8859_4.h>
#include <Tutf/iso_8859_5.h>
#include <Tutf/iso_8859_6.h>
#include <Tutf/iso_8859_7.h>
#include <Tutf/iso_8859_8.h>
#include <Tutf/iso_8859_9.h>
#include <Tutf/iso_8859_10.h>
#include <Tutf/iso_8859_13.h>
#include <Tutf/iso_8859_14.h>
#include <Tutf/iso_8859_15.h>
#include <Tutf/iso_8859_16.h>

#include <Tutf/cp437.h>
#include <Tutf/cp737.h>
#include <Tutf/cp775.h>
#include <Tutf/cp850.h>
#include <Tutf/cp852.h>
#include <Tutf/cp855.h>
#include <Tutf/cp857.h>
#include <Tutf/cp860.h>
#include <Tutf/cp861.h>
#include <Tutf/cp862.h>
#include <Tutf/cp863.h>
#include <Tutf/cp864.h>
#include <Tutf/cp865.h>
#include <Tutf/cp866.h>
#include <Tutf/cp869.h>
#include <Tutf/cp874.h>
    
#include <Tutf/cp1250.h>
#include <Tutf/cp1251.h>
#include <Tutf/cp1252.h>
#include <Tutf/cp1253.h>
#include <Tutf/cp1254.h>
#include <Tutf/cp1255.h>
#include <Tutf/cp1256.h>
#include <Tutf/cp1257.h>

#endif /* _TUTF_DEFS_H */


