




/* This file was automatically generated from m4/create_init.m4, do not edit! */

/*
 *
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 */

/*
 *  create_init_m4.h  --  initialization of libTT internal Create* function pointers
 *
 */

/*  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */

























                                                
    Create1_ttvector,
    CreateA_ttvector,
    CreateL_ttvector,
    CreateR_ttvector,
    CreateV_ttvector,
    CreateY_ttvector,               
    Create_ttevent, 
    Create8_tteventbig,
    Create4_tteventbig,
    Create4s4_tteventbig, 
    Create_ttbitmask,
    CreateB2_ttbitmask,
    CreateR_ttbitmask,
    CreateR2_ttbitmask, 
    CreateB_tteventmask,
    CreateB2_tteventmask,      
    Create_ttlistener,
    CreateE_ttlistener,
    CreateR_ttlistener,
    CreateDel_ttlistener,
    CreateAskclose_ttlistener,
    CreateKey_ttlistener,
    CreateMouse_ttlistener,
    CreateActivate_ttlistener,
    CreateChange_ttlistener,
    CreateKeyData_ttlistener,
    CreateExpose_ttlistener,         
    CreateR_tttimer,
    Create_tttimer,                                                   
          
    Create_ttboxlayout,              
    Create_ttnative,                                      
    Create_ttlabel,                           
    Create_ttanybutton,     
    Create_ttbutton,     
    Create_ttcheckbutton, 
    Create_ttradiobutton,                              
    Create1_ttscrollbar,
    Create4_ttscrollbar,             
    Create_ttslider, 
    Create_ttprogressbar, 
    Create_ttscrollpane,                  
    Create_ttframe, 
    Create_ttscroller,          
    Create_ttapplication,




