
/*
 * This file was automatically generated
 * from clisp sources with "from-kernel.bash"
 * Do not edit!
 */


#ifndef _TUTF_MAC_DINGBAT_H
#define _TUTF_MAC_DINGBAT_H

#define T_MAC_DINGBAT_NULL	0x0000
#define T_MAC_DINGBAT_START_OF_HEADING	0x0001
#define T_MAC_DINGBAT_START_OF_TEXT	0x0002
#define T_MAC_DINGBAT_END_OF_TEXT	0x0003
#define T_MAC_DINGBAT_END_OF_TRANSMISSION	0x0004
#define T_MAC_DINGBAT_ENQUIRY	0x0005
#define T_MAC_DINGBAT_ACKNOWLEDGE	0x0006
#define T_MAC_DINGBAT_BELL	0x0007
#define T_MAC_DINGBAT_BACKSPACE	0x0008
#define T_MAC_DINGBAT_HORIZONTAL_TABULATION	0x0009
#define T_MAC_DINGBAT_LINE_FEED	0x000A
#define T_MAC_DINGBAT_VERTICAL_TABULATION	0x000B
#define T_MAC_DINGBAT_FORM_FEED	0x000C
#define T_MAC_DINGBAT_CARRIAGE_RETURN	0x000D
#define T_MAC_DINGBAT_SHIFT_OUT	0x000E
#define T_MAC_DINGBAT_SHIFT_IN	0x000F
#define T_MAC_DINGBAT_DATA_LINK_ESCAPE	0x0010
#define T_MAC_DINGBAT_DEVICE_CONTROL_ONE	0x0011
#define T_MAC_DINGBAT_DEVICE_CONTROL_TWO	0x0012
#define T_MAC_DINGBAT_DEVICE_CONTROL_THREE	0x0013
#define T_MAC_DINGBAT_DEVICE_CONTROL_FOUR	0x0014
#define T_MAC_DINGBAT_NEGATIVE_ACKNOWLEDGE	0x0015
#define T_MAC_DINGBAT_SYNCHRONOUS_IDLE	0x0016
#define T_MAC_DINGBAT_END_OF_TRANSMISSION_BLOCK	0x0017
#define T_MAC_DINGBAT_CANCEL	0x0018
#define T_MAC_DINGBAT_END_OF_MEDIUM	0x0019
#define T_MAC_DINGBAT_SUBSTITUTE	0x001A
#define T_MAC_DINGBAT_ESCAPE	0x001B
#define T_MAC_DINGBAT_FILE_SEPARATOR	0x001C
#define T_MAC_DINGBAT_GROUP_SEPARATOR	0x001D
#define T_MAC_DINGBAT_RECORD_SEPARATOR	0x001E
#define T_MAC_DINGBAT_UNIT_SEPARATOR	0x001F
#define T_MAC_DINGBAT_SPACE	0x0020
#define T_MAC_DINGBAT_UPPER_BLADE_SCISSORS	0x0021
#define T_MAC_DINGBAT_BLACK_SCISSORS	0x0022
#define T_MAC_DINGBAT_LOWER_BLADE_SCISSORS	0x0023
#define T_MAC_DINGBAT_WHITE_SCISSORS	0x0024
#define T_MAC_DINGBAT_BLACK_TELEPHONE	0x0025
#define T_MAC_DINGBAT_TELEPHONE_LOCATION_SIGN	0x0026
#define T_MAC_DINGBAT_TAPE_DRIVE	0x0027
#define T_MAC_DINGBAT_AIRPLANE	0x0028
#define T_MAC_DINGBAT_ENVELOPE	0x0029
#define T_MAC_DINGBAT_BLACK_RIGHT_POINTING_INDEX	0x002A
#define T_MAC_DINGBAT_WHITE_RIGHT_POINTING_INDEX	0x002B
#define T_MAC_DINGBAT_VICTORY_HAND	0x002C
#define T_MAC_DINGBAT_WRITING_HAND	0x002D
#define T_MAC_DINGBAT_LOWER_RIGHT_PENCIL	0x002E
#define T_MAC_DINGBAT_PENCIL	0x002F
#define T_MAC_DINGBAT_UPPER_RIGHT_PENCIL	0x0030
#define T_MAC_DINGBAT_WHITE_NIB	0x0031
#define T_MAC_DINGBAT_BLACK_NIB	0x0032
#define T_MAC_DINGBAT_CHECK_MARK	0x0033
#define T_MAC_DINGBAT_HEAVY_CHECK_MARK	0x0034
#define T_MAC_DINGBAT_MULTIPLICATION_X	0x0035
#define T_MAC_DINGBAT_HEAVY_MULTIPLICATION_X	0x0036
#define T_MAC_DINGBAT_BALLOT_X	0x0037
#define T_MAC_DINGBAT_HEAVY_BALLOT_X	0x0038
#define T_MAC_DINGBAT_OUTLINED_GREEK_CROSS	0x0039
#define T_MAC_DINGBAT_HEAVY_GREEK_CROSS	0x003A
#define T_MAC_DINGBAT_OPEN_CENTRE_CROSS	0x003B
#define T_MAC_DINGBAT_HEAVY_OPEN_CENTRE_CROSS	0x003C
#define T_MAC_DINGBAT_LATIN_CROSS	0x003D
#define T_MAC_DINGBAT_SHADOWED_WHITE_LATIN_CROSS	0x003E
#define T_MAC_DINGBAT_OUTLINED_LATIN_CROSS	0x003F
#define T_MAC_DINGBAT_MALTESE_CROSS	0x0040
#define T_MAC_DINGBAT_STAR_OF_DAVID	0x0041
#define T_MAC_DINGBAT_FOUR_TEARDROP_SPOKED_ASTERISK	0x0042
#define T_MAC_DINGBAT_FOUR_BALLOON_SPOKED_ASTERISK	0x0043
#define T_MAC_DINGBAT_HEAVY_FOUR_BALLOON_SPOKED_ASTERISK	0x0044
#define T_MAC_DINGBAT_FOUR_CLUB_SPOKED_ASTERISK	0x0045
#define T_MAC_DINGBAT_BLACK_FOUR_POINTED_STAR	0x0046
#define T_MAC_DINGBAT_WHITE_FOUR_POINTED_STAR	0x0047
#define T_MAC_DINGBAT_BLACK_STAR	0x0048
#define T_MAC_DINGBAT_STRESS_OUTLINED_WHITE_STAR	0x0049
#define T_MAC_DINGBAT_CIRCLED_WHITE_STAR	0x004A
#define T_MAC_DINGBAT_OPEN_CENTRE_BLACK_STAR	0x004B
#define T_MAC_DINGBAT_BLACK_CENTRE_WHITE_STAR	0x004C
#define T_MAC_DINGBAT_OUTLINED_BLACK_STAR	0x004D
#define T_MAC_DINGBAT_HEAVY_OUTLINED_BLACK_STAR	0x004E
#define T_MAC_DINGBAT_PINWHEEL_STAR	0x004F
#define T_MAC_DINGBAT_SHADOWED_WHITE_STAR	0x0050
#define T_MAC_DINGBAT_HEAVY_ASTERISK	0x0051
#define T_MAC_DINGBAT_OPEN_CENTRE_ASTERISK	0x0052
#define T_MAC_DINGBAT_EIGHT_SPOKED_ASTERISK	0x0053
#define T_MAC_DINGBAT_EIGHT_POINTED_BLACK_STAR	0x0054
#define T_MAC_DINGBAT_EIGHT_POINTED_PINWHEEL_STAR	0x0055
#define T_MAC_DINGBAT_SIX_POINTED_BLACK_STAR	0x0056
#define T_MAC_DINGBAT_EIGHT_POINTED_RECTILINEAR_BLACK_STAR	0x0057
#define T_MAC_DINGBAT_HEAVY_EIGHT_POINTED_RECTILINEAR_BLACK_STAR	0x0058
#define T_MAC_DINGBAT_TWELVE_POINTED_BLACK_STAR	0x0059
#define T_MAC_DINGBAT_SIXTEEN_POINTED_ASTERISK	0x005A
#define T_MAC_DINGBAT_TEARDROP_SPOKED_ASTERISK	0x005B
#define T_MAC_DINGBAT_OPEN_CENTRE_TEARDROP_SPOKED_ASTERISK	0x005C
#define T_MAC_DINGBAT_HEAVY_TEARDROP_SPOKED_ASTERISK	0x005D
#define T_MAC_DINGBAT_SIX_PETALLED_BLACK_AND_WHITE_FLORETTE	0x005E
#define T_MAC_DINGBAT_BLACK_FLORETTE	0x005F
#define T_MAC_DINGBAT_WHITE_FLORETTE	0x0060
#define T_MAC_DINGBAT_EIGHT_PETALLED_OUTLINED_BLACK_FLORETTE	0x0061
#define T_MAC_DINGBAT_CIRCLED_OPEN_CENTRE_EIGHT_POINTED_STAR	0x0062
#define T_MAC_DINGBAT_HEAVY_TEARDROP_SPOKED_PINWHEEL_ASTERISK	0x0063
#define T_MAC_DINGBAT_SNOWFLAKE	0x0064
#define T_MAC_DINGBAT_TIGHT_TRIFOLIATE_SNOWFLAKE	0x0065
#define T_MAC_DINGBAT_HEAVY_CHEVRON_SNOWFLAKE	0x0066
#define T_MAC_DINGBAT_SPARKLE	0x0067
#define T_MAC_DINGBAT_HEAVY_SPARKLE	0x0068
#define T_MAC_DINGBAT_BALLOON_SPOKED_ASTERISK	0x0069
#define T_MAC_DINGBAT_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK	0x006A
#define T_MAC_DINGBAT_HEAVY_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK	0x006B
#define T_MAC_DINGBAT_BLACK_CIRCLE	0x006C
#define T_MAC_DINGBAT_SHADOWED_WHITE_CIRCLE	0x006D
#define T_MAC_DINGBAT_BLACK_SQUARE	0x006E
#define T_MAC_DINGBAT_LOWER_RIGHT_DROP_SHADOWED_WHITE_SQUARE	0x006F
#define T_MAC_DINGBAT_UPPER_RIGHT_DROP_SHADOWED_WHITE_SQUARE	0x0070
#define T_MAC_DINGBAT_LOWER_RIGHT_SHADOWED_WHITE_SQUARE	0x0071
#define T_MAC_DINGBAT_UPPER_RIGHT_SHADOWED_WHITE_SQUARE	0x0072
#define T_MAC_DINGBAT_BLACK_UP_POINTING_TRIANGLE	0x0073
#define T_MAC_DINGBAT_BLACK_DOWN_POINTING_TRIANGLE	0x0074
#define T_MAC_DINGBAT_BLACK_DIAMOND	0x0075
#define T_MAC_DINGBAT_BLACK_DIAMOND_MINUS_WHITE_X	0x0076
#define T_MAC_DINGBAT_RIGHT_HALF_BLACK_CIRCLE	0x0077
#define T_MAC_DINGBAT_LIGHT_VERTICAL_BAR	0x0078
#define T_MAC_DINGBAT_MEDIUM_VERTICAL_BAR	0x0079
#define T_MAC_DINGBAT_HEAVY_VERTICAL_BAR	0x007A
#define T_MAC_DINGBAT_HEAVY_SINGLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT	0x007B
#define T_MAC_DINGBAT_HEAVY_SINGLE_COMMA_QUOTATION_MARK_ORNAMENT	0x007C
#define T_MAC_DINGBAT_HEAVY_DOUBLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT	0x007D
#define T_MAC_DINGBAT_HEAVY_DOUBLE_COMMA_QUOTATION_MARK_ORNAMENT	0x007E
#define T_MAC_DINGBAT_UNDEFINED_007F	0x007F
#define T_MAC_DINGBAT_UNDEFINED_0080	0x0080
#define T_MAC_DINGBAT_UNDEFINED_0081	0x0081
#define T_MAC_DINGBAT_UNDEFINED_0082	0x0082
#define T_MAC_DINGBAT_UNDEFINED_0083	0x0083
#define T_MAC_DINGBAT_UNDEFINED_0084	0x0084
#define T_MAC_DINGBAT_UNDEFINED_0085	0x0085
#define T_MAC_DINGBAT_UNDEFINED_0086	0x0086
#define T_MAC_DINGBAT_UNDEFINED_0087	0x0087
#define T_MAC_DINGBAT_UNDEFINED_0088	0x0088
#define T_MAC_DINGBAT_UNDEFINED_0089	0x0089
#define T_MAC_DINGBAT_UNDEFINED_008A	0x008A
#define T_MAC_DINGBAT_UNDEFINED_008B	0x008B
#define T_MAC_DINGBAT_UNDEFINED_008C	0x008C
#define T_MAC_DINGBAT_UNDEFINED_008D	0x008D
#define T_MAC_DINGBAT_UNDEFINED_008E	0x008E
#define T_MAC_DINGBAT_UNDEFINED_008F	0x008F
#define T_MAC_DINGBAT_UNDEFINED_0090	0x0090
#define T_MAC_DINGBAT_UNDEFINED_0091	0x0091
#define T_MAC_DINGBAT_UNDEFINED_0092	0x0092
#define T_MAC_DINGBAT_UNDEFINED_0093	0x0093
#define T_MAC_DINGBAT_UNDEFINED_0094	0x0094
#define T_MAC_DINGBAT_UNDEFINED_0095	0x0095
#define T_MAC_DINGBAT_UNDEFINED_0096	0x0096
#define T_MAC_DINGBAT_UNDEFINED_0097	0x0097
#define T_MAC_DINGBAT_UNDEFINED_0098	0x0098
#define T_MAC_DINGBAT_UNDEFINED_0099	0x0099
#define T_MAC_DINGBAT_UNDEFINED_009A	0x009A
#define T_MAC_DINGBAT_UNDEFINED_009B	0x009B
#define T_MAC_DINGBAT_UNDEFINED_009C	0x009C
#define T_MAC_DINGBAT_UNDEFINED_009D	0x009D
#define T_MAC_DINGBAT_UNDEFINED_009E	0x009E
#define T_MAC_DINGBAT_UNDEFINED_009F	0x009F
#define T_MAC_DINGBAT_UNDEFINED_00A0	0x00A0
#define T_MAC_DINGBAT_CURVED_STEM_PARAGRAPH_SIGN_ORNAMENT	0x00A1
#define T_MAC_DINGBAT_HEAVY_EXCLAMATION_MARK_ORNAMENT	0x00A2
#define T_MAC_DINGBAT_HEAVY_HEART_EXCLAMATION_MARK_ORNAMENT	0x00A3
#define T_MAC_DINGBAT_HEAVY_BLACK_HEART	0x00A4
#define T_MAC_DINGBAT_ROTATED_HEAVY_BLACK_HEART_BULLET	0x00A5
#define T_MAC_DINGBAT_FLORAL_HEART	0x00A6
#define T_MAC_DINGBAT_ROTATED_FLORAL_HEART_BULLET	0x00A7
#define T_MAC_DINGBAT_BLACK_CLUB_SUIT	0x00A8
#define T_MAC_DINGBAT_BLACK_DIAMOND_SUIT	0x00A9
#define T_MAC_DINGBAT_BLACK_HEART_SUIT	0x00AA
#define T_MAC_DINGBAT_BLACK_SPADE_SUIT	0x00AB
#define T_MAC_DINGBAT_CIRCLED_DIGIT_ONE	0x00AC
#define T_MAC_DINGBAT_CIRCLED_DIGIT_TWO	0x00AD
#define T_MAC_DINGBAT_CIRCLED_DIGIT_THREE	0x00AE
#define T_MAC_DINGBAT_CIRCLED_DIGIT_FOUR	0x00AF
#define T_MAC_DINGBAT_CIRCLED_DIGIT_FIVE	0x00B0
#define T_MAC_DINGBAT_CIRCLED_DIGIT_SIX	0x00B1
#define T_MAC_DINGBAT_CIRCLED_DIGIT_SEVEN	0x00B2
#define T_MAC_DINGBAT_CIRCLED_DIGIT_EIGHT	0x00B3
#define T_MAC_DINGBAT_CIRCLED_DIGIT_NINE	0x00B4
#define T_MAC_DINGBAT_CIRCLED_NUMBER_TEN	0x00B5
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_ONE	0x00B6
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_TWO	0x00B7
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_THREE	0x00B8
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FOUR	0x00B9
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FIVE	0x00BA
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SIX	0x00BB
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SEVEN	0x00BC
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_EIGHT	0x00BD
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_DIGIT_NINE	0x00BE
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_NUMBER_TEN	0x00BF
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_ONE	0x00C0
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_TWO	0x00C1
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_THREE	0x00C2
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FOUR	0x00C3
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FIVE	0x00C4
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SIX	0x00C5
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SEVEN	0x00C6
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_EIGHT	0x00C7
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_NINE	0x00C8
#define T_MAC_DINGBAT_DINGBAT_CIRCLED_SANS_SERIF_NUMBER_TEN	0x00C9
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_ONE	0x00CA
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_TWO	0x00CB
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_THREE	0x00CC
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FOUR	0x00CD
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FIVE	0x00CE
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SIX	0x00CF
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SEVEN	0x00D0
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_EIGHT	0x00D1
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_NINE	0x00D2
#define T_MAC_DINGBAT_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_NUMBER_TEN	0x00D3
#define T_MAC_DINGBAT_HEAVY_WIDE_HEADED_RIGHTWARDS_ARROW	0x00D4
#define T_MAC_DINGBAT_RIGHTWARDS_ARROW	0x00D5
#define T_MAC_DINGBAT_LEFT_RIGHT_ARROW	0x00D6
#define T_MAC_DINGBAT_UP_DOWN_ARROW	0x00D7
#define T_MAC_DINGBAT_HEAVY_SOUTH_EAST_ARROW	0x00D8
#define T_MAC_DINGBAT_HEAVY_RIGHTWARDS_ARROW	0x00D9
#define T_MAC_DINGBAT_HEAVY_NORTH_EAST_ARROW	0x00DA
#define T_MAC_DINGBAT_DRAFTING_POINT_RIGHTWARDS_ARROW	0x00DB
#define T_MAC_DINGBAT_HEAVY_ROUND_TIPPED_RIGHTWARDS_ARROW	0x00DC
#define T_MAC_DINGBAT_TRIANGLE_HEADED_RIGHTWARDS_ARROW	0x00DD
#define T_MAC_DINGBAT_HEAVY_TRIANGLE_HEADED_RIGHTWARDS_ARROW	0x00DE
#define T_MAC_DINGBAT_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW	0x00DF
#define T_MAC_DINGBAT_HEAVY_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW	0x00E0
#define T_MAC_DINGBAT_BLACK_RIGHTWARDS_ARROW	0x00E1
#define T_MAC_DINGBAT_THREE_D_TOP_LIGHTED_RIGHTWARDS_ARROWHEAD	0x00E2
#define T_MAC_DINGBAT_THREE_D_BOTTOM_LIGHTED_RIGHTWARDS_ARROWHEAD	0x00E3
#define T_MAC_DINGBAT_BLACK_RIGHTWARDS_ARROWHEAD	0x00E4
#define T_MAC_DINGBAT_HEAVY_BLACK_CURVED_DOWNWARDS_AND_RIGHTWARDS_ARROW	0x00E5
#define T_MAC_DINGBAT_HEAVY_BLACK_CURVED_UPWARDS_AND_RIGHTWARDS_ARROW	0x00E6
#define T_MAC_DINGBAT_SQUAT_BLACK_RIGHTWARDS_ARROW	0x00E7
#define T_MAC_DINGBAT_HEAVY_CONCAVE_POINTED_BLACK_RIGHTWARDS_ARROW	0x00E8
#define T_MAC_DINGBAT_RIGHT_SHADED_WHITE_RIGHTWARDS_ARROW	0x00E9
#define T_MAC_DINGBAT_LEFT_SHADED_WHITE_RIGHTWARDS_ARROW	0x00EA
#define T_MAC_DINGBAT_BACK_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00EB
#define T_MAC_DINGBAT_FRONT_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00EC
#define T_MAC_DINGBAT_HEAVY_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00ED
#define T_MAC_DINGBAT_HEAVY_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00EE
#define T_MAC_DINGBAT_NOTCHED_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00EF
#define T_MAC_DINGBAT_UNDEFINED_00F0	0x00F0
#define T_MAC_DINGBAT_NOTCHED_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW	0x00F1
#define T_MAC_DINGBAT_CIRCLED_HEAVY_WHITE_RIGHTWARDS_ARROW	0x00F2
#define T_MAC_DINGBAT_WHITE_FEATHERED_RIGHTWARDS_ARROW	0x00F3
#define T_MAC_DINGBAT_BLACK_FEATHERED_SOUTH_EAST_ARROW	0x00F4
#define T_MAC_DINGBAT_BLACK_FEATHERED_RIGHTWARDS_ARROW	0x00F5
#define T_MAC_DINGBAT_BLACK_FEATHERED_NORTH_EAST_ARROW	0x00F6
#define T_MAC_DINGBAT_HEAVY_BLACK_FEATHERED_SOUTH_EAST_ARROW	0x00F7
#define T_MAC_DINGBAT_HEAVY_BLACK_FEATHERED_RIGHTWARDS_ARROW	0x00F8
#define T_MAC_DINGBAT_HEAVY_BLACK_FEATHERED_NORTH_EAST_ARROW	0x00F9
#define T_MAC_DINGBAT_TEARDROP_BARBED_RIGHTWARDS_ARROW	0x00FA
#define T_MAC_DINGBAT_HEAVY_TEARDROP_SHANKED_RIGHTWARDS_ARROW	0x00FB
#define T_MAC_DINGBAT_WEDGE_TAILED_RIGHTWARDS_ARROW	0x00FC
#define T_MAC_DINGBAT_HEAVY_WEDGE_TAILED_RIGHTWARDS_ARROW	0x00FD
#define T_MAC_DINGBAT_OPEN_OUTLINED_RIGHTWARDS_ARROW	0x00FE
#define T_MAC_DINGBAT_UNDEFINED_00FF	0x00FF

/* list of all characters */
#define T_LIST_MAC_DINGBAT(EL) \
    EL(_NULL) \
    EL(_START_OF_HEADING) \
    EL(_START_OF_TEXT) \
    EL(_END_OF_TEXT) \
    EL(_END_OF_TRANSMISSION) \
    EL(_ENQUIRY) \
    EL(_ACKNOWLEDGE) \
    EL(_BELL) \
    EL(_BACKSPACE) \
    EL(_HORIZONTAL_TABULATION) \
    EL(_LINE_FEED) \
    EL(_VERTICAL_TABULATION) \
    EL(_FORM_FEED) \
    EL(_CARRIAGE_RETURN) \
    EL(_SHIFT_OUT) \
    EL(_SHIFT_IN) \
    EL(_DATA_LINK_ESCAPE) \
    EL(_DEVICE_CONTROL_ONE) \
    EL(_DEVICE_CONTROL_TWO) \
    EL(_DEVICE_CONTROL_THREE) \
    EL(_DEVICE_CONTROL_FOUR) \
    EL(_NEGATIVE_ACKNOWLEDGE) \
    EL(_SYNCHRONOUS_IDLE) \
    EL(_END_OF_TRANSMISSION_BLOCK) \
    EL(_CANCEL) \
    EL(_END_OF_MEDIUM) \
    EL(_SUBSTITUTE) \
    EL(_ESCAPE) \
    EL(_FILE_SEPARATOR) \
    EL(_GROUP_SEPARATOR) \
    EL(_RECORD_SEPARATOR) \
    EL(_UNIT_SEPARATOR) \
    EL(_SPACE) \
    EL(_UPPER_BLADE_SCISSORS) \
    EL(_BLACK_SCISSORS) \
    EL(_LOWER_BLADE_SCISSORS) \
    EL(_WHITE_SCISSORS) \
    EL(_BLACK_TELEPHONE) \
    EL(_TELEPHONE_LOCATION_SIGN) \
    EL(_TAPE_DRIVE) \
    EL(_AIRPLANE) \
    EL(_ENVELOPE) \
    EL(_BLACK_RIGHT_POINTING_INDEX) \
    EL(_WHITE_RIGHT_POINTING_INDEX) \
    EL(_VICTORY_HAND) \
    EL(_WRITING_HAND) \
    EL(_LOWER_RIGHT_PENCIL) \
    EL(_PENCIL) \
    EL(_UPPER_RIGHT_PENCIL) \
    EL(_WHITE_NIB) \
    EL(_BLACK_NIB) \
    EL(_CHECK_MARK) \
    EL(_HEAVY_CHECK_MARK) \
    EL(_MULTIPLICATION_X) \
    EL(_HEAVY_MULTIPLICATION_X) \
    EL(_BALLOT_X) \
    EL(_HEAVY_BALLOT_X) \
    EL(_OUTLINED_GREEK_CROSS) \
    EL(_HEAVY_GREEK_CROSS) \
    EL(_OPEN_CENTRE_CROSS) \
    EL(_HEAVY_OPEN_CENTRE_CROSS) \
    EL(_LATIN_CROSS) \
    EL(_SHADOWED_WHITE_LATIN_CROSS) \
    EL(_OUTLINED_LATIN_CROSS) \
    EL(_MALTESE_CROSS) \
    EL(_STAR_OF_DAVID) \
    EL(_FOUR_TEARDROP_SPOKED_ASTERISK) \
    EL(_FOUR_BALLOON_SPOKED_ASTERISK) \
    EL(_HEAVY_FOUR_BALLOON_SPOKED_ASTERISK) \
    EL(_FOUR_CLUB_SPOKED_ASTERISK) \
    EL(_BLACK_FOUR_POINTED_STAR) \
    EL(_WHITE_FOUR_POINTED_STAR) \
    EL(_BLACK_STAR) \
    EL(_STRESS_OUTLINED_WHITE_STAR) \
    EL(_CIRCLED_WHITE_STAR) \
    EL(_OPEN_CENTRE_BLACK_STAR) \
    EL(_BLACK_CENTRE_WHITE_STAR) \
    EL(_OUTLINED_BLACK_STAR) \
    EL(_HEAVY_OUTLINED_BLACK_STAR) \
    EL(_PINWHEEL_STAR) \
    EL(_SHADOWED_WHITE_STAR) \
    EL(_HEAVY_ASTERISK) \
    EL(_OPEN_CENTRE_ASTERISK) \
    EL(_EIGHT_SPOKED_ASTERISK) \
    EL(_EIGHT_POINTED_BLACK_STAR) \
    EL(_EIGHT_POINTED_PINWHEEL_STAR) \
    EL(_SIX_POINTED_BLACK_STAR) \
    EL(_EIGHT_POINTED_RECTILINEAR_BLACK_STAR) \
    EL(_HEAVY_EIGHT_POINTED_RECTILINEAR_BLACK_STAR) \
    EL(_TWELVE_POINTED_BLACK_STAR) \
    EL(_SIXTEEN_POINTED_ASTERISK) \
    EL(_TEARDROP_SPOKED_ASTERISK) \
    EL(_OPEN_CENTRE_TEARDROP_SPOKED_ASTERISK) \
    EL(_HEAVY_TEARDROP_SPOKED_ASTERISK) \
    EL(_SIX_PETALLED_BLACK_AND_WHITE_FLORETTE) \
    EL(_BLACK_FLORETTE) \
    EL(_WHITE_FLORETTE) \
    EL(_EIGHT_PETALLED_OUTLINED_BLACK_FLORETTE) \
    EL(_CIRCLED_OPEN_CENTRE_EIGHT_POINTED_STAR) \
    EL(_HEAVY_TEARDROP_SPOKED_PINWHEEL_ASTERISK) \
    EL(_SNOWFLAKE) \
    EL(_TIGHT_TRIFOLIATE_SNOWFLAKE) \
    EL(_HEAVY_CHEVRON_SNOWFLAKE) \
    EL(_SPARKLE) \
    EL(_HEAVY_SPARKLE) \
    EL(_BALLOON_SPOKED_ASTERISK) \
    EL(_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK) \
    EL(_HEAVY_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK) \
    EL(_BLACK_CIRCLE) \
    EL(_SHADOWED_WHITE_CIRCLE) \
    EL(_BLACK_SQUARE) \
    EL(_LOWER_RIGHT_DROP_SHADOWED_WHITE_SQUARE) \
    EL(_UPPER_RIGHT_DROP_SHADOWED_WHITE_SQUARE) \
    EL(_LOWER_RIGHT_SHADOWED_WHITE_SQUARE) \
    EL(_UPPER_RIGHT_SHADOWED_WHITE_SQUARE) \
    EL(_BLACK_UP_POINTING_TRIANGLE) \
    EL(_BLACK_DOWN_POINTING_TRIANGLE) \
    EL(_BLACK_DIAMOND) \
    EL(_BLACK_DIAMOND_MINUS_WHITE_X) \
    EL(_RIGHT_HALF_BLACK_CIRCLE) \
    EL(_LIGHT_VERTICAL_BAR) \
    EL(_MEDIUM_VERTICAL_BAR) \
    EL(_HEAVY_VERTICAL_BAR) \
    EL(_HEAVY_SINGLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_SINGLE_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_DOUBLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_DOUBLE_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_UNDEFINED_007F) \
    EL(_UNDEFINED_0080) \
    EL(_UNDEFINED_0081) \
    EL(_UNDEFINED_0082) \
    EL(_UNDEFINED_0083) \
    EL(_UNDEFINED_0084) \
    EL(_UNDEFINED_0085) \
    EL(_UNDEFINED_0086) \
    EL(_UNDEFINED_0087) \
    EL(_UNDEFINED_0088) \
    EL(_UNDEFINED_0089) \
    EL(_UNDEFINED_008A) \
    EL(_UNDEFINED_008B) \
    EL(_UNDEFINED_008C) \
    EL(_UNDEFINED_008D) \
    EL(_UNDEFINED_008E) \
    EL(_UNDEFINED_008F) \
    EL(_UNDEFINED_0090) \
    EL(_UNDEFINED_0091) \
    EL(_UNDEFINED_0092) \
    EL(_UNDEFINED_0093) \
    EL(_UNDEFINED_0094) \
    EL(_UNDEFINED_0095) \
    EL(_UNDEFINED_0096) \
    EL(_UNDEFINED_0097) \
    EL(_UNDEFINED_0098) \
    EL(_UNDEFINED_0099) \
    EL(_UNDEFINED_009A) \
    EL(_UNDEFINED_009B) \
    EL(_UNDEFINED_009C) \
    EL(_UNDEFINED_009D) \
    EL(_UNDEFINED_009E) \
    EL(_UNDEFINED_009F) \
    EL(_UNDEFINED_00A0) \
    EL(_CURVED_STEM_PARAGRAPH_SIGN_ORNAMENT) \
    EL(_HEAVY_EXCLAMATION_MARK_ORNAMENT) \
    EL(_HEAVY_HEART_EXCLAMATION_MARK_ORNAMENT) \
    EL(_HEAVY_BLACK_HEART) \
    EL(_ROTATED_HEAVY_BLACK_HEART_BULLET) \
    EL(_FLORAL_HEART) \
    EL(_ROTATED_FLORAL_HEART_BULLET) \
    EL(_BLACK_CLUB_SUIT) \
    EL(_BLACK_DIAMOND_SUIT) \
    EL(_BLACK_HEART_SUIT) \
    EL(_BLACK_SPADE_SUIT) \
    EL(_CIRCLED_DIGIT_ONE) \
    EL(_CIRCLED_DIGIT_TWO) \
    EL(_CIRCLED_DIGIT_THREE) \
    EL(_CIRCLED_DIGIT_FOUR) \
    EL(_CIRCLED_DIGIT_FIVE) \
    EL(_CIRCLED_DIGIT_SIX) \
    EL(_CIRCLED_DIGIT_SEVEN) \
    EL(_CIRCLED_DIGIT_EIGHT) \
    EL(_CIRCLED_DIGIT_NINE) \
    EL(_CIRCLED_NUMBER_TEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_ONE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_TWO) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_THREE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FOUR) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FIVE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SIX) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SEVEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_EIGHT) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_NINE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_NUMBER_TEN) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_ONE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_TWO) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_THREE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FOUR) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FIVE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SIX) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SEVEN) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_EIGHT) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_NINE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_NUMBER_TEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_ONE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_TWO) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_THREE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FOUR) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FIVE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SIX) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SEVEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_EIGHT) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_NINE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_NUMBER_TEN) \
    EL(_HEAVY_WIDE_HEADED_RIGHTWARDS_ARROW) \
    EL(_RIGHTWARDS_ARROW) \
    EL(_LEFT_RIGHT_ARROW) \
    EL(_UP_DOWN_ARROW) \
    EL(_HEAVY_SOUTH_EAST_ARROW) \
    EL(_HEAVY_RIGHTWARDS_ARROW) \
    EL(_HEAVY_NORTH_EAST_ARROW) \
    EL(_DRAFTING_POINT_RIGHTWARDS_ARROW) \
    EL(_HEAVY_ROUND_TIPPED_RIGHTWARDS_ARROW) \
    EL(_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_BLACK_RIGHTWARDS_ARROW) \
    EL(_THREE_D_TOP_LIGHTED_RIGHTWARDS_ARROWHEAD) \
    EL(_THREE_D_BOTTOM_LIGHTED_RIGHTWARDS_ARROWHEAD) \
    EL(_BLACK_RIGHTWARDS_ARROWHEAD) \
    EL(_HEAVY_BLACK_CURVED_DOWNWARDS_AND_RIGHTWARDS_ARROW) \
    EL(_HEAVY_BLACK_CURVED_UPWARDS_AND_RIGHTWARDS_ARROW) \
    EL(_SQUAT_BLACK_RIGHTWARDS_ARROW) \
    EL(_HEAVY_CONCAVE_POINTED_BLACK_RIGHTWARDS_ARROW) \
    EL(_RIGHT_SHADED_WHITE_RIGHTWARDS_ARROW) \
    EL(_LEFT_SHADED_WHITE_RIGHTWARDS_ARROW) \
    EL(_BACK_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_FRONT_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_HEAVY_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_HEAVY_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_NOTCHED_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_UNDEFINED_00F0) \
    EL(_NOTCHED_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_CIRCLED_HEAVY_WHITE_RIGHTWARDS_ARROW) \
    EL(_WHITE_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_BLACK_FEATHERED_SOUTH_EAST_ARROW) \
    EL(_BLACK_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_BLACK_FEATHERED_NORTH_EAST_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_SOUTH_EAST_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_NORTH_EAST_ARROW) \
    EL(_TEARDROP_BARBED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_TEARDROP_SHANKED_RIGHTWARDS_ARROW) \
    EL(_WEDGE_TAILED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_WEDGE_TAILED_RIGHTWARDS_ARROW) \
    EL(_OPEN_OUTLINED_RIGHTWARDS_ARROW) \
    EL(_UNDEFINED_00FF)

/* list of all characters NOT IDENTICALLY MAPPED to UTF-16 */
#define T_NLIST_MAC_DINGBAT(EL) \
    EL(_UPPER_BLADE_SCISSORS) \
    EL(_BLACK_SCISSORS) \
    EL(_LOWER_BLADE_SCISSORS) \
    EL(_WHITE_SCISSORS) \
    EL(_BLACK_TELEPHONE) \
    EL(_TELEPHONE_LOCATION_SIGN) \
    EL(_TAPE_DRIVE) \
    EL(_AIRPLANE) \
    EL(_ENVELOPE) \
    EL(_BLACK_RIGHT_POINTING_INDEX) \
    EL(_WHITE_RIGHT_POINTING_INDEX) \
    EL(_VICTORY_HAND) \
    EL(_WRITING_HAND) \
    EL(_LOWER_RIGHT_PENCIL) \
    EL(_PENCIL) \
    EL(_UPPER_RIGHT_PENCIL) \
    EL(_WHITE_NIB) \
    EL(_BLACK_NIB) \
    EL(_CHECK_MARK) \
    EL(_HEAVY_CHECK_MARK) \
    EL(_MULTIPLICATION_X) \
    EL(_HEAVY_MULTIPLICATION_X) \
    EL(_BALLOT_X) \
    EL(_HEAVY_BALLOT_X) \
    EL(_OUTLINED_GREEK_CROSS) \
    EL(_HEAVY_GREEK_CROSS) \
    EL(_OPEN_CENTRE_CROSS) \
    EL(_HEAVY_OPEN_CENTRE_CROSS) \
    EL(_LATIN_CROSS) \
    EL(_SHADOWED_WHITE_LATIN_CROSS) \
    EL(_OUTLINED_LATIN_CROSS) \
    EL(_MALTESE_CROSS) \
    EL(_STAR_OF_DAVID) \
    EL(_FOUR_TEARDROP_SPOKED_ASTERISK) \
    EL(_FOUR_BALLOON_SPOKED_ASTERISK) \
    EL(_HEAVY_FOUR_BALLOON_SPOKED_ASTERISK) \
    EL(_FOUR_CLUB_SPOKED_ASTERISK) \
    EL(_BLACK_FOUR_POINTED_STAR) \
    EL(_WHITE_FOUR_POINTED_STAR) \
    EL(_BLACK_STAR) \
    EL(_STRESS_OUTLINED_WHITE_STAR) \
    EL(_CIRCLED_WHITE_STAR) \
    EL(_OPEN_CENTRE_BLACK_STAR) \
    EL(_BLACK_CENTRE_WHITE_STAR) \
    EL(_OUTLINED_BLACK_STAR) \
    EL(_HEAVY_OUTLINED_BLACK_STAR) \
    EL(_PINWHEEL_STAR) \
    EL(_SHADOWED_WHITE_STAR) \
    EL(_HEAVY_ASTERISK) \
    EL(_OPEN_CENTRE_ASTERISK) \
    EL(_EIGHT_SPOKED_ASTERISK) \
    EL(_EIGHT_POINTED_BLACK_STAR) \
    EL(_EIGHT_POINTED_PINWHEEL_STAR) \
    EL(_SIX_POINTED_BLACK_STAR) \
    EL(_EIGHT_POINTED_RECTILINEAR_BLACK_STAR) \
    EL(_HEAVY_EIGHT_POINTED_RECTILINEAR_BLACK_STAR) \
    EL(_TWELVE_POINTED_BLACK_STAR) \
    EL(_SIXTEEN_POINTED_ASTERISK) \
    EL(_TEARDROP_SPOKED_ASTERISK) \
    EL(_OPEN_CENTRE_TEARDROP_SPOKED_ASTERISK) \
    EL(_HEAVY_TEARDROP_SPOKED_ASTERISK) \
    EL(_SIX_PETALLED_BLACK_AND_WHITE_FLORETTE) \
    EL(_BLACK_FLORETTE) \
    EL(_WHITE_FLORETTE) \
    EL(_EIGHT_PETALLED_OUTLINED_BLACK_FLORETTE) \
    EL(_CIRCLED_OPEN_CENTRE_EIGHT_POINTED_STAR) \
    EL(_HEAVY_TEARDROP_SPOKED_PINWHEEL_ASTERISK) \
    EL(_SNOWFLAKE) \
    EL(_TIGHT_TRIFOLIATE_SNOWFLAKE) \
    EL(_HEAVY_CHEVRON_SNOWFLAKE) \
    EL(_SPARKLE) \
    EL(_HEAVY_SPARKLE) \
    EL(_BALLOON_SPOKED_ASTERISK) \
    EL(_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK) \
    EL(_HEAVY_EIGHT_TEARDROP_SPOKED_PROPELLER_ASTERISK) \
    EL(_BLACK_CIRCLE) \
    EL(_SHADOWED_WHITE_CIRCLE) \
    EL(_BLACK_SQUARE) \
    EL(_LOWER_RIGHT_DROP_SHADOWED_WHITE_SQUARE) \
    EL(_UPPER_RIGHT_DROP_SHADOWED_WHITE_SQUARE) \
    EL(_LOWER_RIGHT_SHADOWED_WHITE_SQUARE) \
    EL(_UPPER_RIGHT_SHADOWED_WHITE_SQUARE) \
    EL(_BLACK_UP_POINTING_TRIANGLE) \
    EL(_BLACK_DOWN_POINTING_TRIANGLE) \
    EL(_BLACK_DIAMOND) \
    EL(_BLACK_DIAMOND_MINUS_WHITE_X) \
    EL(_RIGHT_HALF_BLACK_CIRCLE) \
    EL(_LIGHT_VERTICAL_BAR) \
    EL(_MEDIUM_VERTICAL_BAR) \
    EL(_HEAVY_VERTICAL_BAR) \
    EL(_HEAVY_SINGLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_SINGLE_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_DOUBLE_TURNED_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_HEAVY_DOUBLE_COMMA_QUOTATION_MARK_ORNAMENT) \
    EL(_UNDEFINED_00A0) \
    EL(_CURVED_STEM_PARAGRAPH_SIGN_ORNAMENT) \
    EL(_HEAVY_EXCLAMATION_MARK_ORNAMENT) \
    EL(_HEAVY_HEART_EXCLAMATION_MARK_ORNAMENT) \
    EL(_HEAVY_BLACK_HEART) \
    EL(_ROTATED_HEAVY_BLACK_HEART_BULLET) \
    EL(_FLORAL_HEART) \
    EL(_ROTATED_FLORAL_HEART_BULLET) \
    EL(_BLACK_CLUB_SUIT) \
    EL(_BLACK_DIAMOND_SUIT) \
    EL(_BLACK_HEART_SUIT) \
    EL(_BLACK_SPADE_SUIT) \
    EL(_CIRCLED_DIGIT_ONE) \
    EL(_CIRCLED_DIGIT_TWO) \
    EL(_CIRCLED_DIGIT_THREE) \
    EL(_CIRCLED_DIGIT_FOUR) \
    EL(_CIRCLED_DIGIT_FIVE) \
    EL(_CIRCLED_DIGIT_SIX) \
    EL(_CIRCLED_DIGIT_SEVEN) \
    EL(_CIRCLED_DIGIT_EIGHT) \
    EL(_CIRCLED_DIGIT_NINE) \
    EL(_CIRCLED_NUMBER_TEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_ONE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_TWO) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_THREE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FOUR) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_FIVE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SIX) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_SEVEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_EIGHT) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_DIGIT_NINE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_NUMBER_TEN) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_ONE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_TWO) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_THREE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FOUR) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_FIVE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SIX) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_SEVEN) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_EIGHT) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_DIGIT_NINE) \
    EL(_DINGBAT_CIRCLED_SANS_SERIF_NUMBER_TEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_ONE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_TWO) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_THREE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FOUR) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_FIVE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SIX) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_SEVEN) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_EIGHT) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_DIGIT_NINE) \
    EL(_DINGBAT_NEGATIVE_CIRCLED_SANS_SERIF_NUMBER_TEN) \
    EL(_HEAVY_WIDE_HEADED_RIGHTWARDS_ARROW) \
    EL(_RIGHTWARDS_ARROW) \
    EL(_LEFT_RIGHT_ARROW) \
    EL(_UP_DOWN_ARROW) \
    EL(_HEAVY_SOUTH_EAST_ARROW) \
    EL(_HEAVY_RIGHTWARDS_ARROW) \
    EL(_HEAVY_NORTH_EAST_ARROW) \
    EL(_DRAFTING_POINT_RIGHTWARDS_ARROW) \
    EL(_HEAVY_ROUND_TIPPED_RIGHTWARDS_ARROW) \
    EL(_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_DASHED_TRIANGLE_HEADED_RIGHTWARDS_ARROW) \
    EL(_BLACK_RIGHTWARDS_ARROW) \
    EL(_THREE_D_TOP_LIGHTED_RIGHTWARDS_ARROWHEAD) \
    EL(_THREE_D_BOTTOM_LIGHTED_RIGHTWARDS_ARROWHEAD) \
    EL(_BLACK_RIGHTWARDS_ARROWHEAD) \
    EL(_HEAVY_BLACK_CURVED_DOWNWARDS_AND_RIGHTWARDS_ARROW) \
    EL(_HEAVY_BLACK_CURVED_UPWARDS_AND_RIGHTWARDS_ARROW) \
    EL(_SQUAT_BLACK_RIGHTWARDS_ARROW) \
    EL(_HEAVY_CONCAVE_POINTED_BLACK_RIGHTWARDS_ARROW) \
    EL(_RIGHT_SHADED_WHITE_RIGHTWARDS_ARROW) \
    EL(_LEFT_SHADED_WHITE_RIGHTWARDS_ARROW) \
    EL(_BACK_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_FRONT_TILTED_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_HEAVY_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_HEAVY_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_NOTCHED_LOWER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_UNDEFINED_00F0) \
    EL(_NOTCHED_UPPER_RIGHT_SHADOWED_WHITE_RIGHTWARDS_ARROW) \
    EL(_CIRCLED_HEAVY_WHITE_RIGHTWARDS_ARROW) \
    EL(_WHITE_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_BLACK_FEATHERED_SOUTH_EAST_ARROW) \
    EL(_BLACK_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_BLACK_FEATHERED_NORTH_EAST_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_SOUTH_EAST_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_BLACK_FEATHERED_NORTH_EAST_ARROW) \
    EL(_TEARDROP_BARBED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_TEARDROP_SHANKED_RIGHTWARDS_ARROW) \
    EL(_WEDGE_TAILED_RIGHTWARDS_ARROW) \
    EL(_HEAVY_WEDGE_TAILED_RIGHTWARDS_ARROW) \
    EL(_OPEN_OUTLINED_RIGHTWARDS_ARROW) \
    EL(_UNDEFINED_00FF)

#endif /* _TUTF_MAC_DINGBAT_H */
