
/*
 * This file was automatically generated
 * from Linux kernel sources with "from-kernel.bash"
 * Do not edit!
 */


#ifndef _TUTF_KOI8_U_H
#define _TUTF_KOI8_U_H

#define T_KOI8_U_NULL	0x0000
#define T_KOI8_U_START_OF_HEADING	0x0001
#define T_KOI8_U_START_OF_TEXT	0x0002
#define T_KOI8_U_END_OF_TEXT	0x0003
#define T_KOI8_U_END_OF_TRANSMISSION	0x0004
#define T_KOI8_U_ENQUIRY	0x0005
#define T_KOI8_U_ACKNOWLEDGE	0x0006
#define T_KOI8_U_BELL	0x0007
#define T_KOI8_U_BACKSPACE	0x0008
#define T_KOI8_U_HORIZONTAL_TABULATION	0x0009
#define T_KOI8_U_LINE_FEED	0x000A
#define T_KOI8_U_VERTICAL_TABULATION	0x000B
#define T_KOI8_U_FORM_FEED	0x000C
#define T_KOI8_U_CARRIAGE_RETURN	0x000D
#define T_KOI8_U_SHIFT_OUT	0x000E
#define T_KOI8_U_SHIFT_IN	0x000F
#define T_KOI8_U_DATA_LINK_ESCAPE	0x0010
#define T_KOI8_U_DEVICE_CONTROL_ONE	0x0011
#define T_KOI8_U_DEVICE_CONTROL_TWO	0x0012
#define T_KOI8_U_DEVICE_CONTROL_THREE	0x0013
#define T_KOI8_U_DEVICE_CONTROL_FOUR	0x0014
#define T_KOI8_U_NEGATIVE_ACKNOWLEDGE	0x0015
#define T_KOI8_U_SYNCHRONOUS_IDLE	0x0016
#define T_KOI8_U_END_OF_TRANSMISSION_BLOCK	0x0017
#define T_KOI8_U_CANCEL	0x0018
#define T_KOI8_U_END_OF_MEDIUM	0x0019
#define T_KOI8_U_SUBSTITUTE	0x001A
#define T_KOI8_U_ESCAPE	0x001B
#define T_KOI8_U_FILE_SEPARATOR	0x001C
#define T_KOI8_U_GROUP_SEPARATOR	0x001D
#define T_KOI8_U_RECORD_SEPARATOR	0x001E
#define T_KOI8_U_UNIT_SEPARATOR	0x001F
#define T_KOI8_U_SPACE	0x0020
#define T_KOI8_U_EXCLAMATION_MARK	0x0021
#define T_KOI8_U_QUOTATION_MARK	0x0022
#define T_KOI8_U_NUMBER_SIGN	0x0023
#define T_KOI8_U_DOLLAR_SIGN	0x0024
#define T_KOI8_U_PERCENT_SIGN	0x0025
#define T_KOI8_U_AMPERSAND	0x0026
#define T_KOI8_U_APOSTROPHE	0x0027
#define T_KOI8_U_LEFT_PARENTHESIS	0x0028
#define T_KOI8_U_RIGHT_PARENTHESIS	0x0029
#define T_KOI8_U_ASTERISK	0x002A
#define T_KOI8_U_PLUS_SIGN	0x002B
#define T_KOI8_U_COMMA	0x002C
#define T_KOI8_U_HYPHEN_MINUS	0x002D
#define T_KOI8_U_FULL_STOP	0x002E
#define T_KOI8_U_SOLIDUS	0x002F
#define T_KOI8_U_DIGIT_ZERO	0x0030
#define T_KOI8_U_DIGIT_ONE	0x0031
#define T_KOI8_U_DIGIT_TWO	0x0032
#define T_KOI8_U_DIGIT_THREE	0x0033
#define T_KOI8_U_DIGIT_FOUR	0x0034
#define T_KOI8_U_DIGIT_FIVE	0x0035
#define T_KOI8_U_DIGIT_SIX	0x0036
#define T_KOI8_U_DIGIT_SEVEN	0x0037
#define T_KOI8_U_DIGIT_EIGHT	0x0038
#define T_KOI8_U_DIGIT_NINE	0x0039
#define T_KOI8_U_COLON	0x003A
#define T_KOI8_U_SEMICOLON	0x003B
#define T_KOI8_U_LESS_THAN_SIGN	0x003C
#define T_KOI8_U_EQUALS_SIGN	0x003D
#define T_KOI8_U_GREATER_THAN_SIGN	0x003E
#define T_KOI8_U_QUESTION_MARK	0x003F
#define T_KOI8_U_COMMERCIAL_AT	0x0040
#define T_KOI8_U_LATIN_CAPITAL_LETTER_A	0x0041
#define T_KOI8_U_LATIN_CAPITAL_LETTER_B	0x0042
#define T_KOI8_U_LATIN_CAPITAL_LETTER_C	0x0043
#define T_KOI8_U_LATIN_CAPITAL_LETTER_D	0x0044
#define T_KOI8_U_LATIN_CAPITAL_LETTER_E	0x0045
#define T_KOI8_U_LATIN_CAPITAL_LETTER_F	0x0046
#define T_KOI8_U_LATIN_CAPITAL_LETTER_G	0x0047
#define T_KOI8_U_LATIN_CAPITAL_LETTER_H	0x0048
#define T_KOI8_U_LATIN_CAPITAL_LETTER_I	0x0049
#define T_KOI8_U_LATIN_CAPITAL_LETTER_J	0x004A
#define T_KOI8_U_LATIN_CAPITAL_LETTER_K	0x004B
#define T_KOI8_U_LATIN_CAPITAL_LETTER_L	0x004C
#define T_KOI8_U_LATIN_CAPITAL_LETTER_M	0x004D
#define T_KOI8_U_LATIN_CAPITAL_LETTER_N	0x004E
#define T_KOI8_U_LATIN_CAPITAL_LETTER_O	0x004F
#define T_KOI8_U_LATIN_CAPITAL_LETTER_P	0x0050
#define T_KOI8_U_LATIN_CAPITAL_LETTER_Q	0x0051
#define T_KOI8_U_LATIN_CAPITAL_LETTER_R	0x0052
#define T_KOI8_U_LATIN_CAPITAL_LETTER_S	0x0053
#define T_KOI8_U_LATIN_CAPITAL_LETTER_T	0x0054
#define T_KOI8_U_LATIN_CAPITAL_LETTER_U	0x0055
#define T_KOI8_U_LATIN_CAPITAL_LETTER_V	0x0056
#define T_KOI8_U_LATIN_CAPITAL_LETTER_W	0x0057
#define T_KOI8_U_LATIN_CAPITAL_LETTER_X	0x0058
#define T_KOI8_U_LATIN_CAPITAL_LETTER_Y	0x0059
#define T_KOI8_U_LATIN_CAPITAL_LETTER_Z	0x005A
#define T_KOI8_U_LEFT_SQUARE_BRACKET	0x005B
#define T_KOI8_U_REVERSE_SOLIDUS	0x005C
#define T_KOI8_U_RIGHT_SQUARE_BRACKET	0x005D
#define T_KOI8_U_CIRCUMFLEX_ACCENT	0x005E
#define T_KOI8_U_LOW_LINE	0x005F
#define T_KOI8_U_GRAVE_ACCENT	0x0060
#define T_KOI8_U_LATIN_SMALL_LETTER_A	0x0061
#define T_KOI8_U_LATIN_SMALL_LETTER_B	0x0062
#define T_KOI8_U_LATIN_SMALL_LETTER_C	0x0063
#define T_KOI8_U_LATIN_SMALL_LETTER_D	0x0064
#define T_KOI8_U_LATIN_SMALL_LETTER_E	0x0065
#define T_KOI8_U_LATIN_SMALL_LETTER_F	0x0066
#define T_KOI8_U_LATIN_SMALL_LETTER_G	0x0067
#define T_KOI8_U_LATIN_SMALL_LETTER_H	0x0068
#define T_KOI8_U_LATIN_SMALL_LETTER_I	0x0069
#define T_KOI8_U_LATIN_SMALL_LETTER_J	0x006A
#define T_KOI8_U_LATIN_SMALL_LETTER_K	0x006B
#define T_KOI8_U_LATIN_SMALL_LETTER_L	0x006C
#define T_KOI8_U_LATIN_SMALL_LETTER_M	0x006D
#define T_KOI8_U_LATIN_SMALL_LETTER_N	0x006E
#define T_KOI8_U_LATIN_SMALL_LETTER_O	0x006F
#define T_KOI8_U_LATIN_SMALL_LETTER_P	0x0070
#define T_KOI8_U_LATIN_SMALL_LETTER_Q	0x0071
#define T_KOI8_U_LATIN_SMALL_LETTER_R	0x0072
#define T_KOI8_U_LATIN_SMALL_LETTER_S	0x0073
#define T_KOI8_U_LATIN_SMALL_LETTER_T	0x0074
#define T_KOI8_U_LATIN_SMALL_LETTER_U	0x0075
#define T_KOI8_U_LATIN_SMALL_LETTER_V	0x0076
#define T_KOI8_U_LATIN_SMALL_LETTER_W	0x0077
#define T_KOI8_U_LATIN_SMALL_LETTER_X	0x0078
#define T_KOI8_U_LATIN_SMALL_LETTER_Y	0x0079
#define T_KOI8_U_LATIN_SMALL_LETTER_Z	0x007A
#define T_KOI8_U_LEFT_CURLY_BRACKET	0x007B
#define T_KOI8_U_VERTICAL_LINE	0x007C
#define T_KOI8_U_RIGHT_CURLY_BRACKET	0x007D
#define T_KOI8_U_TILDE	0x007E
#define T_KOI8_U_UNDEFINED_007F	0x007F
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_HORIZONTAL	0x0080
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_VERTICAL	0x0081
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT	0x0082
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT	0x0083
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT	0x0084
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_UP_AND_LEFT	0x0085
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT	0x0086
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT	0x0087
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL	0x0088
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL	0x0089
#define T_KOI8_U_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL	0x008A
#define T_KOI8_U_UPPER_HALF_BLOCK	0x008B
#define T_KOI8_U_LOWER_HALF_BLOCK	0x008C
#define T_KOI8_U_FULL_BLOCK	0x008D
#define T_KOI8_U_LEFT_HALF_BLOCK	0x008E
#define T_KOI8_U_RIGHT_HALF_BLOCK	0x008F
#define T_KOI8_U_LIGHT_SHADE	0x0090
#define T_KOI8_U_MEDIUM_SHADE	0x0091
#define T_KOI8_U_DARK_SHADE	0x0092
#define T_KOI8_U_TOP_HALF_INTEGRAL	0x0093
#define T_KOI8_U_BLACK_SQUARE	0x0094
#define T_KOI8_U_BULLET_OPERATOR	0x0095
#define T_KOI8_U_SQUARE_ROOT	0x0096
#define T_KOI8_U_ALMOST_EQUAL_TO	0x0097
#define T_KOI8_U_LESS_THAN_OR_EQUAL_TO	0x0098
#define T_KOI8_U_GREATER_THAN_OR_EQUAL_TO	0x0099
#define T_KOI8_U_NO_BREAK_SPACE	0x009A
#define T_KOI8_U_BOTTOM_HALF_INTEGRAL	0x009B
#define T_KOI8_U_DEGREE_SIGN	0x009C
#define T_KOI8_U_SUPERSCRIPT_TWO	0x009D
#define T_KOI8_U_MIDDLE_DOT	0x009E
#define T_KOI8_U_DIVISION_SIGN	0x009F
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_HORIZONTAL	0x00A0
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_VERTICAL	0x00A1
#define T_KOI8_U_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE	0x00A2
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_IO	0x00A3
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_UKRAINIAN_IE	0x00A4
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT	0x00A5
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I	0x00A6
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_YI	0x00A7
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT	0x00A8
#define T_KOI8_U_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE	0x00A9
#define T_KOI8_U_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE	0x00AA
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT	0x00AB
#define T_KOI8_U_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE	0x00AC
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_GHE_WITH_UPTURN	0x00AD
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT	0x00AE
#define T_KOI8_U_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE	0x00AF
#define T_KOI8_U_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE	0x00B0
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT	0x00B1
#define T_KOI8_U_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE	0x00B2
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_IO	0x00B3
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_UKRAINIAN_IE	0x00B4
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT	0x00B5
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I	0x00B6
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_YI	0x00B7
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL	0x00B8
#define T_KOI8_U_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE	0x00B9
#define T_KOI8_U_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE	0x00BA
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL	0x00BB
#define T_KOI8_U_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE	0x00BC
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_GHE_WITH_UPTURN	0x00BD
#define T_KOI8_U_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL	0x00BE
#define T_KOI8_U_COPYRIGHT_SIGN	0x00BF
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_YU	0x00C0
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_A	0x00C1
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_BE	0x00C2
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_TSE	0x00C3
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_DE	0x00C4
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_IE	0x00C5
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_EF	0x00C6
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_GHE	0x00C7
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_HA	0x00C8
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_I	0x00C9
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_SHORT_I	0x00CA
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_KA	0x00CB
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_EL	0x00CC
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_EM	0x00CD
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_EN	0x00CE
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_O	0x00CF
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_PE	0x00D0
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_YA	0x00D1
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_ER	0x00D2
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_ES	0x00D3
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_TE	0x00D4
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_U	0x00D5
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_ZHE	0x00D6
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_VE	0x00D7
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_SOFT_SIGN	0x00D8
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_YERU	0x00D9
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_ZE	0x00DA
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_SHA	0x00DB
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_E	0x00DC
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_SHCHA	0x00DD
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_CHE	0x00DE
#define T_KOI8_U_CYRILLIC_SMALL_LETTER_HARD_SIGN	0x00DF
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_YU	0x00E0
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_A	0x00E1
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_BE	0x00E2
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_TSE	0x00E3
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_DE	0x00E4
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_IE	0x00E5
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_EF	0x00E6
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_GHE	0x00E7
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_HA	0x00E8
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_I	0x00E9
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_SHORT_I	0x00EA
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_KA	0x00EB
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_EL	0x00EC
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_EM	0x00ED
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_EN	0x00EE
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_O	0x00EF
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_PE	0x00F0
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_YA	0x00F1
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_ER	0x00F2
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_ES	0x00F3
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_TE	0x00F4
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_U	0x00F5
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_ZHE	0x00F6
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_VE	0x00F7
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_SOFT_SIGN	0x00F8
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_YERU	0x00F9
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_ZE	0x00FA
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_SHA	0x00FB
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_E	0x00FC
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_SHCHA	0x00FD
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_CHE	0x00FE
#define T_KOI8_U_CYRILLIC_CAPITAL_LETTER_HARD_SIGN	0x00FF

/* list of all characters */
#define T_LIST_KOI8_U(EL) \
    EL(_NULL) \
    EL(_START_OF_HEADING) \
    EL(_START_OF_TEXT) \
    EL(_END_OF_TEXT) \
    EL(_END_OF_TRANSMISSION) \
    EL(_ENQUIRY) \
    EL(_ACKNOWLEDGE) \
    EL(_BELL) \
    EL(_BACKSPACE) \
    EL(_HORIZONTAL_TABULATION) \
    EL(_LINE_FEED) \
    EL(_VERTICAL_TABULATION) \
    EL(_FORM_FEED) \
    EL(_CARRIAGE_RETURN) \
    EL(_SHIFT_OUT) \
    EL(_SHIFT_IN) \
    EL(_DATA_LINK_ESCAPE) \
    EL(_DEVICE_CONTROL_ONE) \
    EL(_DEVICE_CONTROL_TWO) \
    EL(_DEVICE_CONTROL_THREE) \
    EL(_DEVICE_CONTROL_FOUR) \
    EL(_NEGATIVE_ACKNOWLEDGE) \
    EL(_SYNCHRONOUS_IDLE) \
    EL(_END_OF_TRANSMISSION_BLOCK) \
    EL(_CANCEL) \
    EL(_END_OF_MEDIUM) \
    EL(_SUBSTITUTE) \
    EL(_ESCAPE) \
    EL(_FILE_SEPARATOR) \
    EL(_GROUP_SEPARATOR) \
    EL(_RECORD_SEPARATOR) \
    EL(_UNIT_SEPARATOR) \
    EL(_SPACE) \
    EL(_EXCLAMATION_MARK) \
    EL(_QUOTATION_MARK) \
    EL(_NUMBER_SIGN) \
    EL(_DOLLAR_SIGN) \
    EL(_PERCENT_SIGN) \
    EL(_AMPERSAND) \
    EL(_APOSTROPHE) \
    EL(_LEFT_PARENTHESIS) \
    EL(_RIGHT_PARENTHESIS) \
    EL(_ASTERISK) \
    EL(_PLUS_SIGN) \
    EL(_COMMA) \
    EL(_HYPHEN_MINUS) \
    EL(_FULL_STOP) \
    EL(_SOLIDUS) \
    EL(_DIGIT_ZERO) \
    EL(_DIGIT_ONE) \
    EL(_DIGIT_TWO) \
    EL(_DIGIT_THREE) \
    EL(_DIGIT_FOUR) \
    EL(_DIGIT_FIVE) \
    EL(_DIGIT_SIX) \
    EL(_DIGIT_SEVEN) \
    EL(_DIGIT_EIGHT) \
    EL(_DIGIT_NINE) \
    EL(_COLON) \
    EL(_SEMICOLON) \
    EL(_LESS_THAN_SIGN) \
    EL(_EQUALS_SIGN) \
    EL(_GREATER_THAN_SIGN) \
    EL(_QUESTION_MARK) \
    EL(_COMMERCIAL_AT) \
    EL(_LATIN_CAPITAL_LETTER_A) \
    EL(_LATIN_CAPITAL_LETTER_B) \
    EL(_LATIN_CAPITAL_LETTER_C) \
    EL(_LATIN_CAPITAL_LETTER_D) \
    EL(_LATIN_CAPITAL_LETTER_E) \
    EL(_LATIN_CAPITAL_LETTER_F) \
    EL(_LATIN_CAPITAL_LETTER_G) \
    EL(_LATIN_CAPITAL_LETTER_H) \
    EL(_LATIN_CAPITAL_LETTER_I) \
    EL(_LATIN_CAPITAL_LETTER_J) \
    EL(_LATIN_CAPITAL_LETTER_K) \
    EL(_LATIN_CAPITAL_LETTER_L) \
    EL(_LATIN_CAPITAL_LETTER_M) \
    EL(_LATIN_CAPITAL_LETTER_N) \
    EL(_LATIN_CAPITAL_LETTER_O) \
    EL(_LATIN_CAPITAL_LETTER_P) \
    EL(_LATIN_CAPITAL_LETTER_Q) \
    EL(_LATIN_CAPITAL_LETTER_R) \
    EL(_LATIN_CAPITAL_LETTER_S) \
    EL(_LATIN_CAPITAL_LETTER_T) \
    EL(_LATIN_CAPITAL_LETTER_U) \
    EL(_LATIN_CAPITAL_LETTER_V) \
    EL(_LATIN_CAPITAL_LETTER_W) \
    EL(_LATIN_CAPITAL_LETTER_X) \
    EL(_LATIN_CAPITAL_LETTER_Y) \
    EL(_LATIN_CAPITAL_LETTER_Z) \
    EL(_LEFT_SQUARE_BRACKET) \
    EL(_REVERSE_SOLIDUS) \
    EL(_RIGHT_SQUARE_BRACKET) \
    EL(_CIRCUMFLEX_ACCENT) \
    EL(_LOW_LINE) \
    EL(_GRAVE_ACCENT) \
    EL(_LATIN_SMALL_LETTER_A) \
    EL(_LATIN_SMALL_LETTER_B) \
    EL(_LATIN_SMALL_LETTER_C) \
    EL(_LATIN_SMALL_LETTER_D) \
    EL(_LATIN_SMALL_LETTER_E) \
    EL(_LATIN_SMALL_LETTER_F) \
    EL(_LATIN_SMALL_LETTER_G) \
    EL(_LATIN_SMALL_LETTER_H) \
    EL(_LATIN_SMALL_LETTER_I) \
    EL(_LATIN_SMALL_LETTER_J) \
    EL(_LATIN_SMALL_LETTER_K) \
    EL(_LATIN_SMALL_LETTER_L) \
    EL(_LATIN_SMALL_LETTER_M) \
    EL(_LATIN_SMALL_LETTER_N) \
    EL(_LATIN_SMALL_LETTER_O) \
    EL(_LATIN_SMALL_LETTER_P) \
    EL(_LATIN_SMALL_LETTER_Q) \
    EL(_LATIN_SMALL_LETTER_R) \
    EL(_LATIN_SMALL_LETTER_S) \
    EL(_LATIN_SMALL_LETTER_T) \
    EL(_LATIN_SMALL_LETTER_U) \
    EL(_LATIN_SMALL_LETTER_V) \
    EL(_LATIN_SMALL_LETTER_W) \
    EL(_LATIN_SMALL_LETTER_X) \
    EL(_LATIN_SMALL_LETTER_Y) \
    EL(_LATIN_SMALL_LETTER_Z) \
    EL(_LEFT_CURLY_BRACKET) \
    EL(_VERTICAL_LINE) \
    EL(_RIGHT_CURLY_BRACKET) \
    EL(_TILDE) \
    EL(_UNDEFINED_007F) \
    EL(_BOX_DRAWINGS_LIGHT_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL) \
    EL(_UPPER_HALF_BLOCK) \
    EL(_LOWER_HALF_BLOCK) \
    EL(_FULL_BLOCK) \
    EL(_LEFT_HALF_BLOCK) \
    EL(_RIGHT_HALF_BLOCK) \
    EL(_LIGHT_SHADE) \
    EL(_MEDIUM_SHADE) \
    EL(_DARK_SHADE) \
    EL(_TOP_HALF_INTEGRAL) \
    EL(_BLACK_SQUARE) \
    EL(_BULLET_OPERATOR) \
    EL(_SQUARE_ROOT) \
    EL(_ALMOST_EQUAL_TO) \
    EL(_LESS_THAN_OR_EQUAL_TO) \
    EL(_GREATER_THAN_OR_EQUAL_TO) \
    EL(_NO_BREAK_SPACE) \
    EL(_BOTTOM_HALF_INTEGRAL) \
    EL(_DEGREE_SIGN) \
    EL(_SUPERSCRIPT_TWO) \
    EL(_MIDDLE_DOT) \
    EL(_DIVISION_SIGN) \
    EL(_BOX_DRAWINGS_DOUBLE_HORIZONTAL) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL) \
    EL(_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_CYRILLIC_SMALL_LETTER_IO) \
    EL(_CYRILLIC_SMALL_LETTER_UKRAINIAN_IE) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT) \
    EL(_CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I) \
    EL(_CYRILLIC_SMALL_LETTER_YI) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE) \
    EL(_CYRILLIC_SMALL_LETTER_GHE_WITH_UPTURN) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE) \
    EL(_CYRILLIC_CAPITAL_LETTER_IO) \
    EL(_CYRILLIC_CAPITAL_LETTER_UKRAINIAN_IE) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT) \
    EL(_CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_YI) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE) \
    EL(_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE) \
    EL(_CYRILLIC_CAPITAL_LETTER_GHE_WITH_UPTURN) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL) \
    EL(_COPYRIGHT_SIGN) \
    EL(_CYRILLIC_SMALL_LETTER_YU) \
    EL(_CYRILLIC_SMALL_LETTER_A) \
    EL(_CYRILLIC_SMALL_LETTER_BE) \
    EL(_CYRILLIC_SMALL_LETTER_TSE) \
    EL(_CYRILLIC_SMALL_LETTER_DE) \
    EL(_CYRILLIC_SMALL_LETTER_IE) \
    EL(_CYRILLIC_SMALL_LETTER_EF) \
    EL(_CYRILLIC_SMALL_LETTER_GHE) \
    EL(_CYRILLIC_SMALL_LETTER_HA) \
    EL(_CYRILLIC_SMALL_LETTER_I) \
    EL(_CYRILLIC_SMALL_LETTER_SHORT_I) \
    EL(_CYRILLIC_SMALL_LETTER_KA) \
    EL(_CYRILLIC_SMALL_LETTER_EL) \
    EL(_CYRILLIC_SMALL_LETTER_EM) \
    EL(_CYRILLIC_SMALL_LETTER_EN) \
    EL(_CYRILLIC_SMALL_LETTER_O) \
    EL(_CYRILLIC_SMALL_LETTER_PE) \
    EL(_CYRILLIC_SMALL_LETTER_YA) \
    EL(_CYRILLIC_SMALL_LETTER_ER) \
    EL(_CYRILLIC_SMALL_LETTER_ES) \
    EL(_CYRILLIC_SMALL_LETTER_TE) \
    EL(_CYRILLIC_SMALL_LETTER_U) \
    EL(_CYRILLIC_SMALL_LETTER_ZHE) \
    EL(_CYRILLIC_SMALL_LETTER_VE) \
    EL(_CYRILLIC_SMALL_LETTER_SOFT_SIGN) \
    EL(_CYRILLIC_SMALL_LETTER_YERU) \
    EL(_CYRILLIC_SMALL_LETTER_ZE) \
    EL(_CYRILLIC_SMALL_LETTER_SHA) \
    EL(_CYRILLIC_SMALL_LETTER_E) \
    EL(_CYRILLIC_SMALL_LETTER_SHCHA) \
    EL(_CYRILLIC_SMALL_LETTER_CHE) \
    EL(_CYRILLIC_SMALL_LETTER_HARD_SIGN) \
    EL(_CYRILLIC_CAPITAL_LETTER_YU) \
    EL(_CYRILLIC_CAPITAL_LETTER_A) \
    EL(_CYRILLIC_CAPITAL_LETTER_BE) \
    EL(_CYRILLIC_CAPITAL_LETTER_TSE) \
    EL(_CYRILLIC_CAPITAL_LETTER_DE) \
    EL(_CYRILLIC_CAPITAL_LETTER_IE) \
    EL(_CYRILLIC_CAPITAL_LETTER_EF) \
    EL(_CYRILLIC_CAPITAL_LETTER_GHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_HA) \
    EL(_CYRILLIC_CAPITAL_LETTER_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHORT_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_KA) \
    EL(_CYRILLIC_CAPITAL_LETTER_EL) \
    EL(_CYRILLIC_CAPITAL_LETTER_EM) \
    EL(_CYRILLIC_CAPITAL_LETTER_EN) \
    EL(_CYRILLIC_CAPITAL_LETTER_O) \
    EL(_CYRILLIC_CAPITAL_LETTER_PE) \
    EL(_CYRILLIC_CAPITAL_LETTER_YA) \
    EL(_CYRILLIC_CAPITAL_LETTER_ER) \
    EL(_CYRILLIC_CAPITAL_LETTER_ES) \
    EL(_CYRILLIC_CAPITAL_LETTER_TE) \
    EL(_CYRILLIC_CAPITAL_LETTER_U) \
    EL(_CYRILLIC_CAPITAL_LETTER_ZHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_VE) \
    EL(_CYRILLIC_CAPITAL_LETTER_SOFT_SIGN) \
    EL(_CYRILLIC_CAPITAL_LETTER_YERU) \
    EL(_CYRILLIC_CAPITAL_LETTER_ZE) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHA) \
    EL(_CYRILLIC_CAPITAL_LETTER_E) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHCHA) \
    EL(_CYRILLIC_CAPITAL_LETTER_CHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_HARD_SIGN)

/* list of all characters NOT IDENTICALLY MAPPED to UTF-16 */
#define T_NLIST_KOI8_U(EL) \
    EL(_BOX_DRAWINGS_LIGHT_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT) \
    EL(_BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL) \
    EL(_UPPER_HALF_BLOCK) \
    EL(_LOWER_HALF_BLOCK) \
    EL(_FULL_BLOCK) \
    EL(_LEFT_HALF_BLOCK) \
    EL(_RIGHT_HALF_BLOCK) \
    EL(_LIGHT_SHADE) \
    EL(_MEDIUM_SHADE) \
    EL(_DARK_SHADE) \
    EL(_TOP_HALF_INTEGRAL) \
    EL(_BLACK_SQUARE) \
    EL(_BULLET_OPERATOR) \
    EL(_SQUARE_ROOT) \
    EL(_ALMOST_EQUAL_TO) \
    EL(_LESS_THAN_OR_EQUAL_TO) \
    EL(_GREATER_THAN_OR_EQUAL_TO) \
    EL(_NO_BREAK_SPACE) \
    EL(_BOTTOM_HALF_INTEGRAL) \
    EL(_DEGREE_SIGN) \
    EL(_SUPERSCRIPT_TWO) \
    EL(_MIDDLE_DOT) \
    EL(_DIVISION_SIGN) \
    EL(_BOX_DRAWINGS_DOUBLE_HORIZONTAL) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL) \
    EL(_BOX_DRAWINGS_DOWN_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_CYRILLIC_SMALL_LETTER_IO) \
    EL(_CYRILLIC_SMALL_LETTER_UKRAINIAN_IE) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_RIGHT) \
    EL(_CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I) \
    EL(_CYRILLIC_SMALL_LETTER_YI) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_LEFT) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_BOX_DRAWINGS_UP_DOUBLE_AND_RIGHT_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_RIGHT) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_LEFT_DOUBLE) \
    EL(_CYRILLIC_SMALL_LETTER_GHE_WITH_UPTURN) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_LEFT) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_RIGHT_DOUBLE) \
    EL(_BOX_DRAWINGS_VERTICAL_DOUBLE_AND_RIGHT_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_RIGHT) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_LEFT_DOUBLE) \
    EL(_CYRILLIC_CAPITAL_LETTER_IO) \
    EL(_CYRILLIC_CAPITAL_LETTER_UKRAINIAN_IE) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_LEFT) \
    EL(_CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_YI) \
    EL(_BOX_DRAWINGS_DOUBLE_DOWN_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_UP_SINGLE_AND_HORIZONTAL_DOUBLE) \
    EL(_BOX_DRAWINGS_UP_DOUBLE_AND_HORIZONTAL_SINGLE) \
    EL(_BOX_DRAWINGS_DOUBLE_UP_AND_HORIZONTAL) \
    EL(_BOX_DRAWINGS_VERTICAL_SINGLE_AND_HORIZONTAL_DOUBLE) \
    EL(_CYRILLIC_CAPITAL_LETTER_GHE_WITH_UPTURN) \
    EL(_BOX_DRAWINGS_DOUBLE_VERTICAL_AND_HORIZONTAL) \
    EL(_COPYRIGHT_SIGN) \
    EL(_CYRILLIC_SMALL_LETTER_YU) \
    EL(_CYRILLIC_SMALL_LETTER_A) \
    EL(_CYRILLIC_SMALL_LETTER_BE) \
    EL(_CYRILLIC_SMALL_LETTER_TSE) \
    EL(_CYRILLIC_SMALL_LETTER_DE) \
    EL(_CYRILLIC_SMALL_LETTER_IE) \
    EL(_CYRILLIC_SMALL_LETTER_EF) \
    EL(_CYRILLIC_SMALL_LETTER_GHE) \
    EL(_CYRILLIC_SMALL_LETTER_HA) \
    EL(_CYRILLIC_SMALL_LETTER_I) \
    EL(_CYRILLIC_SMALL_LETTER_SHORT_I) \
    EL(_CYRILLIC_SMALL_LETTER_KA) \
    EL(_CYRILLIC_SMALL_LETTER_EL) \
    EL(_CYRILLIC_SMALL_LETTER_EM) \
    EL(_CYRILLIC_SMALL_LETTER_EN) \
    EL(_CYRILLIC_SMALL_LETTER_O) \
    EL(_CYRILLIC_SMALL_LETTER_PE) \
    EL(_CYRILLIC_SMALL_LETTER_YA) \
    EL(_CYRILLIC_SMALL_LETTER_ER) \
    EL(_CYRILLIC_SMALL_LETTER_ES) \
    EL(_CYRILLIC_SMALL_LETTER_TE) \
    EL(_CYRILLIC_SMALL_LETTER_U) \
    EL(_CYRILLIC_SMALL_LETTER_ZHE) \
    EL(_CYRILLIC_SMALL_LETTER_VE) \
    EL(_CYRILLIC_SMALL_LETTER_SOFT_SIGN) \
    EL(_CYRILLIC_SMALL_LETTER_YERU) \
    EL(_CYRILLIC_SMALL_LETTER_ZE) \
    EL(_CYRILLIC_SMALL_LETTER_SHA) \
    EL(_CYRILLIC_SMALL_LETTER_E) \
    EL(_CYRILLIC_SMALL_LETTER_SHCHA) \
    EL(_CYRILLIC_SMALL_LETTER_CHE) \
    EL(_CYRILLIC_SMALL_LETTER_HARD_SIGN) \
    EL(_CYRILLIC_CAPITAL_LETTER_YU) \
    EL(_CYRILLIC_CAPITAL_LETTER_A) \
    EL(_CYRILLIC_CAPITAL_LETTER_BE) \
    EL(_CYRILLIC_CAPITAL_LETTER_TSE) \
    EL(_CYRILLIC_CAPITAL_LETTER_DE) \
    EL(_CYRILLIC_CAPITAL_LETTER_IE) \
    EL(_CYRILLIC_CAPITAL_LETTER_EF) \
    EL(_CYRILLIC_CAPITAL_LETTER_GHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_HA) \
    EL(_CYRILLIC_CAPITAL_LETTER_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHORT_I) \
    EL(_CYRILLIC_CAPITAL_LETTER_KA) \
    EL(_CYRILLIC_CAPITAL_LETTER_EL) \
    EL(_CYRILLIC_CAPITAL_LETTER_EM) \
    EL(_CYRILLIC_CAPITAL_LETTER_EN) \
    EL(_CYRILLIC_CAPITAL_LETTER_O) \
    EL(_CYRILLIC_CAPITAL_LETTER_PE) \
    EL(_CYRILLIC_CAPITAL_LETTER_YA) \
    EL(_CYRILLIC_CAPITAL_LETTER_ER) \
    EL(_CYRILLIC_CAPITAL_LETTER_ES) \
    EL(_CYRILLIC_CAPITAL_LETTER_TE) \
    EL(_CYRILLIC_CAPITAL_LETTER_U) \
    EL(_CYRILLIC_CAPITAL_LETTER_ZHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_VE) \
    EL(_CYRILLIC_CAPITAL_LETTER_SOFT_SIGN) \
    EL(_CYRILLIC_CAPITAL_LETTER_YERU) \
    EL(_CYRILLIC_CAPITAL_LETTER_ZE) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHA) \
    EL(_CYRILLIC_CAPITAL_LETTER_E) \
    EL(_CYRILLIC_CAPITAL_LETTER_SHCHA) \
    EL(_CYRILLIC_CAPITAL_LETTER_CHE) \
    EL(_CYRILLIC_CAPITAL_LETTER_HARD_SIGN)

#endif /* _TUTF_KOI8_U_H */
