




/* This file was automatically generated from m4/tt.m4hh, do not edit! */


/*
 *  tttextfield.h  --  public header for libTT declarations related to "tttextfield"
 *
 *  Copyright (C) 2001-2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */



#ifndef _TT_TTTEXTFIELD_H
#define _TT_TTTEXTFIELD_H







/* object structure */
#ifndef s_tttextfield


/** tttextfield: a single line user editable text-only widget */
        struct s_tttextfield;
typedef struct s_tttextfield *	tttextfield;

#endif /* s_tttextfield */



/* global runtime class type; returned by TTCLASSOF(<some object>) */
/** class of tttextfield */
extern ttclass	TTClass_tttextfield;





/* tttextfield methods */









/* TTGet*_tttextfield and TTSet*_tttextfield methods */










#endif /* _TT_TTTEXTFIELD_H */

