
#line 3 "../../twin-0.5.1+2/server/rcparse_lex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 31
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart(yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart (FILE *input_file  );
void yy_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE yy_create_buffer (FILE *file,int size  );
void yy_delete_buffer (YY_BUFFER_STATE b  );
void yy_flush_buffer (YY_BUFFER_STATE b  );
void yypush_buffer_state (YY_BUFFER_STATE new_buffer  );
void yypop_buffer_state (void );

static void yyensure_buffer_stack (void );
static void yy_load_buffer_state (void );
static void yy_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE yy_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE yy_scan_bytes (yyconst char *bytes,int len  );

void *yyalloc (yy_size_t  );
void *yyrealloc (void *,yy_size_t  );
void yyfree (void *  );

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;

typedef int yy_state_type;

extern int yylineno;

int yylineno = 1;

extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 90
#define YY_END_OF_BUFFER 91
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[461] =
    {   0,
        0,    0,   91,   89,   88,   79,   87,   81,   84,   83,
       83,   80,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   88,   87,   86,   87,   81,   83,
       85,   85,   83,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   50,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   87,   86,   87,   83,

       82,   82,   67,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   15,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   44,
       51,   82,   82,   82,   82,   82,   74,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       37,   82,   82,   71,   78,   82,   82,   82,   82,   68,
       82,   73,   82,   82,   11,   82,   82,   82,   82,   82,
       82,   82,   41,   55,   82,   82,   82,   16,   82,   82,

       19,   21,   82,   22,   45,   82,   23,   82,   82,   82,
       82,   36,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   33,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   70,   61,   82,   82,   82,   39,   82,   82,
       82,   82,   35,   82,   82,   72,   82,   82,   82,   42,
       82,   82,   82,   82,   82,   82,   17,   82,   82,   46,
       82,   82,   82,   56,   82,   82,   82,   82,   82,   69,
       31,   82,   82,   82,   77,   82,   82,   53,   82,   82,
       82,   82,   82,   82,    5,    6,   38,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,

       82,   18,   82,   82,   82,   82,   25,   82,   27,   28,
       82,   82,   82,   30,   82,   52,   34,   76,   82,   82,
       82,   60,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   12,   82,   82,   82,   82,   82,   75,   82,
       82,   82,   82,   82,   82,   82,   48,   82,   24,   82,
       82,   82,   59,   82,   82,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
       63,   54,   82,   43,   64,   82,   82,   82,   82,   82,
       82,   82,   82,   82,   82,    1,    2,    3,   82,   82,
       82,   82,   82,   82,   82,   82,   82,   82,   82,   82,

       82,   65,   82,   82,   82,   82,   82,   82,   82,   82,
       82,   82,    4,   82,   82,   82,   82,    7,    8,   82,
       66,   40,   82,   82,   20,   82,   47,   82,   82,   82,
       82,   82,   49,   82,   57,   82,   82,   82,   82,   13,
       14,   82,   82,   82,   82,   82,   62,   82,    9,   10,
       26,   82,   29,   32,   82,   82,   82,   82,   58,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        2,    2,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    5,    1,    1,    1,    1,    6,
        6,    1,    6,    1,    7,    1,    1,    8,    9,    9,
        9,    9,    9,    9,    9,   10,   10,    1,   11,    1,
        1,    1,    1,    1,   12,   13,   14,   15,   16,   17,
       18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
       28,   29,   30,   31,   21,   21,   32,   33,   34,   21,
        1,   35,    1,    1,   21,    1,   36,   37,   38,   39,

       40,   41,   42,   43,   44,   21,   45,   46,   47,   48,
       49,   50,   21,   51,   52,   53,   54,   55,   56,   57,
       58,   59,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[60] =
    {   0,
        1,    1,    2,    1,    1,    1,    3,    4,    4,    4,
        1,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    1,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4
    } ;

static yyconst flex_int16_t yy_base[467] =
    {   0,
        0,    0,  968,  969,  965,  969,   56,    0,  969,   53,
       55,  969,   59,   60,   62,   61,   64,   63,   67,   75,
       74,  959,   83,   85,   88,   86,   92,  100,   76,  122,
      132,   82,  125,   71,  963,   69,  969,  138,    0,   67,
        0,  183,  135,    0,  957,  131,  140,  148,  139,  141,
      158,  157,  152,  167,  160,  156,  175,  147,  142,  145,
      146,  173,  143,  169,  199,  203,  207,  206,  209,  210,
      218,  220,  221,  224,  223,  222,  225,  956,  229,  227,
      226,  231,  246,  234,  237,  233,  240,  258,  228,  250,
      242,  249,  251,  252,  245,  254,  269,  288,  302,    0,

      264,  290,   70,  292,  294,  296,  300,  302,  303,  305,
      306,  304,  307,  309,  308,  310,  311,  312,  315,  318,
      317,  319,  326,  324,  327,  323,  320,  328,  955,  329,
      325,  333,  334,  335,  322,  340,  341,  342,  344,  954,
      953,  346,  362,  363,  370,  365,  952,  373,  374,  376,
      377,  380,  381,  389,  391,  384,  393,  394,  378,  395,
      379,  382,  399,  396,  385,  388,  398,  400,  383,  401,
      951,  403,  405,  950,  949,  404,  402,  406,  429,  948,
      407,  947,  408,  432,  445,  438,  447,  446,  448,  230,
      450,  451,  946,  945,  453,  452,  458,   65,  460,  457,

      456,  944,  459,  943,  942,  461,  941,  467,  463,  472,
      464,  940,  471,  466,  476,  478,  474,  465,  477,  468,
      473,  480,  939,  481,  482,  484,  485,  486,  517,  487,
      488,  491,  938,  937,  503,  508,  506,  936,  513,  521,
      522,  523,  935,  525,  529,  934,  531,  536,  532,  933,
      537,  538,  539,  540,  541,  544,  932,  548,  545,  542,
      543,  552,  546,  931,  555,  559,  560,  562,  563,  930,
      929,  565,  566,  567,  928,  570,  571,  927,  572,  578,
      580,  582,  584,  587,  926,  594,  925,  592,  601,  586,
      589,  593,  616,  595,  602,  603,  610,  608,  615,  621,

      622,  924,  623,  627,  631,  632,  633,  635,  641,  923,
      638,  642,  643,  922,  644,  921,  646,  920,  645,  648,
      651,  919,  649,  652,  653,  654,  658,  657,  659,  669,
      663,  666,  918,  670,  671,  672,  676,  677,  917,  679,
      680,  683,  685,  684,  686,  691,  916,  693,  905,  695,
      700,  698,  898,  701,  707,  705,  716,  711,  715,  720,
      718,  721,  722,  727,  728,  731,  730,  736,  733,  735,
      893,  891,  739,  890,  889,  734,  737,  741,  745,  748,
      749,  750,  751,  752,  743,  888,  886,  884,  753,  754,
      755,  756,  765,  757,  768,  762,  767,  777,  779,  780,

      781,  882,  782,  785,  790,  797,  796,  802,  804,  806,
      805,  807,  881,  810,  811,  812,  813,  875,  873,  814,
      871,  869,  817,  819,  868,  818,  867,  824,  822,  825,
      831,  832,  866,  826,  860,  828,  833,  835,  836,  854,
      243,  839,  840,  841,  844,  845,  241,  846,  239,  153,
      150,  848,  149,   81,  850,  853,  856,  858,   73,  969,
      906,  910,   75,  912,  916,  918
    } ;

static yyconst flex_int16_t yy_def[467] =
    {   0,
      460,    1,  460,  460,  460,  460,  461,  462,  460,  463,
      463,  460,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  460,  461,  460,  465,  462,  463,
      463,  463,  463,  466,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  461,  461,  465,   42,

      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,

      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,

      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,

      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,  464,
      464,  464,  464,  464,  464,  464,  464,  464,  464,    0,
      460,  460,  460,  460,  460,  460
    } ;

static yyconst flex_int16_t yy_nxt[1029] =
    {   0,
        4,    5,    6,    7,    8,    9,    9,   10,   11,   11,
       12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
       22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
       32,   33,   22,   34,    4,   22,   22,   22,   22,   22,
       22,   22,   22,   22,   22,   22,   22,   22,   22,   22,
       22,   22,   22,   22,   22,   22,   22,   22,   22,   37,
       40,   40,   43,   43,   43,   44,   44,   44,   44,   44,
       44,   44,   37,   44,   40,   40,   44,   44,   41,   44,
       44,   44,   44,  253,  254,   42,  168,   44,   44,   44,
       38,   44,   44,  255,   44,   49,   46,   47,   44,   50,

       59,   54,   60,   38,   48,   51,   44,   55,   52,   42,
       96,   62,   64,   53,   56,   57,   63,   65,   66,   58,
       61,   67,   68,   72,   70,   75,   69,   73,   44,   81,
       92,   44,   77,   71,   76,   79,   74,   44,   44,   78,
       36,   98,   43,   43,   43,   44,   44,   44,   44,   44,
       80,   44,   44,   44,   44,   44,   44,   82,   44,   44,
       93,   83,   44,   44,   44,   84,   44,   94,   95,   86,
       85,   87,   99,   44,   88,   44,  105,   89,  102,   44,
      106,   44,  117,  101,   90,  120,  119,  118,  122,   91,
      100,  100,  100,  107,  100,  100,  100,  100,  100,  100,

      103,  108,  110,  104,  112,   44,  115,  111,  114,   44,
      121,  109,   44,   44,  113,   44,   44,  123,  100,  100,
      100,  100,  100,  100,   44,  116,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,  124,   44,
       44,  125,  127,   44,  126,   44,   44,   44,   44,   44,
      131,   44,   44,  247,  130,   44,   44,   44,   44,  128,
       44,  133,  136,  129,   44,  141,  143,  157,  135,  144,
       44,  140,   37,  132,  145,  150,  134,  137,  138,  139,
      142,  146,  151,  152,  147,  153,  148,  154,  158,  159,
      160,   37,  163,  155,  161,  162,   44,  149,   44,  164,

       44,  156,   44,   38,   36,   98,   44,  165,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,  166,
      167,   44,   38,   44,   44,   44,   44,  169,   44,   44,
       44,   44,   44,   44,   44,   44,   99,  172,  170,   44,
       44,   44,  174,  175,  176,  171,   44,   44,   44,  173,
       44,  183,   44,  180,  184,  185,  177,  191,  182,  178,
      179,  181,  188,  189,  187,  175,  190,  192,   44,   44,
      186,   44,  195,  196,  193,  198,   44,  194,  197,   44,
       44,  201,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,  200,  199,   44,   44,  202,   44,  203,   44,

       44,   44,   44,  207,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,  205,  204,  209,  211,  213,
      215,  206,  212,  208,  218,  228,  210,  216,  214,  217,
      221,  223,  219,  220,  224,   44,  222,  225,   44,  227,
      231,  226,  232,  235,   44,  237,  229,  233,  230,  234,
      236,   44,   44,   44,   44,  239,   44,   44,   44,   44,
      240,  241,   44,   44,   44,   44,   44,   44,  238,   44,
       44,   44,   44,   44,   44,  242,  244,   44,   44,   44,
       44,  245,   44,   44,   44,  258,   44,   44,   44,  243,
       44,   44,   44,   44,   44,  246,  257,   44,  259,  251,

      260,  249,  248,  250,  252,  256,  261,  263,  268,   44,
      265,  266,   44,  267,   44,  273,  264,  270,  272,   44,
      275,  262,  269,   44,  278,  274,  271,   44,   44,   44,
      276,   44,  277,  280,  282,   44,  279,   44,   44,  283,
      281,  284,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,  285,   44,  286,  287,  304,   44,  290,
      289,   44,  292,  288,  305,   44,   44,  296,   44,   44,
      294,   44,   44,   44,  293,  291,   44,   44,   44,  295,
      301,  298,  299,  302,   44,  303,   44,  300,   44,  297,
       44,  307,   44,   44,  306,   44,  308,  323,   44,   44,

       44,   44,  309,  327,  310,  315,  316,   44,   44,   44,
      312,  319,  311,  328,   44,  314,   44,  329,  313,  321,
      325,   44,   44,  326,  330,  317,  318,   44,   44,   44,
      331,  320,  335,   44,  322,  324,  332,   44,   44,   44,
      338,   44,  336,  334,   44,  339,  333,   44,   44,   44,
       44,   44,   44,  341,   44,   44,  337,   44,   44,   44,
       44,  342,  348,   44,   44,   44,  340,  343,  355,   44,
      350,  352,   44,  344,  347,   44,   44,   44,   44,  346,
      345,  351,   44,   44,  356,   44,   44,  349,  361,   44,
       44,   44,   44,  362,  353,  357,  354,   44,  358,   44,

      367,   44,  359,  363,   44,  360,   44,   44,  366,  369,
      364,   44,  365,   44,  368,  372,  370,   44,  374,  371,
      376,   44,   44,  377,   44,  371,   44,   44,   44,  373,
      380,  375,  381,   44,   44,  383,   44,   44,  378,   44,
       44,   44,   44,   44,  384,   44,  379,   44,  382,   44,
      385,   44,  386,  387,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,  388,  389,  392,  390,   44,  391,
      400,   44,  399,   44,   44,  395,  403,  393,  396,  394,
      397,  398,  401,   44,  405,   44,   44,   44,   44,  410,
      402,   44,  413,  404,  411,  415,   44,  408,  406,  407,

      416,  409,   44,   44,  412,  418,  420,  414,   44,  417,
       44,   44,   44,   44,  430,  419,   44,   44,   44,   44,
       44,  423,  421,   44,   44,   44,  422,  432,   44,  425,
       44,   44,   44,  426,   44,  424,  428,   44,   44,   44,
      427,   44,   44,  431,  429,   44,   44,   44,  436,  435,
       44,   44,   44,  439,   44,  434,   44,  433,  441,   44,
       44,  438,   44,  442,   44,  435,   44,  443,  440,  437,
      445,  446,   44,   44,   44,   44,  447,   44,  444,   44,
      448,   44,  449,  450,  447,  421,  451,   44,   44,  455,
       44,  453,   44,  452,   44,   44,   44,   44,  457,   44,

      456,  458,  454,  459,   44,  459,   36,   36,   36,   36,
       39,   44,   39,   39,   45,   45,   97,   97,   97,   97,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   35,   44,   35,  460,    3,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,

      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460
    } ;

static yyconst flex_int16_t yy_chk[1029] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    7,
       10,   10,   11,   11,   11,   13,   14,   16,   15,   18,
       17,  198,   36,   19,   40,   40,  103,   34,  463,  459,
       21,   20,   29,  198,  198,   10,  103,  454,   32,   23,
        7,   24,   26,  198,   25,   14,   13,   13,   27,   14,

       16,   15,   17,   36,   13,   14,   28,   15,   14,   10,
       34,   18,   19,   14,   15,   15,   18,   19,   20,   15,
       17,   21,   23,   25,   24,   26,   23,   25,   30,   29,
       32,   33,   27,   24,   26,   28,   25,   46,   31,   27,
       38,   38,   43,   43,   43,   49,   47,   50,   59,   63,
       28,   60,   61,   58,   48,  453,  451,   30,   53,  450,
       33,   30,   56,   52,   51,   30,   55,   33,   33,   31,
       30,   31,   38,   54,   31,   64,   49,   31,   47,   62,
       50,   57,   58,   46,   31,   61,   60,   59,   63,   31,
       42,   42,   42,   51,   42,   42,   42,   42,   42,   42,

       48,   51,   52,   48,   53,   65,   56,   52,   55,   66,
       62,   51,   68,   67,   54,   69,   70,   64,   42,   42,
       42,   42,   42,   42,   71,   57,   72,   73,   76,   75,
       74,   77,   81,   80,   89,   79,  190,   82,   65,   86,
       84,   66,   67,   85,   66,  449,   87,  447,   91,  441,
       70,   95,   83,  190,   69,   92,   90,   93,   94,   67,
       96,   72,   74,   68,   88,   77,   80,   89,   73,   81,
      101,   76,   97,   71,   82,   84,   72,   74,   74,   75,
       79,   83,   85,   86,   83,   87,   83,   87,   90,   91,
       92,   98,   95,   88,   93,   94,  102,   83,  104,   96,

      105,   88,  106,   97,   99,   99,  107,  101,  108,  109,
      112,  110,  111,  113,  115,  114,  116,  117,  118,  102,
      102,  119,   98,  121,  120,  122,  127,  104,  135,  126,
      124,  131,  123,  125,  128,  130,   99,  107,  105,  132,
      133,  134,  109,  110,  111,  106,  136,  137,  138,  108,
      139,  118,  142,  115,  119,  120,  112,  127,  117,  113,
      114,  116,  123,  124,  122,  126,  125,  128,  143,  144,
      121,  146,  132,  133,  130,  135,  145,  131,  134,  148,
      149,  138,  150,  151,  159,  161,  152,  153,  162,  169,
      156,  165,  137,  136,  166,  154,  139,  155,  142,  157,

      158,  160,  164,  146,  167,  163,  168,  170,  177,  172,
      176,  173,  178,  181,  183,  144,  143,  149,  150,  152,
      153,  145,  151,  148,  156,  166,  149,  154,  152,  155,
      159,  161,  157,  158,  162,  179,  160,  163,  184,  165,
      169,  164,  170,  176,  186,  178,  167,  172,  168,  173,
      177,  185,  188,  187,  189,  181,  191,  192,  196,  195,
      183,  184,  201,  200,  197,  203,  199,  206,  179,  209,
      211,  218,  214,  208,  220,  185,  187,  213,  210,  221,
      217,  188,  215,  219,  216,  201,  222,  224,  225,  186,
      226,  227,  228,  230,  231,  189,  200,  232,  203,  196,

      206,  192,  191,  195,  197,  199,  208,  210,  216,  235,
      213,  214,  237,  215,  236,  221,  211,  218,  220,  239,
      224,  209,  217,  229,  227,  222,  219,  240,  241,  242,
      225,  244,  226,  229,  230,  245,  228,  247,  249,  231,
      229,  232,  248,  251,  252,  253,  254,  255,  260,  261,
      256,  259,  263,  235,  258,  236,  237,  259,  262,  241,
      240,  265,  244,  239,  260,  266,  267,  249,  268,  269,
      247,  272,  273,  274,  245,  242,  276,  277,  279,  248,
      255,  252,  253,  256,  280,  258,  281,  254,  282,  251,
      283,  262,  290,  284,  261,  291,  263,  283,  288,  292,

      286,  294,  265,  288,  266,  273,  274,  289,  295,  296,
      268,  279,  267,  289,  298,  272,  297,  289,  269,  281,
      286,  299,  293,  286,  289,  276,  277,  300,  301,  303,
      289,  280,  293,  304,  282,  284,  290,  305,  306,  307,
      296,  308,  294,  292,  311,  297,  291,  309,  312,  313,
      315,  319,  317,  299,  320,  323,  295,  321,  324,  325,
      326,  300,  307,  328,  327,  329,  298,  301,  317,  331,
      309,  312,  332,  303,  306,  330,  334,  335,  336,  305,
      304,  311,  337,  338,  319,  340,  341,  308,  325,  342,
      344,  343,  345,  326,  313,  320,  315,  346,  321,  348,

      331,  350,  323,  327,  352,  324,  351,  354,  330,  334,
      328,  356,  329,  355,  332,  337,  335,  358,  340,  341,
      343,  359,  357,  344,  361,  336,  360,  362,  363,  338,
      348,  342,  350,  364,  365,  352,  367,  366,  345,  369,
      376,  370,  368,  377,  354,  373,  346,  378,  351,  385,
      355,  379,  356,  357,  380,  381,  382,  383,  384,  389,
      390,  391,  392,  394,  358,  359,  362,  360,  396,  361,
      370,  393,  369,  397,  395,  365,  377,  363,  366,  364,
      367,  368,  373,  398,  379,  399,  400,  401,  403,  384,
      376,  404,  390,  378,  385,  392,  405,  382,  380,  381,

      393,  383,  407,  406,  389,  395,  397,  391,  408,  394,
      409,  411,  410,  412,  408,  396,  414,  415,  416,  417,
      420,  400,  398,  423,  426,  424,  399,  410,  429,  403,
      428,  430,  434,  404,  436,  401,  406,  431,  432,  437,
      405,  438,  439,  409,  407,  442,  443,  444,  415,  414,
      445,  446,  448,  420,  452,  412,  455,  411,  424,  456,
      440,  417,  457,  428,  458,  426,  435,  429,  423,  416,
      431,  432,  433,  427,  425,  422,  434,  421,  430,  419,
      436,  418,  438,  439,  437,  443,  442,  413,  402,  448,
      388,  445,  387,  444,  386,  375,  374,  372,  455,  371,

      452,  456,  446,  457,  353,  458,  461,  461,  461,  461,
      462,  349,  462,  462,  464,  464,  465,  465,  465,  465,
      466,  466,  347,  339,  333,  322,  318,  316,  314,  310,
      302,  287,  285,  278,  275,  271,  270,  264,  257,  250,
      246,  243,  238,  234,  233,  223,  212,  207,  205,  204,
      202,  194,  193,  182,  180,  175,  174,  171,  147,  141,
      140,  129,   78,   45,   35,   22,    5,    3,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,

      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460,  460,  460,
      460,  460,  460,  460,  460,  460,  460,  460
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
/* -*-c-*-
 *
 *  rcparse.l  --  lex back-end of ~/.twinrc parser for twin
 *
 *  Copyright (C) 2000-2001 by Massimiliano Ghilardi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 */

#include "twin.h"
#include "rctypes.h"
#include "rcparse_tab.h"

#define IS_OCTDIGIT(c) ((c) >= '0' && (c) <= '7')
#define IS_HEXDIGIT(c) (((c) >= '0' && (c) <= '9') || ((c) >= 'A' && (c) <= 'F') || ((c) >= 'a' && (c) <= 'f'))
#define HEX2INT(c)     ((c) >= 'a' ? (c) - 'a' + 10 : (c) >= 'A' ? (c) - 'A' + 10 : (c) - '0')

#define MAX_READ_DEPTH 64
static YY_BUFFER_STATE read_stack[MAX_READ_DEPTH];
int read_stack_curr = 0;
byte *file_name[MAX_READ_DEPTH];
int line_no[MAX_READ_DEPTH];

#define LINE_NO line_no[read_stack_curr-1]
#define FILE_NAME file_name[read_stack_curr-1]
    
static int yywrap(void);

static char errbuf[256];

/* twin.h typedefs `msg'... avoid it */
#define msg Msg

#ifdef PURE_PARSER
# define YY_DECL int yylex(YYSTYPE *_yylval,void *yylloc)
# define yylval (*_yylval)
#endif
    
#define YY_SKIP_YYWRAP
#define YY_NO_SCAN_BYTES
#define YY_NO_SCAN_STRING
#define YY_NO_UNPUT
#define YY_NO_SCAN_BUFFER
/*#undef  YY_STACK_USED*/

#define YY_USE_PROTOS

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap (void );
#else
extern int yywrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
	if ( (yy_init) )
		{
		(yy_init) = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer(yyin,YY_BUF_SIZE );
		}

		yy_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 461 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_current_state != 460 );
		yy_cp = (yy_last_accepting_cpos);
		yy_current_state = (yy_last_accepting_state);

yy_find_action:
		yy_act = yy_accept[yy_current_state];

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
return ADDSCREEN;
	YY_BREAK
case 2:
YY_RULE_SETUP
return ADDTOFUNC;
	YY_BREAK
case 3:
YY_RULE_SETUP
return ADDTOMENU;
	YY_BREAK
case 4:
YY_RULE_SETUP
return BACKGROUND;
	YY_BREAK
case 5:
YY_RULE_SETUP
return BORDER;
	YY_BREAK
case 6:
YY_RULE_SETUP
return BUTTON;
	YY_BREAK
case 7:
YY_RULE_SETUP
return DELETEFUNC;
	YY_BREAK
case 8:
YY_RULE_SETUP
return DELETEMENU;
	YY_BREAK
case 9:
YY_RULE_SETUP
return DELETEBUTTON;
	YY_BREAK
case 10:
YY_RULE_SETUP
return DELETESCREEN;
	YY_BREAK
case 11:
YY_RULE_SETUP
return EXEC;
	YY_BREAK
case 12:
YY_RULE_SETUP
return EXECTTY;
	YY_BREAK
case 13:
YY_RULE_SETUP
return GLOBALFLAGS;
	YY_BREAK
case 14:
YY_RULE_SETUP
return INTERACTIVE;
	YY_BREAK
case 15:
YY_RULE_SETUP
return KEY;
	YY_BREAK
case 16:
YY_RULE_SETUP
return MENU;
	YY_BREAK
case 17:
YY_RULE_SETUP
return MOUSE;
	YY_BREAK
case 18:
YY_RULE_SETUP
return MODULE;
	YY_BREAK
case 19:
YY_RULE_SETUP
return MOVE;
	YY_BREAK
case 20:
YY_RULE_SETUP
return MOVESCREEN;
	YY_BREAK
case 21:
YY_RULE_SETUP
return NEXT;
	YY_BREAK
case 22:
YY_RULE_SETUP
return PREV;
	YY_BREAK
case 23:
YY_RULE_SETUP
return READ;
	YY_BREAK
case 24:
YY_RULE_SETUP
return RESTART;
	YY_BREAK
case 25:
YY_RULE_SETUP
return RESIZE;
	YY_BREAK
case 26:
YY_RULE_SETUP
return RESIZESCREEN;
	YY_BREAK
case 27:
YY_RULE_SETUP
return SCREEN;
	YY_BREAK
case 28:
YY_RULE_SETUP
return SCROLL;
	YY_BREAK
case 29:
YY_RULE_SETUP
return SENDTOSCREEN;
	YY_BREAK
case 30:
YY_RULE_SETUP
return STDERR;
	YY_BREAK
case 31:
YY_RULE_SETUP
return SLEEP;
	YY_BREAK
case 32:
YY_RULE_SETUP
return SYNTHETICKEY;
	YY_BREAK
case 33:
YY_RULE_SETUP
return WAIT;
	YY_BREAK
case 34:
YY_RULE_SETUP
return WINDOW;
	YY_BREAK
case 35:
YY_RULE_SETUP
{ yylval.val = FOCUS;      return FLAG_FUNC; }
	YY_BREAK
case 36:
YY_RULE_SETUP
{ yylval.val = ROLL;       return FLAG_FUNC; }
	YY_BREAK
case 37:
YY_RULE_SETUP
{ yylval.val = BEEP;       return EASY_FUNC; }
	YY_BREAK
case 38:
YY_RULE_SETUP
{ yylval.val = CENTER;     return EASY_FUNC; }
	YY_BREAK
case 39:
YY_RULE_SETUP
{ yylval.val = CLOSE;      return EASY_FUNC; }
	YY_BREAK
case 40:
YY_RULE_SETUP
{ yylval.val = FULLSCREEN; return EASY_FUNC; }
	YY_BREAK
case 41:
YY_RULE_SETUP
{ yylval.val = KILL;       return EASY_FUNC; }
	YY_BREAK
case 42:
YY_RULE_SETUP
{ yylval.val = LOWER;      return EASY_FUNC; }
	YY_BREAK
case 43:
YY_RULE_SETUP
{ yylval.val = MAXIMIZE;   return EASY_FUNC; }
	YY_BREAK
case 44:
YY_RULE_SETUP
{ yylval.val = NOP;	   return EASY_FUNC; }
	YY_BREAK
case 45:
YY_RULE_SETUP
{ yylval.val = QUIT;       return EASY_FUNC; }
	YY_BREAK
case 46:
YY_RULE_SETUP
{ yylval.val = RAISE;      return EASY_FUNC; }
	YY_BREAK
case 47:
YY_RULE_SETUP
{ yylval.val = RAISELOWER; return EASY_FUNC; }
	YY_BREAK
case 48:
YY_RULE_SETUP
{ yylval.val = REFRESH;    return EASY_FUNC; }
	YY_BREAK
case 49:
YY_RULE_SETUP
{ yylval.val = WINDOWLIST; return EASY_FUNC; }
	YY_BREAK
case 50:
YY_RULE_SETUP
return FL_ON;
	YY_BREAK
case 51:
YY_RULE_SETUP
return FL_OFF;
	YY_BREAK
case 52:
YY_RULE_SETUP
return FL_TOGGLE;
	YY_BREAK
case 53:
YY_RULE_SETUP
return FL_ACTIVE;
	YY_BREAK
case 54:
YY_RULE_SETUP
return FL_INACTIVE;
	YY_BREAK
case 55:
YY_RULE_SETUP
return FL_LEFT;
	YY_BREAK
case 56:
YY_RULE_SETUP
return FL_RIGHT;
	YY_BREAK
case 57:
YY_RULE_SETUP
return BUTTON_PASTE;
	YY_BREAK
case 58:
YY_RULE_SETUP
return BUTTON_SELECTION;
	YY_BREAK
case 59:
YY_RULE_SETUP
return SHADOWS;
	YY_BREAK
case 60:
YY_RULE_SETUP
{ yylval.val = ALTFONT;      return GLOBAL_FLAG; }
	YY_BREAK
case 61:
YY_RULE_SETUP
{ yylval.val = BLINK;        return GLOBAL_FLAG; }
	YY_BREAK
case 62:
YY_RULE_SETUP
{
		  yylval.val = CURSOR_ALWAYS;return GLOBAL_FLAG; }
	YY_BREAK
case 63:
YY_RULE_SETUP
{
		  yylval.val = MENU_HIDE;    return GLOBAL_FLAG; }
	YY_BREAK
case 64:
YY_RULE_SETUP
{ yylval.val = MENU_INFO;    return GLOBAL_FLAG; }
	YY_BREAK
case 65:
YY_RULE_SETUP
{ yylval.val = MENU_RELAX;   return GLOBAL_FLAG; }
	YY_BREAK
case 66:
YY_RULE_SETUP
{
		  yylval.val = SCREEN_SCROLL;return GLOBAL_FLAG; }
	YY_BREAK
case 67:
YY_RULE_SETUP
{ yylval.val = KBD_ALT_FL;   return KBD_FLAG; }
	YY_BREAK
case 68:
YY_RULE_SETUP
{ yylval.val = KBD_CTRL_FL;  return KBD_FLAG; }
	YY_BREAK
case 69:
YY_RULE_SETUP
{ yylval.val = KBD_SHIFT_FL; return KBD_FLAG; }
	YY_BREAK
case 70:
YY_RULE_SETUP
{ yylval.val = BLACK;	return COLOR; }
	YY_BREAK
case 71:
YY_RULE_SETUP
{ yylval.val = BLUE;	return COLOR; }
	YY_BREAK
case 72:
YY_RULE_SETUP
{ yylval.val = GREEN;	return COLOR; }
	YY_BREAK
case 73:
YY_RULE_SETUP
{ yylval.val = CYAN;	return COLOR; }
	YY_BREAK
case 74:
YY_RULE_SETUP
{ yylval.val = RED;	return COLOR; }
	YY_BREAK
case 75:
YY_RULE_SETUP
{ yylval.val = MAGENTA;	return COLOR; }
	YY_BREAK
case 76:
YY_RULE_SETUP
{ yylval.val = YELLOW;	return COLOR; }
	YY_BREAK
case 77:
YY_RULE_SETUP
{ yylval.val = WHITE;	return COLOR; }
	YY_BREAK
case 78:
YY_RULE_SETUP
return COL_HIGH;
	YY_BREAK
case 79:
/* rule 79 can match eol */
YY_RULE_SETUP
{ LINE_NO++; return '\n'; }
	YY_BREAK
case 80:
YY_RULE_SETUP
{ return '\n'; }
	YY_BREAK
case 81:
YY_RULE_SETUP
/* eat comments */
	YY_BREAK
case 82:
YY_RULE_SETUP
{
		    yylval._string = my_strdup(yytext);
		    return STRING;
		}
	YY_BREAK
case 83:
YY_RULE_SETUP
{
		    yylval.val = strtol(yytext, NULL, 0);
		    return NUMBER;
		}
	YY_BREAK
case 84:
YY_RULE_SETUP
return (unsigned char)yytext[0];
	YY_BREAK
case 85:
YY_RULE_SETUP
{
		    sprintf(errbuf, "twin: %.200s:%d: invalid identifier `%s'\n",
			    FILE_NAME, LINE_NO, yytext);
		    YY_FATAL_ERROR(errbuf);
		}
	YY_BREAK
case 86:
/* rule 86 can match eol */
YY_RULE_SETUP
{
			int len = strlen(yytext + 1);
			char c, *p = yytext + 1;
			char *res = (char *)my_malloc(len), *s = res;
			
			while ((c = *p)) {
			    if (c != '\\' || !p[1])
				*s++ = *p++;
			    else switch(*++p) {
			      case 'a': *s++ = '\a'; break;
			      case 'b': *s++ = '\b'; break;
			      case 'e': *s++ = '\033'; break;
			      case 'f': *s++ = '\f'; break;
			      case 'n': *s++ = '\n'; break;
			      case 'r': *s++ = '\r'; break;
			      case 't': *s++ = '\t'; break;
			      case 'v': *s++ = '\v'; break;
			      case '0': *s++ = '\0'; break;
			      case 'u':
				/* FIXME: unicode is ok, but s is a (char *), not (hwfont *) ! */
				if (IS_HEXDIGIT(p[1]) &&
				    IS_HEXDIGIT(p[2]) &&
				    IS_HEXDIGIT(p[3]) &&
				    IS_HEXDIGIT(p[4])) {
				    *s++ = (((HEX2INT(p[1]) << 4) | HEX2INT(p[2])) << 8)
				        | ((HEX2INT(p[3]) << 4) | HEX2INT(p[2]));
				    p += 5;
				}
				break;
			      case 'x':
				if (IS_HEXDIGIT(p[1]) &&
				    IS_HEXDIGIT(p[2])) {
				    *s++ = (HEX2INT(p[1]) << 4)	| HEX2INT(p[2]);
				    p += 3;
				}
				break;
			      default:
				if (IS_OCTDIGIT(p[0]) &&
				    IS_OCTDIGIT(p[1]) &&
				    IS_OCTDIGIT(p[2])) {
				    *s++ = ((p[0] - '0') << 6)
					| ((p[1] - '0') << 3)
					|  (p[2] - '0');
				    p += 3;
				} else if (*p == '0')
				    *s++ = '\0', p++;
				else
				    *s++ = *p++;
				break;
			    }
			}
			s[-1] = '\0'; /* overwrite final \" */
			yylval._string = res;
			return STRING;
		}
	YY_BREAK
case 87:
/* rule 87 can match eol */
YY_RULE_SETUP
{
		    char *buf = AllocMem(256 + strlen(yytext));
		    if (buf) {
			sprintf(buf, "twin: %.200s:%d: unterminated string:\n%s\n",
				FILE_NAME, LINE_NO, yytext);
		    } else
			buf = "twin: unterminated string\n";
		    YY_FATAL_ERROR(buf);
		}
	YY_BREAK
case 88:
YY_RULE_SETUP
/* eat whitespace */
	YY_BREAK
case 89:
YY_RULE_SETUP
{
		    unsigned char ch = yytext[0];
		    sprintf(errbuf, "twin: %.200s:%d: illegal character 0x%02X",
			    FILE_NAME, LINE_NO, ch);
                    if (ch >= 32 && ch < 127) {
			sprintf(errbuf + strlen(errbuf), " `%c'\n", ch);
                    } else
			strcat(errbuf, "\n");
		    YY_FATAL_ERROR(errbuf);
		}
	YY_BREAK
case 90:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_last_accepting_cpos);
				yy_current_state = (yy_last_accepting_state);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart(yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 461 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 461 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 460);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up yytext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart(yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer(yyin,YY_BUF_SIZE );
	}

	yy_init_buffer(YY_CURRENT_BUFFER,input_file );
	yy_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * 
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree((void *) b->yy_ch_buf  );

	yyfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	yy_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack();

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param str a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (yyconst char * yy_str )
{
    
	return yy_scan_bytes(yy_str,strlen(yy_str) );
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (yyconst char * bytes, int  len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yyalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int yyget_lineno  (void)
{
        
    return yylineno;
}

/** Get the input stream.
 * 
 */
FILE *yyget_in  (void)
{
        return yyin;
}

/** Get the output stream.
 * 
 */
FILE *yyget_out  (void)
{
        return yyout;
}

/** Get the length of the current token.
 * 
 */
int yyget_leng  (void)
{
        return yyleng;
}

/** Get the current token.
 * 
 */

char *yyget_text  (void)
{
        return yytext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void yyset_lineno (int  line_number )
{
    
    yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  in_str )
{
        yyin = in_str ;
}

void yyset_out (FILE *  out_str )
{
        yyout = out_str ;
}

int yyget_debug  (void)
{
        return yy_flex_debug;
}

void yyset_debug (int  bdebug )
{
        yy_flex_debug = bdebug ;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
    	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
    	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *yyrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void yyfree (void * ptr )
{
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#undef YY_NEW_FILE
#undef YY_FLUSH_BUFFER
#undef yy_set_bol
#undef yy_new_buffer
#undef yy_set_interactive
#undef yytext_ptr
#undef YY_DO_BEFORE_ACTION

#ifdef YY_DECL_IS_OURS
#undef YY_DECL_IS_OURS
#undef YY_DECL
#endif

int set_yy_file(char *path) {
    uldat len;
    FILE *f;
	
    if (!path)
	return 1;
    
    if (read_stack_curr >= MAX_READ_DEPTH) {
	fprintf(stderr, "twin: %s:%d: `Read' commands nested too deeply!\n",
		FILE_NAME, LINE_NO);
	return 0;
    }

    if (!(f = fopen(path, "r")))
	return 1;

    len = LenStr(path) + 1;
    
    read_stack[read_stack_curr++] = YY_CURRENT_BUFFER;
    LINE_NO = 1;
    FILE_NAME = my_malloc(len);
    CopyMem(path, FILE_NAME, len);

    yy_switch_to_buffer(yy_create_buffer(yyin = f,YY_BUF_SIZE));
    BEGIN(INITIAL);
    return 0;
}

static int yywrap(void) {
    if ( read_stack_curr > 0 ) {
	fclose(yyin);
	yy_delete_buffer(YY_CURRENT_BUFFER );
	yy_switch_to_buffer(read_stack[--read_stack_curr]);
	BEGIN(INITIAL);
    }
    return read_stack_curr <= 0;
}

