




/* This file was automatically generated from m4/tt.m4hh, do not edit! */


/*
 *  ttcheckmenuitem.h  --  public header for libTT declarations related to "ttcheckmenuitem"
 *
 *  Copyright (C) 2001-2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */



#ifndef _TT_TTCHECKMENUITEM_H
#define _TT_TTCHECKMENUITEM_H







/* object structure */
#ifndef s_ttcheckmenuitem


/** ttcheckmenuitem: a checkable line in a menu */
        struct s_ttcheckmenuitem;
typedef struct s_ttcheckmenuitem *	ttcheckmenuitem;

#endif /* s_ttcheckmenuitem */



/* global runtime class type; returned by TTCLASSOF(<some object>) */
/** class of ttcheckmenuitem */
extern ttclass	TTClass_ttcheckmenuitem;





/* ttcheckmenuitem methods */









/* TTGet*_ttcheckmenuitem and TTSet*_ttcheckmenuitem methods */










#endif /* _TT_TTCHECKMENUITEM_H */

