
/*
 * This file was automatically generated
 * from clisp sources with "from-kernel.bash"
 * Do not edit!
 */


#ifndef _TUTF_MAC_SYMBOL_H
#define _TUTF_MAC_SYMBOL_H

#define T_MAC_SYMBOL_NULL	0x0000
#define T_MAC_SYMBOL_START_OF_HEADING	0x0001
#define T_MAC_SYMBOL_START_OF_TEXT	0x0002
#define T_MAC_SYMBOL_END_OF_TEXT	0x0003
#define T_MAC_SYMBOL_END_OF_TRANSMISSION	0x0004
#define T_MAC_SYMBOL_ENQUIRY	0x0005
#define T_MAC_SYMBOL_ACKNOWLEDGE	0x0006
#define T_MAC_SYMBOL_BELL	0x0007
#define T_MAC_SYMBOL_BACKSPACE	0x0008
#define T_MAC_SYMBOL_HORIZONTAL_TABULATION	0x0009
#define T_MAC_SYMBOL_LINE_FEED	0x000A
#define T_MAC_SYMBOL_VERTICAL_TABULATION	0x000B
#define T_MAC_SYMBOL_FORM_FEED	0x000C
#define T_MAC_SYMBOL_CARRIAGE_RETURN	0x000D
#define T_MAC_SYMBOL_SHIFT_OUT	0x000E
#define T_MAC_SYMBOL_SHIFT_IN	0x000F
#define T_MAC_SYMBOL_DATA_LINK_ESCAPE	0x0010
#define T_MAC_SYMBOL_DEVICE_CONTROL_ONE	0x0011
#define T_MAC_SYMBOL_DEVICE_CONTROL_TWO	0x0012
#define T_MAC_SYMBOL_DEVICE_CONTROL_THREE	0x0013
#define T_MAC_SYMBOL_DEVICE_CONTROL_FOUR	0x0014
#define T_MAC_SYMBOL_NEGATIVE_ACKNOWLEDGE	0x0015
#define T_MAC_SYMBOL_SYNCHRONOUS_IDLE	0x0016
#define T_MAC_SYMBOL_END_OF_TRANSMISSION_BLOCK	0x0017
#define T_MAC_SYMBOL_CANCEL	0x0018
#define T_MAC_SYMBOL_END_OF_MEDIUM	0x0019
#define T_MAC_SYMBOL_SUBSTITUTE	0x001A
#define T_MAC_SYMBOL_ESCAPE	0x001B
#define T_MAC_SYMBOL_FILE_SEPARATOR	0x001C
#define T_MAC_SYMBOL_GROUP_SEPARATOR	0x001D
#define T_MAC_SYMBOL_RECORD_SEPARATOR	0x001E
#define T_MAC_SYMBOL_UNIT_SEPARATOR	0x001F
#define T_MAC_SYMBOL_SPACE	0x0020
#define T_MAC_SYMBOL_EXCLAMATION_MARK	0x0021
#define T_MAC_SYMBOL_FOR_ALL	0x0022
#define T_MAC_SYMBOL_NUMBER_SIGN	0x0023
#define T_MAC_SYMBOL_THERE_EXISTS	0x0024
#define T_MAC_SYMBOL_PERCENT_SIGN	0x0025
#define T_MAC_SYMBOL_AMPERSAND	0x0026
#define T_MAC_SYMBOL_SMALL_CONTAINS_AS_MEMBER	0x0027
#define T_MAC_SYMBOL_LEFT_PARENTHESIS	0x0028
#define T_MAC_SYMBOL_RIGHT_PARENTHESIS	0x0029
#define T_MAC_SYMBOL_ASTERISK_OPERATOR	0x002A
#define T_MAC_SYMBOL_PLUS_SIGN	0x002B
#define T_MAC_SYMBOL_COMMA	0x002C
#define T_MAC_SYMBOL_MINUS_SIGN	0x002D
#define T_MAC_SYMBOL_FULL_STOP	0x002E
#define T_MAC_SYMBOL_SOLIDUS	0x002F
#define T_MAC_SYMBOL_DIGIT_ZERO	0x0030
#define T_MAC_SYMBOL_DIGIT_ONE	0x0031
#define T_MAC_SYMBOL_DIGIT_TWO	0x0032
#define T_MAC_SYMBOL_DIGIT_THREE	0x0033
#define T_MAC_SYMBOL_DIGIT_FOUR	0x0034
#define T_MAC_SYMBOL_DIGIT_FIVE	0x0035
#define T_MAC_SYMBOL_DIGIT_SIX	0x0036
#define T_MAC_SYMBOL_DIGIT_SEVEN	0x0037
#define T_MAC_SYMBOL_DIGIT_EIGHT	0x0038
#define T_MAC_SYMBOL_DIGIT_NINE	0x0039
#define T_MAC_SYMBOL_COLON	0x003A
#define T_MAC_SYMBOL_SEMICOLON	0x003B
#define T_MAC_SYMBOL_LESS_THAN_SIGN	0x003C
#define T_MAC_SYMBOL_EQUALS_SIGN	0x003D
#define T_MAC_SYMBOL_GREATER_THAN_SIGN	0x003E
#define T_MAC_SYMBOL_QUESTION_MARK	0x003F
#define T_MAC_SYMBOL_APPROXIMATELY_EQUAL_TO	0x0040
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_ALPHA	0x0041
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_BETA	0x0042
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_CHI	0x0043
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_DELTA	0x0044
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_EPSILON	0x0045
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_PHI	0x0046
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_GAMMA	0x0047
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_ETA	0x0048
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_IOTA	0x0049
#define T_MAC_SYMBOL_GREEK_THETA_SYMBOL	0x004A
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_KAPPA	0x004B
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_LAMDA	0x004C
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_MU	0x004D
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_NU	0x004E
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_OMICRON	0x004F
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_PI	0x0050
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_THETA	0x0051
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_RHO	0x0052
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_SIGMA	0x0053
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_TAU	0x0054
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_UPSILON	0x0055
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_FINAL_SIGMA	0x0056
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_OMEGA	0x0057
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_XI	0x0058
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_PSI	0x0059
#define T_MAC_SYMBOL_GREEK_CAPITAL_LETTER_ZETA	0x005A
#define T_MAC_SYMBOL_LEFT_SQUARE_BRACKET	0x005B
#define T_MAC_SYMBOL_THEREFORE	0x005C
#define T_MAC_SYMBOL_RIGHT_SQUARE_BRACKET	0x005D
#define T_MAC_SYMBOL_UP_TACK	0x005E
#define T_MAC_SYMBOL_LOW_LINE	0x005F
#define T_MAC_SYMBOL_UNDEFINED_0060	0x0060
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_ALPHA	0x0061
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_BETA	0x0062
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_CHI	0x0063
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_DELTA	0x0064
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_EPSILON	0x0065
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_PHI	0x0066
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_GAMMA	0x0067
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_ETA	0x0068
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_IOTA	0x0069
#define T_MAC_SYMBOL_GREEK_PHI_SYMBOL	0x006A
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_KAPPA	0x006B
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_LAMDA	0x006C
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_MU	0x006D
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_NU	0x006E
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_OMICRON	0x006F
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_PI	0x0070
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_THETA	0x0071
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_RHO	0x0072
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_SIGMA	0x0073
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_TAU	0x0074
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_UPSILON	0x0075
#define T_MAC_SYMBOL_GREEK_PI_SYMBOL	0x0076
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_OMEGA	0x0077
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_XI	0x0078
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_PSI	0x0079
#define T_MAC_SYMBOL_GREEK_SMALL_LETTER_ZETA	0x007A
#define T_MAC_SYMBOL_LEFT_CURLY_BRACKET	0x007B
#define T_MAC_SYMBOL_VERTICAL_LINE	0x007C
#define T_MAC_SYMBOL_RIGHT_CURLY_BRACKET	0x007D
#define T_MAC_SYMBOL_TILDE_OPERATOR	0x007E
#define T_MAC_SYMBOL_UNDEFINED_007F	0x007F
#define T_MAC_SYMBOL_UNDEFINED_0080	0x0080
#define T_MAC_SYMBOL_UNDEFINED_0081	0x0081
#define T_MAC_SYMBOL_UNDEFINED_0082	0x0082
#define T_MAC_SYMBOL_UNDEFINED_0083	0x0083
#define T_MAC_SYMBOL_UNDEFINED_0084	0x0084
#define T_MAC_SYMBOL_UNDEFINED_0085	0x0085
#define T_MAC_SYMBOL_UNDEFINED_0086	0x0086
#define T_MAC_SYMBOL_UNDEFINED_0087	0x0087
#define T_MAC_SYMBOL_UNDEFINED_0088	0x0088
#define T_MAC_SYMBOL_UNDEFINED_0089	0x0089
#define T_MAC_SYMBOL_UNDEFINED_008A	0x008A
#define T_MAC_SYMBOL_UNDEFINED_008B	0x008B
#define T_MAC_SYMBOL_UNDEFINED_008C	0x008C
#define T_MAC_SYMBOL_UNDEFINED_008D	0x008D
#define T_MAC_SYMBOL_UNDEFINED_008E	0x008E
#define T_MAC_SYMBOL_UNDEFINED_008F	0x008F
#define T_MAC_SYMBOL_UNDEFINED_0090	0x0090
#define T_MAC_SYMBOL_UNDEFINED_0091	0x0091
#define T_MAC_SYMBOL_UNDEFINED_0092	0x0092
#define T_MAC_SYMBOL_UNDEFINED_0093	0x0093
#define T_MAC_SYMBOL_UNDEFINED_0094	0x0094
#define T_MAC_SYMBOL_UNDEFINED_0095	0x0095
#define T_MAC_SYMBOL_UNDEFINED_0096	0x0096
#define T_MAC_SYMBOL_UNDEFINED_0097	0x0097
#define T_MAC_SYMBOL_UNDEFINED_0098	0x0098
#define T_MAC_SYMBOL_UNDEFINED_0099	0x0099
#define T_MAC_SYMBOL_UNDEFINED_009A	0x009A
#define T_MAC_SYMBOL_UNDEFINED_009B	0x009B
#define T_MAC_SYMBOL_UNDEFINED_009C	0x009C
#define T_MAC_SYMBOL_UNDEFINED_009D	0x009D
#define T_MAC_SYMBOL_UNDEFINED_009E	0x009E
#define T_MAC_SYMBOL_UNDEFINED_009F	0x009F
#define T_MAC_SYMBOL_UNDEFINED_00A0	0x00A0
#define T_MAC_SYMBOL_GREEK_UPSILON_WITH_HOOK_SYMBOL	0x00A1
#define T_MAC_SYMBOL_PRIME	0x00A2
#define T_MAC_SYMBOL_LESS_THAN_OR_EQUAL_TO	0x00A3
#define T_MAC_SYMBOL_FRACTION_SLASH	0x00A4
#define T_MAC_SYMBOL_INFINITY	0x00A5
#define T_MAC_SYMBOL_LATIN_SMALL_LETTER_F_WITH_HOOK	0x00A6
#define T_MAC_SYMBOL_BLACK_CLUB_SUIT	0x00A7
#define T_MAC_SYMBOL_BLACK_DIAMOND_SUIT	0x00A8
#define T_MAC_SYMBOL_BLACK_HEART_SUIT	0x00A9
#define T_MAC_SYMBOL_BLACK_SPADE_SUIT	0x00AA
#define T_MAC_SYMBOL_LEFT_RIGHT_ARROW	0x00AB
#define T_MAC_SYMBOL_LEFTWARDS_ARROW	0x00AC
#define T_MAC_SYMBOL_UPWARDS_ARROW	0x00AD
#define T_MAC_SYMBOL_RIGHTWARDS_ARROW	0x00AE
#define T_MAC_SYMBOL_DOWNWARDS_ARROW	0x00AF
#define T_MAC_SYMBOL_DEGREE_SIGN	0x00B0
#define T_MAC_SYMBOL_PLUS_MINUS_SIGN	0x00B1
#define T_MAC_SYMBOL_DOUBLE_PRIME	0x00B2
#define T_MAC_SYMBOL_GREATER_THAN_OR_EQUAL_TO	0x00B3
#define T_MAC_SYMBOL_MULTIPLICATION_SIGN	0x00B4
#define T_MAC_SYMBOL_PROPORTIONAL_TO	0x00B5
#define T_MAC_SYMBOL_PARTIAL_DIFFERENTIAL	0x00B6
#define T_MAC_SYMBOL_BULLET	0x00B7
#define T_MAC_SYMBOL_DIVISION_SIGN	0x00B8
#define T_MAC_SYMBOL_NOT_EQUAL_TO	0x00B9
#define T_MAC_SYMBOL_IDENTICAL_TO	0x00BA
#define T_MAC_SYMBOL_ALMOST_EQUAL_TO	0x00BB
#define T_MAC_SYMBOL_HORIZONTAL_ELLIPSIS	0x00BC
#define T_MAC_SYMBOL_UNDEFINED_00BD	0x00BD
#define T_MAC_SYMBOL_UNDEFINED_00BE	0x00BE
#define T_MAC_SYMBOL_DOWNWARDS_ARROW_WITH_CORNER_LEFTWARDS	0x00BF
#define T_MAC_SYMBOL_ALEF_SYMBOL	0x00C0
#define T_MAC_SYMBOL_BLACK_LETTER_CAPITAL_I	0x00C1
#define T_MAC_SYMBOL_BLACK_LETTER_CAPITAL_R	0x00C2
#define T_MAC_SYMBOL_SCRIPT_CAPITAL_P	0x00C3
#define T_MAC_SYMBOL_CIRCLED_TIMES	0x00C4
#define T_MAC_SYMBOL_CIRCLED_PLUS	0x00C5
#define T_MAC_SYMBOL_EMPTY_SET	0x00C6
#define T_MAC_SYMBOL_INTERSECTION	0x00C7
#define T_MAC_SYMBOL_UNION	0x00C8
#define T_MAC_SYMBOL_SUPERSET_OF	0x00C9
#define T_MAC_SYMBOL_SUPERSET_OF_OR_EQUAL_TO	0x00CA
#define T_MAC_SYMBOL_NOT_A_SUBSET_OF	0x00CB
#define T_MAC_SYMBOL_SUBSET_OF	0x00CC
#define T_MAC_SYMBOL_SUBSET_OF_OR_EQUAL_TO	0x00CD
#define T_MAC_SYMBOL_ELEMENT_OF	0x00CE
#define T_MAC_SYMBOL_NOT_AN_ELEMENT_OF	0x00CF
#define T_MAC_SYMBOL_ANGLE	0x00D0
#define T_MAC_SYMBOL_NABLA	0x00D1
#define T_MAC_SYMBOL_REGISTERED_SIGN	0x00D2
#define T_MAC_SYMBOL_COPYRIGHT_SIGN	0x00D3
#define T_MAC_SYMBOL_TRADE_MARK_SIGN	0x00D4
#define T_MAC_SYMBOL_N_ARY_PRODUCT	0x00D5
#define T_MAC_SYMBOL_SQUARE_ROOT	0x00D6
#define T_MAC_SYMBOL_DOT_OPERATOR	0x00D7
#define T_MAC_SYMBOL_NOT_SIGN	0x00D8
#define T_MAC_SYMBOL_LOGICAL_AND	0x00D9
#define T_MAC_SYMBOL_LOGICAL_OR	0x00DA
#define T_MAC_SYMBOL_LEFT_RIGHT_DOUBLE_ARROW	0x00DB
#define T_MAC_SYMBOL_LEFTWARDS_DOUBLE_ARROW	0x00DC
#define T_MAC_SYMBOL_UPWARDS_DOUBLE_ARROW	0x00DD
#define T_MAC_SYMBOL_RIGHTWARDS_DOUBLE_ARROW	0x00DE
#define T_MAC_SYMBOL_DOWNWARDS_DOUBLE_ARROW	0x00DF
#define T_MAC_SYMBOL_DIAMOND_OPERATOR	0x00E0
#define T_MAC_SYMBOL_LEFT_POINTING_ANGLE_BRACKET	0x00E1
#define T_MAC_SYMBOL_UNDEFINED_00E2	0x00E2
#define T_MAC_SYMBOL_UNDEFINED_00E3	0x00E3
#define T_MAC_SYMBOL_UNDEFINED_00E4	0x00E4
#define T_MAC_SYMBOL_N_ARY_SUMMATION	0x00E5
#define T_MAC_SYMBOL_UNDEFINED_00E6	0x00E6
#define T_MAC_SYMBOL_UNDEFINED_00E7	0x00E7
#define T_MAC_SYMBOL_UNDEFINED_00E8	0x00E8
#define T_MAC_SYMBOL_UNDEFINED_00E9	0x00E9
#define T_MAC_SYMBOL_UNDEFINED_00EA	0x00EA
#define T_MAC_SYMBOL_UNDEFINED_00EB	0x00EB
#define T_MAC_SYMBOL_UNDEFINED_00EC	0x00EC
#define T_MAC_SYMBOL_UNDEFINED_00ED	0x00ED
#define T_MAC_SYMBOL_UNDEFINED_00EE	0x00EE
#define T_MAC_SYMBOL_UNDEFINED_00EF	0x00EF
#define T_MAC_SYMBOL_UNDEFINED_00F0	0x00F0
#define T_MAC_SYMBOL_RIGHT_POINTING_ANGLE_BRACKET	0x00F1
#define T_MAC_SYMBOL_INTEGRAL	0x00F2
#define T_MAC_SYMBOL_TOP_HALF_INTEGRAL	0x00F3
#define T_MAC_SYMBOL_UNDEFINED_00F4	0x00F4
#define T_MAC_SYMBOL_BOTTOM_HALF_INTEGRAL	0x00F5
#define T_MAC_SYMBOL_UNDEFINED_00F6	0x00F6
#define T_MAC_SYMBOL_UNDEFINED_00F7	0x00F7
#define T_MAC_SYMBOL_UNDEFINED_00F8	0x00F8
#define T_MAC_SYMBOL_UNDEFINED_00F9	0x00F9
#define T_MAC_SYMBOL_UNDEFINED_00FA	0x00FA
#define T_MAC_SYMBOL_UNDEFINED_00FB	0x00FB
#define T_MAC_SYMBOL_UNDEFINED_00FC	0x00FC
#define T_MAC_SYMBOL_UNDEFINED_00FD	0x00FD
#define T_MAC_SYMBOL_UNDEFINED_00FE	0x00FE
#define T_MAC_SYMBOL_UNDEFINED_00FF	0x00FF

/* list of all characters */
#define T_LIST_MAC_SYMBOL(EL) \
    EL(_NULL) \
    EL(_START_OF_HEADING) \
    EL(_START_OF_TEXT) \
    EL(_END_OF_TEXT) \
    EL(_END_OF_TRANSMISSION) \
    EL(_ENQUIRY) \
    EL(_ACKNOWLEDGE) \
    EL(_BELL) \
    EL(_BACKSPACE) \
    EL(_HORIZONTAL_TABULATION) \
    EL(_LINE_FEED) \
    EL(_VERTICAL_TABULATION) \
    EL(_FORM_FEED) \
    EL(_CARRIAGE_RETURN) \
    EL(_SHIFT_OUT) \
    EL(_SHIFT_IN) \
    EL(_DATA_LINK_ESCAPE) \
    EL(_DEVICE_CONTROL_ONE) \
    EL(_DEVICE_CONTROL_TWO) \
    EL(_DEVICE_CONTROL_THREE) \
    EL(_DEVICE_CONTROL_FOUR) \
    EL(_NEGATIVE_ACKNOWLEDGE) \
    EL(_SYNCHRONOUS_IDLE) \
    EL(_END_OF_TRANSMISSION_BLOCK) \
    EL(_CANCEL) \
    EL(_END_OF_MEDIUM) \
    EL(_SUBSTITUTE) \
    EL(_ESCAPE) \
    EL(_FILE_SEPARATOR) \
    EL(_GROUP_SEPARATOR) \
    EL(_RECORD_SEPARATOR) \
    EL(_UNIT_SEPARATOR) \
    EL(_SPACE) \
    EL(_EXCLAMATION_MARK) \
    EL(_FOR_ALL) \
    EL(_NUMBER_SIGN) \
    EL(_THERE_EXISTS) \
    EL(_PERCENT_SIGN) \
    EL(_AMPERSAND) \
    EL(_SMALL_CONTAINS_AS_MEMBER) \
    EL(_LEFT_PARENTHESIS) \
    EL(_RIGHT_PARENTHESIS) \
    EL(_ASTERISK_OPERATOR) \
    EL(_PLUS_SIGN) \
    EL(_COMMA) \
    EL(_MINUS_SIGN) \
    EL(_FULL_STOP) \
    EL(_SOLIDUS) \
    EL(_DIGIT_ZERO) \
    EL(_DIGIT_ONE) \
    EL(_DIGIT_TWO) \
    EL(_DIGIT_THREE) \
    EL(_DIGIT_FOUR) \
    EL(_DIGIT_FIVE) \
    EL(_DIGIT_SIX) \
    EL(_DIGIT_SEVEN) \
    EL(_DIGIT_EIGHT) \
    EL(_DIGIT_NINE) \
    EL(_COLON) \
    EL(_SEMICOLON) \
    EL(_LESS_THAN_SIGN) \
    EL(_EQUALS_SIGN) \
    EL(_GREATER_THAN_SIGN) \
    EL(_QUESTION_MARK) \
    EL(_APPROXIMATELY_EQUAL_TO) \
    EL(_GREEK_CAPITAL_LETTER_ALPHA) \
    EL(_GREEK_CAPITAL_LETTER_BETA) \
    EL(_GREEK_CAPITAL_LETTER_CHI) \
    EL(_GREEK_CAPITAL_LETTER_DELTA) \
    EL(_GREEK_CAPITAL_LETTER_EPSILON) \
    EL(_GREEK_CAPITAL_LETTER_PHI) \
    EL(_GREEK_CAPITAL_LETTER_GAMMA) \
    EL(_GREEK_CAPITAL_LETTER_ETA) \
    EL(_GREEK_CAPITAL_LETTER_IOTA) \
    EL(_GREEK_THETA_SYMBOL) \
    EL(_GREEK_CAPITAL_LETTER_KAPPA) \
    EL(_GREEK_CAPITAL_LETTER_LAMDA) \
    EL(_GREEK_CAPITAL_LETTER_MU) \
    EL(_GREEK_CAPITAL_LETTER_NU) \
    EL(_GREEK_CAPITAL_LETTER_OMICRON) \
    EL(_GREEK_CAPITAL_LETTER_PI) \
    EL(_GREEK_CAPITAL_LETTER_THETA) \
    EL(_GREEK_CAPITAL_LETTER_RHO) \
    EL(_GREEK_CAPITAL_LETTER_SIGMA) \
    EL(_GREEK_CAPITAL_LETTER_TAU) \
    EL(_GREEK_CAPITAL_LETTER_UPSILON) \
    EL(_GREEK_SMALL_LETTER_FINAL_SIGMA) \
    EL(_GREEK_CAPITAL_LETTER_OMEGA) \
    EL(_GREEK_CAPITAL_LETTER_XI) \
    EL(_GREEK_CAPITAL_LETTER_PSI) \
    EL(_GREEK_CAPITAL_LETTER_ZETA) \
    EL(_LEFT_SQUARE_BRACKET) \
    EL(_THEREFORE) \
    EL(_RIGHT_SQUARE_BRACKET) \
    EL(_UP_TACK) \
    EL(_LOW_LINE) \
    EL(_UNDEFINED_0060) \
    EL(_GREEK_SMALL_LETTER_ALPHA) \
    EL(_GREEK_SMALL_LETTER_BETA) \
    EL(_GREEK_SMALL_LETTER_CHI) \
    EL(_GREEK_SMALL_LETTER_DELTA) \
    EL(_GREEK_SMALL_LETTER_EPSILON) \
    EL(_GREEK_SMALL_LETTER_PHI) \
    EL(_GREEK_SMALL_LETTER_GAMMA) \
    EL(_GREEK_SMALL_LETTER_ETA) \
    EL(_GREEK_SMALL_LETTER_IOTA) \
    EL(_GREEK_PHI_SYMBOL) \
    EL(_GREEK_SMALL_LETTER_KAPPA) \
    EL(_GREEK_SMALL_LETTER_LAMDA) \
    EL(_GREEK_SMALL_LETTER_MU) \
    EL(_GREEK_SMALL_LETTER_NU) \
    EL(_GREEK_SMALL_LETTER_OMICRON) \
    EL(_GREEK_SMALL_LETTER_PI) \
    EL(_GREEK_SMALL_LETTER_THETA) \
    EL(_GREEK_SMALL_LETTER_RHO) \
    EL(_GREEK_SMALL_LETTER_SIGMA) \
    EL(_GREEK_SMALL_LETTER_TAU) \
    EL(_GREEK_SMALL_LETTER_UPSILON) \
    EL(_GREEK_PI_SYMBOL) \
    EL(_GREEK_SMALL_LETTER_OMEGA) \
    EL(_GREEK_SMALL_LETTER_XI) \
    EL(_GREEK_SMALL_LETTER_PSI) \
    EL(_GREEK_SMALL_LETTER_ZETA) \
    EL(_LEFT_CURLY_BRACKET) \
    EL(_VERTICAL_LINE) \
    EL(_RIGHT_CURLY_BRACKET) \
    EL(_TILDE_OPERATOR) \
    EL(_UNDEFINED_007F) \
    EL(_UNDEFINED_0080) \
    EL(_UNDEFINED_0081) \
    EL(_UNDEFINED_0082) \
    EL(_UNDEFINED_0083) \
    EL(_UNDEFINED_0084) \
    EL(_UNDEFINED_0085) \
    EL(_UNDEFINED_0086) \
    EL(_UNDEFINED_0087) \
    EL(_UNDEFINED_0088) \
    EL(_UNDEFINED_0089) \
    EL(_UNDEFINED_008A) \
    EL(_UNDEFINED_008B) \
    EL(_UNDEFINED_008C) \
    EL(_UNDEFINED_008D) \
    EL(_UNDEFINED_008E) \
    EL(_UNDEFINED_008F) \
    EL(_UNDEFINED_0090) \
    EL(_UNDEFINED_0091) \
    EL(_UNDEFINED_0092) \
    EL(_UNDEFINED_0093) \
    EL(_UNDEFINED_0094) \
    EL(_UNDEFINED_0095) \
    EL(_UNDEFINED_0096) \
    EL(_UNDEFINED_0097) \
    EL(_UNDEFINED_0098) \
    EL(_UNDEFINED_0099) \
    EL(_UNDEFINED_009A) \
    EL(_UNDEFINED_009B) \
    EL(_UNDEFINED_009C) \
    EL(_UNDEFINED_009D) \
    EL(_UNDEFINED_009E) \
    EL(_UNDEFINED_009F) \
    EL(_UNDEFINED_00A0) \
    EL(_GREEK_UPSILON_WITH_HOOK_SYMBOL) \
    EL(_PRIME) \
    EL(_LESS_THAN_OR_EQUAL_TO) \
    EL(_FRACTION_SLASH) \
    EL(_INFINITY) \
    EL(_LATIN_SMALL_LETTER_F_WITH_HOOK) \
    EL(_BLACK_CLUB_SUIT) \
    EL(_BLACK_DIAMOND_SUIT) \
    EL(_BLACK_HEART_SUIT) \
    EL(_BLACK_SPADE_SUIT) \
    EL(_LEFT_RIGHT_ARROW) \
    EL(_LEFTWARDS_ARROW) \
    EL(_UPWARDS_ARROW) \
    EL(_RIGHTWARDS_ARROW) \
    EL(_DOWNWARDS_ARROW) \
    EL(_DEGREE_SIGN) \
    EL(_PLUS_MINUS_SIGN) \
    EL(_DOUBLE_PRIME) \
    EL(_GREATER_THAN_OR_EQUAL_TO) \
    EL(_MULTIPLICATION_SIGN) \
    EL(_PROPORTIONAL_TO) \
    EL(_PARTIAL_DIFFERENTIAL) \
    EL(_BULLET) \
    EL(_DIVISION_SIGN) \
    EL(_NOT_EQUAL_TO) \
    EL(_IDENTICAL_TO) \
    EL(_ALMOST_EQUAL_TO) \
    EL(_HORIZONTAL_ELLIPSIS) \
    EL(_UNDEFINED_00BD) \
    EL(_UNDEFINED_00BE) \
    EL(_DOWNWARDS_ARROW_WITH_CORNER_LEFTWARDS) \
    EL(_ALEF_SYMBOL) \
    EL(_BLACK_LETTER_CAPITAL_I) \
    EL(_BLACK_LETTER_CAPITAL_R) \
    EL(_SCRIPT_CAPITAL_P) \
    EL(_CIRCLED_TIMES) \
    EL(_CIRCLED_PLUS) \
    EL(_EMPTY_SET) \
    EL(_INTERSECTION) \
    EL(_UNION) \
    EL(_SUPERSET_OF) \
    EL(_SUPERSET_OF_OR_EQUAL_TO) \
    EL(_NOT_A_SUBSET_OF) \
    EL(_SUBSET_OF) \
    EL(_SUBSET_OF_OR_EQUAL_TO) \
    EL(_ELEMENT_OF) \
    EL(_NOT_AN_ELEMENT_OF) \
    EL(_ANGLE) \
    EL(_NABLA) \
    EL(_REGISTERED_SIGN) \
    EL(_COPYRIGHT_SIGN) \
    EL(_TRADE_MARK_SIGN) \
    EL(_N_ARY_PRODUCT) \
    EL(_SQUARE_ROOT) \
    EL(_DOT_OPERATOR) \
    EL(_NOT_SIGN) \
    EL(_LOGICAL_AND) \
    EL(_LOGICAL_OR) \
    EL(_LEFT_RIGHT_DOUBLE_ARROW) \
    EL(_LEFTWARDS_DOUBLE_ARROW) \
    EL(_UPWARDS_DOUBLE_ARROW) \
    EL(_RIGHTWARDS_DOUBLE_ARROW) \
    EL(_DOWNWARDS_DOUBLE_ARROW) \
    EL(_DIAMOND_OPERATOR) \
    EL(_LEFT_POINTING_ANGLE_BRACKET) \
    EL(_UNDEFINED_00E2) \
    EL(_UNDEFINED_00E3) \
    EL(_UNDEFINED_00E4) \
    EL(_N_ARY_SUMMATION) \
    EL(_UNDEFINED_00E6) \
    EL(_UNDEFINED_00E7) \
    EL(_UNDEFINED_00E8) \
    EL(_UNDEFINED_00E9) \
    EL(_UNDEFINED_00EA) \
    EL(_UNDEFINED_00EB) \
    EL(_UNDEFINED_00EC) \
    EL(_UNDEFINED_00ED) \
    EL(_UNDEFINED_00EE) \
    EL(_UNDEFINED_00EF) \
    EL(_UNDEFINED_00F0) \
    EL(_RIGHT_POINTING_ANGLE_BRACKET) \
    EL(_INTEGRAL) \
    EL(_TOP_HALF_INTEGRAL) \
    EL(_UNDEFINED_00F4) \
    EL(_BOTTOM_HALF_INTEGRAL) \
    EL(_UNDEFINED_00F6) \
    EL(_UNDEFINED_00F7) \
    EL(_UNDEFINED_00F8) \
    EL(_UNDEFINED_00F9) \
    EL(_UNDEFINED_00FA) \
    EL(_UNDEFINED_00FB) \
    EL(_UNDEFINED_00FC) \
    EL(_UNDEFINED_00FD) \
    EL(_UNDEFINED_00FE) \
    EL(_UNDEFINED_00FF)

/* list of all characters NOT IDENTICALLY MAPPED to UTF-16 */
#define T_NLIST_MAC_SYMBOL(EL) \
    EL(_FOR_ALL) \
    EL(_THERE_EXISTS) \
    EL(_SMALL_CONTAINS_AS_MEMBER) \
    EL(_ASTERISK_OPERATOR) \
    EL(_MINUS_SIGN) \
    EL(_APPROXIMATELY_EQUAL_TO) \
    EL(_GREEK_CAPITAL_LETTER_ALPHA) \
    EL(_GREEK_CAPITAL_LETTER_BETA) \
    EL(_GREEK_CAPITAL_LETTER_CHI) \
    EL(_GREEK_CAPITAL_LETTER_DELTA) \
    EL(_GREEK_CAPITAL_LETTER_EPSILON) \
    EL(_GREEK_CAPITAL_LETTER_PHI) \
    EL(_GREEK_CAPITAL_LETTER_GAMMA) \
    EL(_GREEK_CAPITAL_LETTER_ETA) \
    EL(_GREEK_CAPITAL_LETTER_IOTA) \
    EL(_GREEK_THETA_SYMBOL) \
    EL(_GREEK_CAPITAL_LETTER_KAPPA) \
    EL(_GREEK_CAPITAL_LETTER_LAMDA) \
    EL(_GREEK_CAPITAL_LETTER_MU) \
    EL(_GREEK_CAPITAL_LETTER_NU) \
    EL(_GREEK_CAPITAL_LETTER_OMICRON) \
    EL(_GREEK_CAPITAL_LETTER_PI) \
    EL(_GREEK_CAPITAL_LETTER_THETA) \
    EL(_GREEK_CAPITAL_LETTER_RHO) \
    EL(_GREEK_CAPITAL_LETTER_SIGMA) \
    EL(_GREEK_CAPITAL_LETTER_TAU) \
    EL(_GREEK_CAPITAL_LETTER_UPSILON) \
    EL(_GREEK_SMALL_LETTER_FINAL_SIGMA) \
    EL(_GREEK_CAPITAL_LETTER_OMEGA) \
    EL(_GREEK_CAPITAL_LETTER_XI) \
    EL(_GREEK_CAPITAL_LETTER_PSI) \
    EL(_GREEK_CAPITAL_LETTER_ZETA) \
    EL(_THEREFORE) \
    EL(_UP_TACK) \
    EL(_UNDEFINED_0060) \
    EL(_GREEK_SMALL_LETTER_ALPHA) \
    EL(_GREEK_SMALL_LETTER_BETA) \
    EL(_GREEK_SMALL_LETTER_CHI) \
    EL(_GREEK_SMALL_LETTER_DELTA) \
    EL(_GREEK_SMALL_LETTER_EPSILON) \
    EL(_GREEK_SMALL_LETTER_PHI) \
    EL(_GREEK_SMALL_LETTER_GAMMA) \
    EL(_GREEK_SMALL_LETTER_ETA) \
    EL(_GREEK_SMALL_LETTER_IOTA) \
    EL(_GREEK_PHI_SYMBOL) \
    EL(_GREEK_SMALL_LETTER_KAPPA) \
    EL(_GREEK_SMALL_LETTER_LAMDA) \
    EL(_GREEK_SMALL_LETTER_MU) \
    EL(_GREEK_SMALL_LETTER_NU) \
    EL(_GREEK_SMALL_LETTER_OMICRON) \
    EL(_GREEK_SMALL_LETTER_PI) \
    EL(_GREEK_SMALL_LETTER_THETA) \
    EL(_GREEK_SMALL_LETTER_RHO) \
    EL(_GREEK_SMALL_LETTER_SIGMA) \
    EL(_GREEK_SMALL_LETTER_TAU) \
    EL(_GREEK_SMALL_LETTER_UPSILON) \
    EL(_GREEK_PI_SYMBOL) \
    EL(_GREEK_SMALL_LETTER_OMEGA) \
    EL(_GREEK_SMALL_LETTER_XI) \
    EL(_GREEK_SMALL_LETTER_PSI) \
    EL(_GREEK_SMALL_LETTER_ZETA) \
    EL(_TILDE_OPERATOR) \
    EL(_UNDEFINED_00A0) \
    EL(_GREEK_UPSILON_WITH_HOOK_SYMBOL) \
    EL(_PRIME) \
    EL(_LESS_THAN_OR_EQUAL_TO) \
    EL(_FRACTION_SLASH) \
    EL(_INFINITY) \
    EL(_LATIN_SMALL_LETTER_F_WITH_HOOK) \
    EL(_BLACK_CLUB_SUIT) \
    EL(_BLACK_DIAMOND_SUIT) \
    EL(_BLACK_HEART_SUIT) \
    EL(_BLACK_SPADE_SUIT) \
    EL(_LEFT_RIGHT_ARROW) \
    EL(_LEFTWARDS_ARROW) \
    EL(_UPWARDS_ARROW) \
    EL(_RIGHTWARDS_ARROW) \
    EL(_DOWNWARDS_ARROW) \
    EL(_DOUBLE_PRIME) \
    EL(_GREATER_THAN_OR_EQUAL_TO) \
    EL(_MULTIPLICATION_SIGN) \
    EL(_PROPORTIONAL_TO) \
    EL(_PARTIAL_DIFFERENTIAL) \
    EL(_BULLET) \
    EL(_DIVISION_SIGN) \
    EL(_NOT_EQUAL_TO) \
    EL(_IDENTICAL_TO) \
    EL(_ALMOST_EQUAL_TO) \
    EL(_HORIZONTAL_ELLIPSIS) \
    EL(_UNDEFINED_00BD) \
    EL(_UNDEFINED_00BE) \
    EL(_DOWNWARDS_ARROW_WITH_CORNER_LEFTWARDS) \
    EL(_ALEF_SYMBOL) \
    EL(_BLACK_LETTER_CAPITAL_I) \
    EL(_BLACK_LETTER_CAPITAL_R) \
    EL(_SCRIPT_CAPITAL_P) \
    EL(_CIRCLED_TIMES) \
    EL(_CIRCLED_PLUS) \
    EL(_EMPTY_SET) \
    EL(_INTERSECTION) \
    EL(_UNION) \
    EL(_SUPERSET_OF) \
    EL(_SUPERSET_OF_OR_EQUAL_TO) \
    EL(_NOT_A_SUBSET_OF) \
    EL(_SUBSET_OF) \
    EL(_SUBSET_OF_OR_EQUAL_TO) \
    EL(_ELEMENT_OF) \
    EL(_NOT_AN_ELEMENT_OF) \
    EL(_ANGLE) \
    EL(_NABLA) \
    EL(_REGISTERED_SIGN) \
    EL(_COPYRIGHT_SIGN) \
    EL(_TRADE_MARK_SIGN) \
    EL(_N_ARY_PRODUCT) \
    EL(_SQUARE_ROOT) \
    EL(_DOT_OPERATOR) \
    EL(_NOT_SIGN) \
    EL(_LOGICAL_AND) \
    EL(_LOGICAL_OR) \
    EL(_LEFT_RIGHT_DOUBLE_ARROW) \
    EL(_LEFTWARDS_DOUBLE_ARROW) \
    EL(_UPWARDS_DOUBLE_ARROW) \
    EL(_RIGHTWARDS_DOUBLE_ARROW) \
    EL(_DOWNWARDS_DOUBLE_ARROW) \
    EL(_DIAMOND_OPERATOR) \
    EL(_LEFT_POINTING_ANGLE_BRACKET) \
    EL(_UNDEFINED_00E2) \
    EL(_UNDEFINED_00E3) \
    EL(_UNDEFINED_00E4) \
    EL(_N_ARY_SUMMATION) \
    EL(_UNDEFINED_00E6) \
    EL(_UNDEFINED_00E7) \
    EL(_UNDEFINED_00E8) \
    EL(_UNDEFINED_00E9) \
    EL(_UNDEFINED_00EA) \
    EL(_UNDEFINED_00EB) \
    EL(_UNDEFINED_00EC) \
    EL(_UNDEFINED_00ED) \
    EL(_UNDEFINED_00EE) \
    EL(_UNDEFINED_00EF) \
    EL(_UNDEFINED_00F0) \
    EL(_RIGHT_POINTING_ANGLE_BRACKET) \
    EL(_INTEGRAL) \
    EL(_TOP_HALF_INTEGRAL) \
    EL(_UNDEFINED_00F4) \
    EL(_BOTTOM_HALF_INTEGRAL) \
    EL(_UNDEFINED_00F6) \
    EL(_UNDEFINED_00F7) \
    EL(_UNDEFINED_00F8) \
    EL(_UNDEFINED_00F9) \
    EL(_UNDEFINED_00FA) \
    EL(_UNDEFINED_00FB) \
    EL(_UNDEFINED_00FC) \
    EL(_UNDEFINED_00FD) \
    EL(_UNDEFINED_00FE) \
    EL(_UNDEFINED_00FF)

#endif /* _TUTF_MAC_SYMBOL_H */
