/* This file was automatically generated from m4/common.m4, do not edit! */


/*
 *  common_m4.h  --  common #defines for all target drivers
 *
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */


#ifndef _TT_HW_COMMON_H
#define _TT_HW_COMMON_H

























 
/* defines for ttclass_ttobj fields: Class, id, size */
#define Class_ttobj   TClass_ttclass
#define id_ttobj      id_ttclass_ttobj
#define size_ttobj    sizeof(struct s_ttobj)
 
/* defines for ttclass_ttclass fields: Class, id, size */
#define Class_ttclass   TClass_ttclass
#define id_ttclass      id_ttclass_ttclass
#define size_ttclass    sizeof(struct s_ttclass)
 
/* defines for ttclass_ttobject fields: Class, id, size */
#define Class_ttobject   TClass_ttclass
#define id_ttobject      id_ttclass_ttobject
#define size_ttobject    sizeof(struct s_ttobject)
 
/* defines for ttclass_ttvector fields: Class, id, size */
#define Class_ttvector   TClass_ttclass
#define id_ttvector      id_ttclass_ttvector
#define size_ttvector    sizeof(struct s_ttvector)
 
/* defines for ttclass_ttfield fields: Class, id, size */
#define Class_ttfield   TClass_ttclass
#define id_ttfield      id_ttclass_ttfield
#define size_ttfield    sizeof(struct s_ttfield)
 
/* defines for ttclass_ttmethod fields: Class, id, size */
#define Class_ttmethod   TClass_ttclass
#define id_ttmethod      id_ttclass_ttmethod
#define size_ttmethod    sizeof(struct s_ttmethod)
 
/* defines for ttclass_ttevent fields: Class, id, size */
#define Class_ttevent   TClass_ttclass
#define id_ttevent      id_ttclass_ttevent
#define size_ttevent    sizeof(struct s_ttevent)
 
/* defines for ttclass_tteventbig fields: Class, id, size */
#define Class_tteventbig   TClass_ttclass
#define id_tteventbig      id_ttclass_tteventbig
#define size_tteventbig    sizeof(struct s_tteventbig)
 
/* defines for ttclass_ttbitmask fields: Class, id, size */
#define Class_ttbitmask   TClass_ttclass
#define id_ttbitmask      id_ttclass_ttbitmask
#define size_ttbitmask    sizeof(struct s_ttbitmask)
 
/* defines for ttclass_tteventmask fields: Class, id, size */
#define Class_tteventmask   TClass_ttclass
#define id_tteventmask      id_ttclass_tteventmask
#define size_tteventmask    sizeof(struct s_tteventmask)
 
/* defines for ttclass_ttcallback fields: Class, id, size */
#define Class_ttcallback   TClass_ttclass
#define id_ttcallback      id_ttclass_ttcallback
#define size_ttcallback    sizeof(struct s_ttcallback)
 
/* defines for ttclass_ttlistener fields: Class, id, size */
#define Class_ttlistener   TClass_ttclass
#define id_ttlistener      id_ttclass_ttlistener
#define size_ttlistener    sizeof(struct s_ttlistener)
 
/* defines for ttclass_tttimer fields: Class, id, size */
#define Class_tttimer   TClass_ttclass
#define id_tttimer      id_ttclass_tttimer
#define size_tttimer    sizeof(struct s_tttimer)
 
/* defines for ttclass_ttcomponent fields: Class, id, size */
#define Class_ttcomponent   TClass_ttclass
#define id_ttcomponent      id_ttclass_ttcomponent
#define size_ttcomponent    sizeof(struct s_ttcomponent)
 
/* defines for ttclass_ttdata fields: Class, id, size */
#define Class_ttdata   TClass_ttclass
#define id_ttdata      id_ttclass_ttdata
#define size_ttdata    sizeof(struct s_ttdata)
 
/* defines for ttclass_ttvisible fields: Class, id, size */
#define Class_ttvisible   TClass_ttclass
#define id_ttvisible      id_ttclass_ttvisible
#define size_ttvisible    sizeof(struct s_ttvisible)
 
/* defines for ttclass_ttlayout fields: Class, id, size */
#define Class_ttlayout   TClass_ttclass
#define id_ttlayout      id_ttclass_ttlayout
#define size_ttlayout    sizeof(struct s_ttlayout)
 
/* defines for ttclass_ttboxlayout fields: Class, id, size */
#define Class_ttboxlayout   TClass_ttclass
#define id_ttboxlayout      id_ttclass_ttboxlayout
#define size_ttboxlayout    sizeof(struct s_ttboxlayout)
 
/* defines for ttclass_ttborderlayout fields: Class, id, size */
#define Class_ttborderlayout   TClass_ttclass
#define id_ttborderlayout      id_ttclass_ttborderlayout
#define size_ttborderlayout    sizeof(struct s_ttborderlayout)
 
/* defines for ttclass_ttnative fields: Class, id, size */
#define Class_ttnative   TClass_ttclass
#define id_ttnative      id_ttclass_ttnative
#define size_ttnative    sizeof(struct s_ttnative)
 
/* defines for ttclass_ttwidget fields: Class, id, size */
#define Class_ttwidget   TClass_ttclass
#define id_ttwidget      id_ttclass_ttwidget
#define size_ttwidget    sizeof(struct s_ttwidget)
 
/* defines for ttclass_ttlabel fields: Class, id, size */
#define Class_ttlabel   TClass_ttclass
#define id_ttlabel      id_ttclass_ttlabel
#define size_ttlabel    sizeof(struct s_ttlabel)
 
/* defines for ttclass_tttooltip fields: Class, id, size */
#define Class_tttooltip   TClass_ttclass
#define id_tttooltip      id_ttclass_tttooltip
#define size_tttooltip    sizeof(struct s_tttooltip)
 
/* defines for ttclass_ttbuttongroup fields: Class, id, size */
#define Class_ttbuttongroup   TClass_ttclass
#define id_ttbuttongroup      id_ttclass_ttbuttongroup
#define size_ttbuttongroup    sizeof(struct s_ttbuttongroup)
 
/* defines for ttclass_ttanybutton fields: Class, id, size */
#define Class_ttanybutton   TClass_ttclass
#define id_ttanybutton      id_ttclass_ttanybutton
#define size_ttanybutton    sizeof(struct s_ttanybutton)
 
/* defines for ttclass_ttbutton fields: Class, id, size */
#define Class_ttbutton   TClass_ttclass
#define id_ttbutton      id_ttclass_ttbutton
#define size_ttbutton    sizeof(struct s_ttbutton)
 
/* defines for ttclass_ttcheckbutton fields: Class, id, size */
#define Class_ttcheckbutton   TClass_ttclass
#define id_ttcheckbutton      id_ttclass_ttcheckbutton
#define size_ttcheckbutton    sizeof(struct s_ttcheckbutton)
 
/* defines for ttclass_ttradiobutton fields: Class, id, size */
#define Class_ttradiobutton   TClass_ttclass
#define id_ttradiobutton      id_ttclass_ttradiobutton
#define size_ttradiobutton    sizeof(struct s_ttradiobutton)
 
/* defines for ttclass_ttanyscroll fields: Class, id, size */
#define Class_ttanyscroll   TClass_ttclass
#define id_ttanyscroll      id_ttclass_ttanyscroll
#define size_ttanyscroll    sizeof(struct s_ttanyscroll)
 
/* defines for ttclass_ttscrollbar fields: Class, id, size */
#define Class_ttscrollbar   TClass_ttclass
#define id_ttscrollbar      id_ttclass_ttscrollbar
#define size_ttscrollbar    sizeof(struct s_ttscrollbar)
 
/* defines for ttclass_ttslider fields: Class, id, size */
#define Class_ttslider   TClass_ttclass
#define id_ttslider      id_ttclass_ttslider
#define size_ttslider    sizeof(struct s_ttslider)
 
/* defines for ttclass_ttprogressbar fields: Class, id, size */
#define Class_ttprogressbar   TClass_ttclass
#define id_ttprogressbar      id_ttclass_ttprogressbar
#define size_ttprogressbar    sizeof(struct s_ttprogressbar)
 
/* defines for ttclass_ttscrollpane fields: Class, id, size */
#define Class_ttscrollpane   TClass_ttclass
#define id_ttscrollpane      id_ttclass_ttscrollpane
#define size_ttscrollpane    sizeof(struct s_ttscrollpane)
 
/* defines for ttclass_ttwindow fields: Class, id, size */
#define Class_ttwindow   TClass_ttclass
#define id_ttwindow      id_ttclass_ttwindow
#define size_ttwindow    sizeof(struct s_ttwindow)
 
/* defines for ttclass_ttframe fields: Class, id, size */
#define Class_ttframe   TClass_ttclass
#define id_ttframe      id_ttclass_ttframe
#define size_ttframe    sizeof(struct s_ttframe)
 
/* defines for ttclass_ttscroller fields: Class, id, size */
#define Class_ttscroller   TClass_ttclass
#define id_ttscroller      id_ttclass_ttscroller
#define size_ttscroller    sizeof(struct s_ttscroller)
 
/* defines for ttclass_ttmenuitem fields: Class, id, size */
#define Class_ttmenuitem   TClass_ttclass
#define id_ttmenuitem      id_ttclass_ttmenuitem
#define size_ttmenuitem    sizeof(struct s_ttmenuitem)
 
/* defines for ttclass_ttcheckmenuitem fields: Class, id, size */
#define Class_ttcheckmenuitem   TClass_ttclass
#define id_ttcheckmenuitem      id_ttclass_ttcheckmenuitem
#define size_ttcheckmenuitem    sizeof(struct s_ttcheckmenuitem)
 
/* defines for ttclass_ttradiomenuitem fields: Class, id, size */
#define Class_ttradiomenuitem   TClass_ttclass
#define id_ttradiomenuitem      id_ttclass_ttradiomenuitem
#define size_ttradiomenuitem    sizeof(struct s_ttradiomenuitem)
 
/* defines for ttclass_ttmenu fields: Class, id, size */
#define Class_ttmenu   TClass_ttclass
#define id_ttmenu      id_ttclass_ttmenu
#define size_ttmenu    sizeof(struct s_ttmenu)
 
/* defines for ttclass_ttmenubar fields: Class, id, size */
#define Class_ttmenubar   TClass_ttclass
#define id_ttmenubar      id_ttclass_ttmenubar
#define size_ttmenubar    sizeof(struct s_ttmenubar)
 
/* defines for ttclass_ttanytext fields: Class, id, size */
#define Class_ttanytext   TClass_ttclass
#define id_ttanytext      id_ttclass_ttanytext
#define size_ttanytext    sizeof(struct s_ttanytext)
 
/* defines for ttclass_tttextfield fields: Class, id, size */
#define Class_tttextfield   TClass_ttclass
#define id_tttextfield      id_ttclass_tttextfield
#define size_tttextfield    sizeof(struct s_tttextfield)
 
/* defines for ttclass_tttextarea fields: Class, id, size */
#define Class_tttextarea   TClass_ttclass
#define id_tttextarea      id_ttclass_tttextarea
#define size_tttextarea    sizeof(struct s_tttextarea)
 
/* defines for ttclass_tttheme fields: Class, id, size */
#define Class_tttheme   TClass_ttclass
#define id_tttheme      id_ttclass_tttheme
#define size_tttheme    sizeof(struct s_tttheme)
 
/* defines for ttclass_ttapplication fields: Class, id, size */
#define Class_ttapplication   TClass_ttclass
#define id_ttapplication      id_ttclass_ttapplication
#define size_ttapplication    sizeof(struct s_ttapplication)



#endif /* _TT_HW_COMMON_H */
