




/* This file was automatically generated from m4/tt.m4hh, do not edit! */


/*
 *  ttradiomenuitem.h  --  public header for libTT declarations related to "ttradiomenuitem"
 *
 *  Copyright (C) 2001-2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */



#ifndef _TT_TTRADIOMENUITEM_H
#define _TT_TTRADIOMENUITEM_H







/* object structure */
#ifndef s_ttradiomenuitem


/** ttradiomenuitem: a radio (mutual-exclusion) line in a menu */
        struct s_ttradiomenuitem;
typedef struct s_ttradiomenuitem *	ttradiomenuitem;

#endif /* s_ttradiomenuitem */



/* global runtime class type; returned by TTCLASSOF(<some object>) */
/** class of ttradiomenuitem */
extern ttclass	TTClass_ttradiomenuitem;





/* ttradiomenuitem methods */









/* TTGet*_ttradiomenuitem and TTSet*_ttradiomenuitem methods */










#endif /* _TT_TTRADIOMENUITEM_H */

