/* Copyright (C) 2000 by Massimiliano Ghilardi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

/*
 * edit this and recompile to change the HotKey (used to activate menu).
 * Usually not needed, as ~/.twinrc parser overrides this.
 */

#define HOT_KEY TW_Pause

