
#define T_MAP_ISO_8859_10	LATIN6
#define T_MAP_ISO_IR_157	LATIN6
#define T_MAP_L6		LATIN6
#define T_MAP_LATIN6		LATIN6

#define T_MAP_CP1252	CP1252
#define T_MAP_1252	CP1252
#define T_MAP_MS_ANSI	CP1252
#define T_MAP_WINDOWS_1252	CP1252

#define T_MAP_CP1253	CP1253
#define T_MAP_1253	CP1253
#define T_MAP_MS_GREEK	CP1253
#define T_MAP_WINDOWS_1253	CP1253

#define T_MAP_CP1254	CP1254
#define T_MAP_1254	CP1254
#define T_MAP_MS_TURK	CP1254
#define T_MAP_WINDOWS_1254	CP1254

#define T_MAP_CP1256	CP1256
#define T_MAP_1256	CP1256
#define T_MAP_MS_ARAB	CP1256
#define T_MAP_WINDOWS_1256	CP1256

#define T_MAP_CP1257	CP1257
#define T_MAP_1257	CP1257
#define T_MAP_WINBALTRIM	CP1257
#define T_MAP_WINDOWS_1257	CP1257

#define T_MAP_IBM851	IBM851
#define T_MAP_851	IBM851
#define T_MAP_CP851	IBM851

