



/* This file was automatically generated from m4/create_defs.m4, do not edit! */

/*
 *
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 */

/*
 *  create_defs_m4.h  --  #defines for libTT internal Create* function pointers
 *
 */

/*  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */





























                                                
#define Create1_ttvector		TTD.CREATE.Create1_ttvector
#define CreateA_ttvector		TTD.CREATE.CreateA_ttvector
#define CreateL_ttvector		TTD.CREATE.CreateL_ttvector
#define CreateR_ttvector		TTD.CREATE.CreateR_ttvector
#define CreateV_ttvector		TTD.CREATE.CreateV_ttvector
#define CreateY_ttvector		TTD.CREATE.CreateY_ttvector               
#define Create_ttevent		TTD.CREATE.Create_ttevent 
#define Create8_tteventbig		TTD.CREATE.Create8_tteventbig
#define Create4_tteventbig		TTD.CREATE.Create4_tteventbig
#define Create4s4_tteventbig		TTD.CREATE.Create4s4_tteventbig 
#define Create_ttbitmask		TTD.CREATE.Create_ttbitmask
#define CreateB2_ttbitmask		TTD.CREATE.CreateB2_ttbitmask
#define CreateR_ttbitmask		TTD.CREATE.CreateR_ttbitmask
#define CreateR2_ttbitmask		TTD.CREATE.CreateR2_ttbitmask 
#define CreateB_tteventmask		TTD.CREATE.CreateB_tteventmask
#define CreateB2_tteventmask		TTD.CREATE.CreateB2_tteventmask     
#define Create_ttcallback		TClass_ttcallback->Create 
#define Create_ttlistener		TTD.CREATE.Create_ttlistener
#define CreateE_ttlistener		TTD.CREATE.CreateE_ttlistener
#define CreateR_ttlistener		TTD.CREATE.CreateR_ttlistener
#define CreateDel_ttlistener		TTD.CREATE.CreateDel_ttlistener
#define CreateAskclose_ttlistener		TTD.CREATE.CreateAskclose_ttlistener
#define CreateKey_ttlistener		TTD.CREATE.CreateKey_ttlistener
#define CreateMouse_ttlistener		TTD.CREATE.CreateMouse_ttlistener
#define CreateActivate_ttlistener		TTD.CREATE.CreateActivate_ttlistener
#define CreateChange_ttlistener		TTD.CREATE.CreateChange_ttlistener
#define CreateKeyData_ttlistener		TTD.CREATE.CreateKeyData_ttlistener
#define CreateExpose_ttlistener		TTD.CREATE.CreateExpose_ttlistener         
#define CreateR_tttimer		TTD.CREATE.CreateR_tttimer
#define Create_tttimer		TTD.CREATE.Create_tttimer                                                   
          
#define Create_ttboxlayout		TTD.CREATE.Create_ttboxlayout              
#define Create_ttnative		TTD.CREATE.Create_ttnative                                      
#define Create_ttlabel		TTD.CREATE.Create_ttlabel                           
#define Create_ttanybutton		TTD.CREATE.Create_ttanybutton     
#define Create_ttbutton		TTD.CREATE.Create_ttbutton     
#define Create_ttcheckbutton		TTD.CREATE.Create_ttcheckbutton 
#define Create_ttradiobutton		TTD.CREATE.Create_ttradiobutton                              
#define Create1_ttscrollbar		TTD.CREATE.Create1_ttscrollbar
#define Create4_ttscrollbar		TTD.CREATE.Create4_ttscrollbar             
#define Create_ttslider		TTD.CREATE.Create_ttslider 
#define Create_ttprogressbar		TTD.CREATE.Create_ttprogressbar 
#define Create_ttscrollpane		TTD.CREATE.Create_ttscrollpane                  
#define Create_ttframe		TTD.CREATE.Create_ttframe 
#define Create_ttscroller		TTD.CREATE.Create_ttscroller          
#define Create_ttapplication		TTD.CREATE.Create_ttapplication





