







/* This file was automatically generated with m4 from m4/Twalias.m4, do not edit! */

/*
 *  alias1_m4.h  --  handy aliases for libTw function prototypes
 *
 *  Copyright (C) 2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */







 
     
    
#define TwDeleteRow			TwDeleteObj                
#define TwCirculateRows			TwCirculateChildrenRow        
#define TwRestackRows			TwRestackChildrenRow    
#define TwRow4Menu			TwCreate4MenuRow 
    
#define TwDeleteMenuItem			TwDeleteObj    
#define TwLowerMenuItem			TwLowerRow    
#define TwRaiseMenuItem			TwRaiseRow    
#define TwCirculateChildrenMenuItem			TwCirculateChildrenRow    
#define TwCirculateMenuItems			TwCirculateChildrenMenuItem    
#define TwRestackChildrenMenuItem			TwRestackChildrenRow    
#define TwRestackMenuItems			TwRestackChildrenMenuItem    
#define TwMenuItem4Menu			TwCreate4MenuMenuItem    
#define TwItem4Menu			TwCreate4MenuMenuItem    
#define TwItem4MenuCommon			TwCreate4MenuCommonMenuItem 
    
#define TwDeleteMenu			TwDeleteObj    
#define TwInfo4Menu			TwSetInfoMenu 
    
#define TwDeleteMsgPort			TwDeleteObj 
    
#define TwDeleteWidget			TwDeleteObj        
#define TwCirculateWidgets			TwCirculateChildrenWidget                                                            
#define TwRestackWidgets			TwRestackChildrenWidget             
    
#define TwDeleteGadget			TwDeleteObj    
#define TwCirculateChildrenGadget			TwCirculateChildrenWidget    
#define TwCirculateGadgets			TwCirculateChildrenGadget    
#define TwDrawGadget			TwDrawWidget    
#define TwDraw2Gadget			TwDraw2Widget    
#define TwDrawTextGadget			TwDrawTextWidget    
#define TwDrawHWFontGadget			TwDrawHWFontWidget    
#define TwDrawHWAttrGadget			TwDrawHWAttrWidget    
#define TwOwnerGadget			TwOwnerWidget    
#define TwO_PrevGadget			TwO_PrevWidget    
#define TwO_NextGadget			TwO_NextWidget    
#define TwLowerGadget			TwLowerWidget    
#define TwRaiseGadget			TwRaiseWidget    
#define TwMapGadget			TwMapWidget    
#define TwRecursiveDeleteGadget			TwRecursiveDeleteWidget    
#define TwResizeGadget			TwResizeWidget    
#define TwRestackChildrenGadget			TwRestackChildrenWidget    
#define TwRestackGadgets			TwRestackChildrenGadget    
#define TwScrollGadget			TwScrollWidget    
#define TwSetXYGadget			TwSetXYWidget    
#define TwUnMapGadget			TwUnMapWidget             
    
#define TwDeleteWindow			TwDeleteObj    
#define TwCirculateChildrenWindow			TwCirculateChildrenWidget    
#define TwCirculateWindows			TwCirculateChildrenWindow    
#define TwDrawWindow			TwDrawWidget    
#define TwDraw2Window			TwDraw2Widget    
#define TwDrawTextWindow			TwDrawTextWidget    
#define TwDrawHWFontWindow			TwDrawHWFontWidget    
#define TwDrawHWAttrWindow			TwDrawHWAttrWidget    
#define TwOwnerWindow			TwOwnerWidget    
#define TwO_PrevWindow			TwO_PrevWidget    
#define TwO_NextWindow			TwO_NextWidget    
#define TwLowerWindow			TwLowerWidget    
#define TwRaiseWindow			TwRaiseWidget    
#define TwMapWindow			TwMapWidget    
#define TwRecursiveDeleteWindow			TwRecursiveDeleteWidget    
#define TwResizeWindow			TwResizeWidget    
#define TwRestackChildrenWindow			TwRestackChildrenWidget    
#define TwRestackWindows			TwRestackChildrenWindow    
#define TwScrollWindow			TwScrollWidget    
#define TwSetXYWindow			TwSetXYWidget    
#define TwUnMapWindow			TwUnMapWidget    
#define TwWin4Menu			TwCreate4MenuWindow 
    
#define TwDeleteScreen			TwDeleteObj    
#define TwCirculateChildrenScreen			TwCirculateChildrenWidget    
#define TwCirculateScreens			TwCirculateChildrenScreen    
#define TwDrawScreen			TwDrawWidget    
#define TwDraw2Screen			TwDraw2Widget    
#define TwDrawTextScreen			TwDrawTextWidget    
#define TwDrawHWFontScreen			TwDrawHWFontWidget    
#define TwDrawHWAttrScreen			TwDrawHWAttrWidget    
#define TwOwnerScreen			TwOwnerWidget    
#define TwO_PrevScreen			TwO_PrevWidget    
#define TwO_NextScreen			TwO_NextWidget    
#define TwLowerScreen			TwLowerWidget    
#define TwRaiseScreen			TwRaiseWidget    
#define TwMapScreen			TwMapWidget    
#define TwRecursiveDeleteScreen			TwRecursiveDeleteWidget    
#define TwResizeScreen			TwResizeWidget    
#define TwRestackChildrenScreen			TwRestackChildrenWidget    
#define TwRestackScreens			TwRestackChildrenScreen    
#define TwScrollScreen			TwScrollWidget    
#define TwSetXYScreen			TwSetXYWidget    
#define TwUnMapScreen			TwUnMapWidget

