




/* This file was automatically generated from m4/tt.m4hh, do not edit! */


/*
 *  ttborderlayout.h  --  public header for libTT declarations related to "ttborderlayout"
 *
 *  Copyright (C) 2001-2002 by Massimiliano Ghilardi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */



#ifndef _TT_TTBORDERLAYOUT_H
#define _TT_TTBORDERLAYOUT_H







/* object structure */
#ifndef s_ttborderlayout


/** ttlayout: a border (north+south+west+east+center) layout */
        struct s_ttborderlayout;
typedef struct s_ttborderlayout *	ttborderlayout;

#endif /* s_ttborderlayout */



/* global runtime class type; returned by TTCLASSOF(<some object>) */
/** class of ttborderlayout */
extern ttclass	TTClass_ttborderlayout;





/* ttborderlayout methods */









/* TTGet*_ttborderlayout and TTSet*_ttborderlayout methods */










#endif /* _TT_TTBORDERLAYOUT_H */

