
/*
 * This file was automatically generated
 * from clisp sources with "from-kernel.bash"
 * Do not edit!
 */


#ifndef _TUTF_JISX0201_H
#define _TUTF_JISX0201_H

#define T_JISX0201_NULL	0x0000
#define T_JISX0201_START_OF_HEADING	0x0001
#define T_JISX0201_START_OF_TEXT	0x0002
#define T_JISX0201_END_OF_TEXT	0x0003
#define T_JISX0201_END_OF_TRANSMISSION	0x0004
#define T_JISX0201_ENQUIRY	0x0005
#define T_JISX0201_ACKNOWLEDGE	0x0006
#define T_JISX0201_BELL	0x0007
#define T_JISX0201_BACKSPACE	0x0008
#define T_JISX0201_HORIZONTAL_TABULATION	0x0009
#define T_JISX0201_LINE_FEED	0x000A
#define T_JISX0201_VERTICAL_TABULATION	0x000B
#define T_JISX0201_FORM_FEED	0x000C
#define T_JISX0201_CARRIAGE_RETURN	0x000D
#define T_JISX0201_SHIFT_OUT	0x000E
#define T_JISX0201_SHIFT_IN	0x000F
#define T_JISX0201_DATA_LINK_ESCAPE	0x0010
#define T_JISX0201_DEVICE_CONTROL_ONE	0x0011
#define T_JISX0201_DEVICE_CONTROL_TWO	0x0012
#define T_JISX0201_DEVICE_CONTROL_THREE	0x0013
#define T_JISX0201_DEVICE_CONTROL_FOUR	0x0014
#define T_JISX0201_NEGATIVE_ACKNOWLEDGE	0x0015
#define T_JISX0201_SYNCHRONOUS_IDLE	0x0016
#define T_JISX0201_END_OF_TRANSMISSION_BLOCK	0x0017
#define T_JISX0201_CANCEL	0x0018
#define T_JISX0201_END_OF_MEDIUM	0x0019
#define T_JISX0201_SUBSTITUTE	0x001A
#define T_JISX0201_ESCAPE	0x001B
#define T_JISX0201_FILE_SEPARATOR	0x001C
#define T_JISX0201_GROUP_SEPARATOR	0x001D
#define T_JISX0201_RECORD_SEPARATOR	0x001E
#define T_JISX0201_UNIT_SEPARATOR	0x001F
#define T_JISX0201_SPACE	0x0020
#define T_JISX0201_EXCLAMATION_MARK	0x0021
#define T_JISX0201_QUOTATION_MARK	0x0022
#define T_JISX0201_NUMBER_SIGN	0x0023
#define T_JISX0201_DOLLAR_SIGN	0x0024
#define T_JISX0201_PERCENT_SIGN	0x0025
#define T_JISX0201_AMPERSAND	0x0026
#define T_JISX0201_APOSTROPHE	0x0027
#define T_JISX0201_LEFT_PARENTHESIS	0x0028
#define T_JISX0201_RIGHT_PARENTHESIS	0x0029
#define T_JISX0201_ASTERISK	0x002A
#define T_JISX0201_PLUS_SIGN	0x002B
#define T_JISX0201_COMMA	0x002C
#define T_JISX0201_HYPHEN_MINUS	0x002D
#define T_JISX0201_FULL_STOP	0x002E
#define T_JISX0201_SOLIDUS	0x002F
#define T_JISX0201_DIGIT_ZERO	0x0030
#define T_JISX0201_DIGIT_ONE	0x0031
#define T_JISX0201_DIGIT_TWO	0x0032
#define T_JISX0201_DIGIT_THREE	0x0033
#define T_JISX0201_DIGIT_FOUR	0x0034
#define T_JISX0201_DIGIT_FIVE	0x0035
#define T_JISX0201_DIGIT_SIX	0x0036
#define T_JISX0201_DIGIT_SEVEN	0x0037
#define T_JISX0201_DIGIT_EIGHT	0x0038
#define T_JISX0201_DIGIT_NINE	0x0039
#define T_JISX0201_COLON	0x003A
#define T_JISX0201_SEMICOLON	0x003B
#define T_JISX0201_LESS_THAN_SIGN	0x003C
#define T_JISX0201_EQUALS_SIGN	0x003D
#define T_JISX0201_GREATER_THAN_SIGN	0x003E
#define T_JISX0201_QUESTION_MARK	0x003F
#define T_JISX0201_COMMERCIAL_AT	0x0040
#define T_JISX0201_LATIN_CAPITAL_LETTER_A	0x0041
#define T_JISX0201_LATIN_CAPITAL_LETTER_B	0x0042
#define T_JISX0201_LATIN_CAPITAL_LETTER_C	0x0043
#define T_JISX0201_LATIN_CAPITAL_LETTER_D	0x0044
#define T_JISX0201_LATIN_CAPITAL_LETTER_E	0x0045
#define T_JISX0201_LATIN_CAPITAL_LETTER_F	0x0046
#define T_JISX0201_LATIN_CAPITAL_LETTER_G	0x0047
#define T_JISX0201_LATIN_CAPITAL_LETTER_H	0x0048
#define T_JISX0201_LATIN_CAPITAL_LETTER_I	0x0049
#define T_JISX0201_LATIN_CAPITAL_LETTER_J	0x004A
#define T_JISX0201_LATIN_CAPITAL_LETTER_K	0x004B
#define T_JISX0201_LATIN_CAPITAL_LETTER_L	0x004C
#define T_JISX0201_LATIN_CAPITAL_LETTER_M	0x004D
#define T_JISX0201_LATIN_CAPITAL_LETTER_N	0x004E
#define T_JISX0201_LATIN_CAPITAL_LETTER_O	0x004F
#define T_JISX0201_LATIN_CAPITAL_LETTER_P	0x0050
#define T_JISX0201_LATIN_CAPITAL_LETTER_Q	0x0051
#define T_JISX0201_LATIN_CAPITAL_LETTER_R	0x0052
#define T_JISX0201_LATIN_CAPITAL_LETTER_S	0x0053
#define T_JISX0201_LATIN_CAPITAL_LETTER_T	0x0054
#define T_JISX0201_LATIN_CAPITAL_LETTER_U	0x0055
#define T_JISX0201_LATIN_CAPITAL_LETTER_V	0x0056
#define T_JISX0201_LATIN_CAPITAL_LETTER_W	0x0057
#define T_JISX0201_LATIN_CAPITAL_LETTER_X	0x0058
#define T_JISX0201_LATIN_CAPITAL_LETTER_Y	0x0059
#define T_JISX0201_LATIN_CAPITAL_LETTER_Z	0x005A
#define T_JISX0201_LEFT_SQUARE_BRACKET	0x005B
#define T_JISX0201_YEN_SIGN	0x005C
#define T_JISX0201_RIGHT_SQUARE_BRACKET	0x005D
#define T_JISX0201_CIRCUMFLEX_ACCENT	0x005E
#define T_JISX0201_LOW_LINE	0x005F
#define T_JISX0201_GRAVE_ACCENT	0x0060
#define T_JISX0201_LATIN_SMALL_LETTER_A	0x0061
#define T_JISX0201_LATIN_SMALL_LETTER_B	0x0062
#define T_JISX0201_LATIN_SMALL_LETTER_C	0x0063
#define T_JISX0201_LATIN_SMALL_LETTER_D	0x0064
#define T_JISX0201_LATIN_SMALL_LETTER_E	0x0065
#define T_JISX0201_LATIN_SMALL_LETTER_F	0x0066
#define T_JISX0201_LATIN_SMALL_LETTER_G	0x0067
#define T_JISX0201_LATIN_SMALL_LETTER_H	0x0068
#define T_JISX0201_LATIN_SMALL_LETTER_I	0x0069
#define T_JISX0201_LATIN_SMALL_LETTER_J	0x006A
#define T_JISX0201_LATIN_SMALL_LETTER_K	0x006B
#define T_JISX0201_LATIN_SMALL_LETTER_L	0x006C
#define T_JISX0201_LATIN_SMALL_LETTER_M	0x006D
#define T_JISX0201_LATIN_SMALL_LETTER_N	0x006E
#define T_JISX0201_LATIN_SMALL_LETTER_O	0x006F
#define T_JISX0201_LATIN_SMALL_LETTER_P	0x0070
#define T_JISX0201_LATIN_SMALL_LETTER_Q	0x0071
#define T_JISX0201_LATIN_SMALL_LETTER_R	0x0072
#define T_JISX0201_LATIN_SMALL_LETTER_S	0x0073
#define T_JISX0201_LATIN_SMALL_LETTER_T	0x0074
#define T_JISX0201_LATIN_SMALL_LETTER_U	0x0075
#define T_JISX0201_LATIN_SMALL_LETTER_V	0x0076
#define T_JISX0201_LATIN_SMALL_LETTER_W	0x0077
#define T_JISX0201_LATIN_SMALL_LETTER_X	0x0078
#define T_JISX0201_LATIN_SMALL_LETTER_Y	0x0079
#define T_JISX0201_LATIN_SMALL_LETTER_Z	0x007A
#define T_JISX0201_LEFT_CURLY_BRACKET	0x007B
#define T_JISX0201_VERTICAL_LINE	0x007C
#define T_JISX0201_RIGHT_CURLY_BRACKET	0x007D
#define T_JISX0201_OVERLINE	0x007E
#define T_JISX0201_UNDEFINED_007F	0x007F
#define T_JISX0201_UNDEFINED_0080	0x0080
#define T_JISX0201_UNDEFINED_0081	0x0081
#define T_JISX0201_UNDEFINED_0082	0x0082
#define T_JISX0201_UNDEFINED_0083	0x0083
#define T_JISX0201_UNDEFINED_0084	0x0084
#define T_JISX0201_UNDEFINED_0085	0x0085
#define T_JISX0201_UNDEFINED_0086	0x0086
#define T_JISX0201_UNDEFINED_0087	0x0087
#define T_JISX0201_UNDEFINED_0088	0x0088
#define T_JISX0201_UNDEFINED_0089	0x0089
#define T_JISX0201_UNDEFINED_008A	0x008A
#define T_JISX0201_UNDEFINED_008B	0x008B
#define T_JISX0201_UNDEFINED_008C	0x008C
#define T_JISX0201_UNDEFINED_008D	0x008D
#define T_JISX0201_UNDEFINED_008E	0x008E
#define T_JISX0201_UNDEFINED_008F	0x008F
#define T_JISX0201_UNDEFINED_0090	0x0090
#define T_JISX0201_UNDEFINED_0091	0x0091
#define T_JISX0201_UNDEFINED_0092	0x0092
#define T_JISX0201_UNDEFINED_0093	0x0093
#define T_JISX0201_UNDEFINED_0094	0x0094
#define T_JISX0201_UNDEFINED_0095	0x0095
#define T_JISX0201_UNDEFINED_0096	0x0096
#define T_JISX0201_UNDEFINED_0097	0x0097
#define T_JISX0201_UNDEFINED_0098	0x0098
#define T_JISX0201_UNDEFINED_0099	0x0099
#define T_JISX0201_UNDEFINED_009A	0x009A
#define T_JISX0201_UNDEFINED_009B	0x009B
#define T_JISX0201_UNDEFINED_009C	0x009C
#define T_JISX0201_UNDEFINED_009D	0x009D
#define T_JISX0201_UNDEFINED_009E	0x009E
#define T_JISX0201_UNDEFINED_009F	0x009F
#define T_JISX0201_UNDEFINED_00A0	0x00A0
#define T_JISX0201_HALFWIDTH_IDEOGRAPHIC_FULL_STOP	0x00A1
#define T_JISX0201_HALFWIDTH_LEFT_CORNER_BRACKET	0x00A2
#define T_JISX0201_HALFWIDTH_RIGHT_CORNER_BRACKET	0x00A3
#define T_JISX0201_HALFWIDTH_IDEOGRAPHIC_COMMA	0x00A4
#define T_JISX0201_HALFWIDTH_KATAKANA_MIDDLE_DOT	0x00A5
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_WO	0x00A6
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_A	0x00A7
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_I	0x00A8
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_U	0x00A9
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_E	0x00AA
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_O	0x00AB
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_YA	0x00AC
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_YU	0x00AD
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_YO	0x00AE
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SMALL_TU	0x00AF
#define T_JISX0201_HALFWIDTH_KATAKANA_HIRAGANA_PROLONGED_SOUND_MARK	0x00B0
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_A	0x00B1
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_I	0x00B2
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_U	0x00B3
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_E	0x00B4
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_O	0x00B5
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_KA	0x00B6
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_KI	0x00B7
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_KU	0x00B8
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_KE	0x00B9
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_KO	0x00BA
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SA	0x00BB
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SI	0x00BC
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SU	0x00BD
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SE	0x00BE
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_SO	0x00BF
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_TA	0x00C0
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_TI	0x00C1
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_TU	0x00C2
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_TE	0x00C3
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_TO	0x00C4
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_NA	0x00C5
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_NI	0x00C6
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_NU	0x00C7
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_NE	0x00C8
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_NO	0x00C9
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_HA	0x00CA
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_HI	0x00CB
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_HU	0x00CC
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_HE	0x00CD
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_HO	0x00CE
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_MA	0x00CF
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_MI	0x00D0
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_MU	0x00D1
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_ME	0x00D2
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_MO	0x00D3
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_YA	0x00D4
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_YU	0x00D5
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_YO	0x00D6
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_RA	0x00D7
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_RI	0x00D8
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_RU	0x00D9
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_RE	0x00DA
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_RO	0x00DB
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_WA	0x00DC
#define T_JISX0201_HALFWIDTH_KATAKANA_LETTER_N	0x00DD
#define T_JISX0201_HALFWIDTH_KATAKANA_VOICED_SOUND_MARK	0x00DE
#define T_JISX0201_HALFWIDTH_KATAKANA_SEMI_VOICED_SOUND_MARK	0x00DF
#define T_JISX0201_UNDEFINED_00E0	0x00E0
#define T_JISX0201_UNDEFINED_00E1	0x00E1
#define T_JISX0201_UNDEFINED_00E2	0x00E2
#define T_JISX0201_UNDEFINED_00E3	0x00E3
#define T_JISX0201_UNDEFINED_00E4	0x00E4
#define T_JISX0201_UNDEFINED_00E5	0x00E5
#define T_JISX0201_UNDEFINED_00E6	0x00E6
#define T_JISX0201_UNDEFINED_00E7	0x00E7
#define T_JISX0201_UNDEFINED_00E8	0x00E8
#define T_JISX0201_UNDEFINED_00E9	0x00E9
#define T_JISX0201_UNDEFINED_00EA	0x00EA
#define T_JISX0201_UNDEFINED_00EB	0x00EB
#define T_JISX0201_UNDEFINED_00EC	0x00EC
#define T_JISX0201_UNDEFINED_00ED	0x00ED
#define T_JISX0201_UNDEFINED_00EE	0x00EE
#define T_JISX0201_UNDEFINED_00EF	0x00EF
#define T_JISX0201_UNDEFINED_00F0	0x00F0
#define T_JISX0201_UNDEFINED_00F1	0x00F1
#define T_JISX0201_UNDEFINED_00F2	0x00F2
#define T_JISX0201_UNDEFINED_00F3	0x00F3
#define T_JISX0201_UNDEFINED_00F4	0x00F4
#define T_JISX0201_UNDEFINED_00F5	0x00F5
#define T_JISX0201_UNDEFINED_00F6	0x00F6
#define T_JISX0201_UNDEFINED_00F7	0x00F7
#define T_JISX0201_UNDEFINED_00F8	0x00F8
#define T_JISX0201_UNDEFINED_00F9	0x00F9
#define T_JISX0201_UNDEFINED_00FA	0x00FA
#define T_JISX0201_UNDEFINED_00FB	0x00FB
#define T_JISX0201_UNDEFINED_00FC	0x00FC
#define T_JISX0201_UNDEFINED_00FD	0x00FD
#define T_JISX0201_UNDEFINED_00FE	0x00FE
#define T_JISX0201_UNDEFINED_00FF	0x00FF

/* list of all characters */
#define T_LIST_JISX0201(EL) \
    EL(_NULL) \
    EL(_START_OF_HEADING) \
    EL(_START_OF_TEXT) \
    EL(_END_OF_TEXT) \
    EL(_END_OF_TRANSMISSION) \
    EL(_ENQUIRY) \
    EL(_ACKNOWLEDGE) \
    EL(_BELL) \
    EL(_BACKSPACE) \
    EL(_HORIZONTAL_TABULATION) \
    EL(_LINE_FEED) \
    EL(_VERTICAL_TABULATION) \
    EL(_FORM_FEED) \
    EL(_CARRIAGE_RETURN) \
    EL(_SHIFT_OUT) \
    EL(_SHIFT_IN) \
    EL(_DATA_LINK_ESCAPE) \
    EL(_DEVICE_CONTROL_ONE) \
    EL(_DEVICE_CONTROL_TWO) \
    EL(_DEVICE_CONTROL_THREE) \
    EL(_DEVICE_CONTROL_FOUR) \
    EL(_NEGATIVE_ACKNOWLEDGE) \
    EL(_SYNCHRONOUS_IDLE) \
    EL(_END_OF_TRANSMISSION_BLOCK) \
    EL(_CANCEL) \
    EL(_END_OF_MEDIUM) \
    EL(_SUBSTITUTE) \
    EL(_ESCAPE) \
    EL(_FILE_SEPARATOR) \
    EL(_GROUP_SEPARATOR) \
    EL(_RECORD_SEPARATOR) \
    EL(_UNIT_SEPARATOR) \
    EL(_SPACE) \
    EL(_EXCLAMATION_MARK) \
    EL(_QUOTATION_MARK) \
    EL(_NUMBER_SIGN) \
    EL(_DOLLAR_SIGN) \
    EL(_PERCENT_SIGN) \
    EL(_AMPERSAND) \
    EL(_APOSTROPHE) \
    EL(_LEFT_PARENTHESIS) \
    EL(_RIGHT_PARENTHESIS) \
    EL(_ASTERISK) \
    EL(_PLUS_SIGN) \
    EL(_COMMA) \
    EL(_HYPHEN_MINUS) \
    EL(_FULL_STOP) \
    EL(_SOLIDUS) \
    EL(_DIGIT_ZERO) \
    EL(_DIGIT_ONE) \
    EL(_DIGIT_TWO) \
    EL(_DIGIT_THREE) \
    EL(_DIGIT_FOUR) \
    EL(_DIGIT_FIVE) \
    EL(_DIGIT_SIX) \
    EL(_DIGIT_SEVEN) \
    EL(_DIGIT_EIGHT) \
    EL(_DIGIT_NINE) \
    EL(_COLON) \
    EL(_SEMICOLON) \
    EL(_LESS_THAN_SIGN) \
    EL(_EQUALS_SIGN) \
    EL(_GREATER_THAN_SIGN) \
    EL(_QUESTION_MARK) \
    EL(_COMMERCIAL_AT) \
    EL(_LATIN_CAPITAL_LETTER_A) \
    EL(_LATIN_CAPITAL_LETTER_B) \
    EL(_LATIN_CAPITAL_LETTER_C) \
    EL(_LATIN_CAPITAL_LETTER_D) \
    EL(_LATIN_CAPITAL_LETTER_E) \
    EL(_LATIN_CAPITAL_LETTER_F) \
    EL(_LATIN_CAPITAL_LETTER_G) \
    EL(_LATIN_CAPITAL_LETTER_H) \
    EL(_LATIN_CAPITAL_LETTER_I) \
    EL(_LATIN_CAPITAL_LETTER_J) \
    EL(_LATIN_CAPITAL_LETTER_K) \
    EL(_LATIN_CAPITAL_LETTER_L) \
    EL(_LATIN_CAPITAL_LETTER_M) \
    EL(_LATIN_CAPITAL_LETTER_N) \
    EL(_LATIN_CAPITAL_LETTER_O) \
    EL(_LATIN_CAPITAL_LETTER_P) \
    EL(_LATIN_CAPITAL_LETTER_Q) \
    EL(_LATIN_CAPITAL_LETTER_R) \
    EL(_LATIN_CAPITAL_LETTER_S) \
    EL(_LATIN_CAPITAL_LETTER_T) \
    EL(_LATIN_CAPITAL_LETTER_U) \
    EL(_LATIN_CAPITAL_LETTER_V) \
    EL(_LATIN_CAPITAL_LETTER_W) \
    EL(_LATIN_CAPITAL_LETTER_X) \
    EL(_LATIN_CAPITAL_LETTER_Y) \
    EL(_LATIN_CAPITAL_LETTER_Z) \
    EL(_LEFT_SQUARE_BRACKET) \
    EL(_YEN_SIGN) \
    EL(_RIGHT_SQUARE_BRACKET) \
    EL(_CIRCUMFLEX_ACCENT) \
    EL(_LOW_LINE) \
    EL(_GRAVE_ACCENT) \
    EL(_LATIN_SMALL_LETTER_A) \
    EL(_LATIN_SMALL_LETTER_B) \
    EL(_LATIN_SMALL_LETTER_C) \
    EL(_LATIN_SMALL_LETTER_D) \
    EL(_LATIN_SMALL_LETTER_E) \
    EL(_LATIN_SMALL_LETTER_F) \
    EL(_LATIN_SMALL_LETTER_G) \
    EL(_LATIN_SMALL_LETTER_H) \
    EL(_LATIN_SMALL_LETTER_I) \
    EL(_LATIN_SMALL_LETTER_J) \
    EL(_LATIN_SMALL_LETTER_K) \
    EL(_LATIN_SMALL_LETTER_L) \
    EL(_LATIN_SMALL_LETTER_M) \
    EL(_LATIN_SMALL_LETTER_N) \
    EL(_LATIN_SMALL_LETTER_O) \
    EL(_LATIN_SMALL_LETTER_P) \
    EL(_LATIN_SMALL_LETTER_Q) \
    EL(_LATIN_SMALL_LETTER_R) \
    EL(_LATIN_SMALL_LETTER_S) \
    EL(_LATIN_SMALL_LETTER_T) \
    EL(_LATIN_SMALL_LETTER_U) \
    EL(_LATIN_SMALL_LETTER_V) \
    EL(_LATIN_SMALL_LETTER_W) \
    EL(_LATIN_SMALL_LETTER_X) \
    EL(_LATIN_SMALL_LETTER_Y) \
    EL(_LATIN_SMALL_LETTER_Z) \
    EL(_LEFT_CURLY_BRACKET) \
    EL(_VERTICAL_LINE) \
    EL(_RIGHT_CURLY_BRACKET) \
    EL(_OVERLINE) \
    EL(_UNDEFINED_007F) \
    EL(_UNDEFINED_0080) \
    EL(_UNDEFINED_0081) \
    EL(_UNDEFINED_0082) \
    EL(_UNDEFINED_0083) \
    EL(_UNDEFINED_0084) \
    EL(_UNDEFINED_0085) \
    EL(_UNDEFINED_0086) \
    EL(_UNDEFINED_0087) \
    EL(_UNDEFINED_0088) \
    EL(_UNDEFINED_0089) \
    EL(_UNDEFINED_008A) \
    EL(_UNDEFINED_008B) \
    EL(_UNDEFINED_008C) \
    EL(_UNDEFINED_008D) \
    EL(_UNDEFINED_008E) \
    EL(_UNDEFINED_008F) \
    EL(_UNDEFINED_0090) \
    EL(_UNDEFINED_0091) \
    EL(_UNDEFINED_0092) \
    EL(_UNDEFINED_0093) \
    EL(_UNDEFINED_0094) \
    EL(_UNDEFINED_0095) \
    EL(_UNDEFINED_0096) \
    EL(_UNDEFINED_0097) \
    EL(_UNDEFINED_0098) \
    EL(_UNDEFINED_0099) \
    EL(_UNDEFINED_009A) \
    EL(_UNDEFINED_009B) \
    EL(_UNDEFINED_009C) \
    EL(_UNDEFINED_009D) \
    EL(_UNDEFINED_009E) \
    EL(_UNDEFINED_009F) \
    EL(_UNDEFINED_00A0) \
    EL(_HALFWIDTH_IDEOGRAPHIC_FULL_STOP) \
    EL(_HALFWIDTH_LEFT_CORNER_BRACKET) \
    EL(_HALFWIDTH_RIGHT_CORNER_BRACKET) \
    EL(_HALFWIDTH_IDEOGRAPHIC_COMMA) \
    EL(_HALFWIDTH_KATAKANA_MIDDLE_DOT) \
    EL(_HALFWIDTH_KATAKANA_LETTER_WO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_A) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_I) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_U) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_E) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_O) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_TU) \
    EL(_HALFWIDTH_KATAKANA_HIRAGANA_PROLONGED_SOUND_MARK) \
    EL(_HALFWIDTH_KATAKANA_LETTER_A) \
    EL(_HALFWIDTH_KATAKANA_LETTER_I) \
    EL(_HALFWIDTH_KATAKANA_LETTER_U) \
    EL(_HALFWIDTH_KATAKANA_LETTER_E) \
    EL(_HALFWIDTH_KATAKANA_LETTER_O) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_ME) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_WA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_N) \
    EL(_HALFWIDTH_KATAKANA_VOICED_SOUND_MARK) \
    EL(_HALFWIDTH_KATAKANA_SEMI_VOICED_SOUND_MARK) \
    EL(_UNDEFINED_00E0) \
    EL(_UNDEFINED_00E1) \
    EL(_UNDEFINED_00E2) \
    EL(_UNDEFINED_00E3) \
    EL(_UNDEFINED_00E4) \
    EL(_UNDEFINED_00E5) \
    EL(_UNDEFINED_00E6) \
    EL(_UNDEFINED_00E7) \
    EL(_UNDEFINED_00E8) \
    EL(_UNDEFINED_00E9) \
    EL(_UNDEFINED_00EA) \
    EL(_UNDEFINED_00EB) \
    EL(_UNDEFINED_00EC) \
    EL(_UNDEFINED_00ED) \
    EL(_UNDEFINED_00EE) \
    EL(_UNDEFINED_00EF) \
    EL(_UNDEFINED_00F0) \
    EL(_UNDEFINED_00F1) \
    EL(_UNDEFINED_00F2) \
    EL(_UNDEFINED_00F3) \
    EL(_UNDEFINED_00F4) \
    EL(_UNDEFINED_00F5) \
    EL(_UNDEFINED_00F6) \
    EL(_UNDEFINED_00F7) \
    EL(_UNDEFINED_00F8) \
    EL(_UNDEFINED_00F9) \
    EL(_UNDEFINED_00FA) \
    EL(_UNDEFINED_00FB) \
    EL(_UNDEFINED_00FC) \
    EL(_UNDEFINED_00FD) \
    EL(_UNDEFINED_00FE) \
    EL(_UNDEFINED_00FF)

/* list of all characters NOT IDENTICALLY MAPPED to UTF-16 */
#define T_NLIST_JISX0201(EL) \
    EL(_YEN_SIGN) \
    EL(_OVERLINE) \
    EL(_UNDEFINED_00A0) \
    EL(_HALFWIDTH_IDEOGRAPHIC_FULL_STOP) \
    EL(_HALFWIDTH_LEFT_CORNER_BRACKET) \
    EL(_HALFWIDTH_RIGHT_CORNER_BRACKET) \
    EL(_HALFWIDTH_IDEOGRAPHIC_COMMA) \
    EL(_HALFWIDTH_KATAKANA_MIDDLE_DOT) \
    EL(_HALFWIDTH_KATAKANA_LETTER_WO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_A) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_I) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_U) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_E) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_O) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_YO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SMALL_TU) \
    EL(_HALFWIDTH_KATAKANA_HIRAGANA_PROLONGED_SOUND_MARK) \
    EL(_HALFWIDTH_KATAKANA_LETTER_A) \
    EL(_HALFWIDTH_KATAKANA_LETTER_I) \
    EL(_HALFWIDTH_KATAKANA_LETTER_U) \
    EL(_HALFWIDTH_KATAKANA_LETTER_E) \
    EL(_HALFWIDTH_KATAKANA_LETTER_O) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_KO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_SO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_TO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_NO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_HO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_ME) \
    EL(_HALFWIDTH_KATAKANA_LETTER_MO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_YO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RI) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RU) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RE) \
    EL(_HALFWIDTH_KATAKANA_LETTER_RO) \
    EL(_HALFWIDTH_KATAKANA_LETTER_WA) \
    EL(_HALFWIDTH_KATAKANA_LETTER_N) \
    EL(_HALFWIDTH_KATAKANA_VOICED_SOUND_MARK) \
    EL(_HALFWIDTH_KATAKANA_SEMI_VOICED_SOUND_MARK) \
    EL(_UNDEFINED_00E0) \
    EL(_UNDEFINED_00E1) \
    EL(_UNDEFINED_00E2) \
    EL(_UNDEFINED_00E3) \
    EL(_UNDEFINED_00E4) \
    EL(_UNDEFINED_00E5) \
    EL(_UNDEFINED_00E6) \
    EL(_UNDEFINED_00E7) \
    EL(_UNDEFINED_00E8) \
    EL(_UNDEFINED_00E9) \
    EL(_UNDEFINED_00EA) \
    EL(_UNDEFINED_00EB) \
    EL(_UNDEFINED_00EC) \
    EL(_UNDEFINED_00ED) \
    EL(_UNDEFINED_00EE) \
    EL(_UNDEFINED_00EF) \
    EL(_UNDEFINED_00F0) \
    EL(_UNDEFINED_00F1) \
    EL(_UNDEFINED_00F2) \
    EL(_UNDEFINED_00F3) \
    EL(_UNDEFINED_00F4) \
    EL(_UNDEFINED_00F5) \
    EL(_UNDEFINED_00F6) \
    EL(_UNDEFINED_00F7) \
    EL(_UNDEFINED_00F8) \
    EL(_UNDEFINED_00F9) \
    EL(_UNDEFINED_00FA) \
    EL(_UNDEFINED_00FB) \
    EL(_UNDEFINED_00FC) \
    EL(_UNDEFINED_00FD) \
    EL(_UNDEFINED_00FE) \
    EL(_UNDEFINED_00FF)

#endif /* _TUTF_JISX0201_H */
